-- neil.e
--> A Graphics Library code-named Neil
--> Pete Eberlein <xseal@harborside.com>
--> July 4, 1998 - Apr. 6, 1999
-->
--> cannot work with safe.e nor graphics.e nor image.e
-->

without warning

-----------------------------------------------------------------
--LOW-LEVEL STUFF------------------------------------------------
-----------------------------------------------------------------

include machine.e

function allocate_proc(sequence data)
    atom handle
    handle = allocate(length(data))
    if handle then
        lock_memory(handle, length(data))
        poke(handle, data)
        return handle
    end if
    handle = machine_func(5, -1)  -- restore graphics mode
    printf(1, "Unable to allocate memory for length-%d sequence...\nProgram aborted.\n", {length(data)})
    abort(1)
end function

function allocate0(integer size)
    atom handle
    handle = allocate(size)
    if handle then
        mem_set(handle, 0, size)
        return handle
    end if
    handle = machine_func(5, -1)  -- restore graphics mode
    printf(1, "Unable to allocate memory for length-%d sequence...\nProgram aborted.\n", {size})
    abort(1)
end function

constant in_byte_result = allocate_proc({0})
constant in_byte_proc = allocate_proc(
   {#50,#52,#BA,#00,#00,#00,#00,#EC,#A2}&int_to_bytes(in_byte_result)&{#5A,#58,#C3})
constant out_byte_proc = allocate_proc(
   {#50,#52,#BA,#00,#00,#00,#00,#B0,#00,#EE,#5A,#58,#C3})
constant out_word_proc = allocate_proc(
   {#50,#52,#BA,#00,#00,#00,#00,#B8,#00,#00,#00,#00,#66,#EF,#5A,#58,#C3})

function in_byte(integer port)
    poke4(in_byte_proc + 3, port)
    call(in_byte_proc)
    return peek(in_byte_result)
end function

procedure out_byte(integer port, object data)
    poke4(out_byte_proc + 3, port)
    if atom(data) then
        data = {data}
    end if
    for i = 1 to length(data) do
        poke(out_byte_proc + 8, data[i])
        call(out_byte_proc)
    end for
end procedure

procedure out_word(integer port, object data)
    poke4(out_word_proc + 3, port)
    if atom(data) then
        data = {data}
    end if
    for i = 1 to length(data) do
        poke4(out_word_proc + 8, data[i])
        call(out_word_proc)
    end for
end procedure

-- returns machine codes for "ebx = ebx + edx * width"
function fast_asm_mul(integer width)
    integer count  sequence s
    count = 0      s = {}
    while width do
        if and_bits(width, 1) then
            if count then
                s = s & {#C1,#E2,count} --shl edx, count
            end if
            s = s & {#01,#D3}       --add ebx, edx
            count = 0
        end if
        count = count + 1
        width = floor(width / 2)
    end while
    return s
end function

-- machine codes to multiply ebx and ecx by the constant 1,2,3 or 4
constant mul_ebx = {   -- and ecx too
   {},
   {#D1,#E3,                -- shl ebx,1
    #D1,#E1},               -- shl ecx,1
   {#8D,#1C,#5B,            -- lea ebx,[ebx+ebx*2]
    #8D,#0C,#49},           -- lea ecx,[ecx+ecx*2]
   {#C1,#E3,#02,            -- shl ebx,2
    #C1,#E1,#02}}           -- shl ecx,2



-----------------------------------------------------------------
-- COLORS -------------------------------------------------------
-----------------------------------------------------------------

--> The following predefined color constants are for pixel and display_image:
-->  BLACK, BLUE, GREEN, CYAN, RED, MAGENTA, BROWN, WHITE, GRAY, BRIGHT_BLUE, 
-->  BRIGHT_GREEN, BRIGHT_CYAN, BRIGHT_RED, BRIGHT_MAGENTA, YELLOW, BRIGHT_WHITE

global constant  -- in #RRGGBB format
        BLACK = #000000,
--        BLUE  = #0000AA,
--        GREEN = #00AA00,
        CYAN =  #00AAAA,
--        RED   = #AA0000,
        MAGENTA =#AA00AA,
        BROWN = #AA5500,
        WHITE = #AAAAAA,
        GRAY  = #555555,
        BRIGHT_BLUE = #5555FF,
        BRIGHT_GREEN = #55FF55,
        BRIGHT_CYAN = #55FFFF,
        BRIGHT_RED = #FF5555,
        BRIGHT_MAGENTA = #FF55FF,
        YELLOW = #FFFF55,
        BRIGHT_WHITE = #FFFFFF

-----------------------------------------------------------------
-----------------------------------------------------------------
-- GRAPHICS MODES -----------------------------------------------
-----------------------------------------------------------------
-----------------------------------------------------------------

constant LINEAR = 1, MODE_X = 2, BANKED = 3          -- mode types

global integer mode_type, bits_per_pixel, bytes_per_line, bytes_per_pixel,
    win_granularity   -- the minimum display window shift in bytes
    mode_type = 0
global integer raw_mode  -- flag for pixel operations without conversions
    raw_mode = 0

global integer SCREEN_W, SCREEN_H
global atom blit, clear_blit, mixer_blit, screen_blit, rle_blit, wait_retrace
atom blit_proc, clear_blit_proc, mixer_blit_proc, bank_blit_proc, 
    modex_blit_proc, rle_blit_proc
global atom video_mem

global atom screen
screen = 0

-----------------------------------------------------------------
-- MODE_X (tweaked VGA mode 19) ---------------------------------
-----------------------------------------------------------------

constant sequ = #3C4,  -- sequencer register base
         crtc = #3D4,  -- CRT controller register base
         misc = #3C2,  -- miscellaneous output register
         ins1 = #3DA,  -- input status #1 register
         attr = #3C0   -- attribute register 

global procedure set_width(integer width)
--> sets the virtual width of the screen (mode19, modeX only)
    out_byte(crtc, #13)
    width = floor((width + 7) / 8)
    out_byte(crtc+1, width)
    poke(#44A, and_bits(width, 255) & floor(width / 256)) -- BIOS Video Columns
    bytes_per_line = width * 8
end procedure


constant modex_width_data =                          
  { {256,0,#5F00,#3F01,#4002,#8203,#4A04,#9A05,#2013},
    {320,0,#5F00,#4F01,#5002,#8203,#5404,#8005,#2813},
    {360,4,#6B00,#5901,#5A02,#8E03,#5E04,#8A05,#2D13},
    {376,4,#6E00,#5D01,#5E02,#9103,#6204,#8F05,#2F13},
    {400,4,#7100,#6301,#6402,#9203,#6504,#8205,#3213} }

constant modex_height_data =
  { {200,#40,#BF06,#1F07,#4109,#9C10,#8E11,#8F12,#9615,#B916},
    {224,#C0,#0B06,#3E07,#4109,#DA10,#9C11,#BF12,#C715,#0416},
    {240,#C0,#0D06,#3E07,#4109,#EA10,#AC11,#DF12,#E715,#0616},
    {256,#C0,#2306,#B207,#6109,#0A10,#AC11,#FF12,#0715,#1A16},
    {270,#C0,#3006,#F007,#6109,#2010,#A911,#1B12,#1F15,#2F16},
--    {282,#C0,#6206,#0F07,#6109,#3710,#8911,#3312,#3C15,#5C16},
    {300,#00,#4606,#1F07,#4009,#3110,#8011,#2B12,#2F15,#4416},
--    {308,#C0,#6206,#0F07,#4009,#3710,#8911,#3312,#3C15,#5C16},
    {360,#40,#BF06,#1F07,#4009,#8810,#8511,#6712,#6D15,#BA16},
    {400,#40,#BF06,#1F07,#4009,#9C10,#8E11,#8F12,#9615,#B916},
    {480,#C0,#0D06,#3E07,#4009,#EA10,#AC11,#DF12,#E715,#0616},
    {564,#C0,#6206,#F007,#6009,#3710,#8911,#3312,#3C15,#5C16},
    {600,#C0,#7006,#F007,#6009,#5B10,#8C11,#5712,#5815,#7016} }

global function mode_x(integer width, integer height)
--> attempts to switch to mode X
-->  width may be 256, 320, 360, 376, or 400
-->  height may be 200, 224, 240, 256, 270, 300, 360, 400, 480, 564, or 600
--> returns 0 on success, 1 otherwise
    integer w, h
    w = 0  h = 0

    for i = 1 to length(modex_width_data) do
        if modex_width_data[i][1] = width then
            w = i
        end if
    end for
    for i = 1 to length(modex_height_data) do
        if modex_height_data[i][1] = height then
            h = i
        end if
    end for

    if w = 0 or h = 0 then
        return 1        
    elsif peek(#449) != 19 then
        if machine_func(5, 19) then  -- attempt graphics mode 19
            return 1
        end if
    end if
        
    -- set planar mode
    out_word(sequ, #0604)

    -- clear the screen
    out_word(sequ, #0F02)
    video_mem = #A0000
    mem_set(video_mem, 0, #10000)

    -- more to set the planar mode
    out_word(sequ, #0100)
    out_byte(misc, #23 + modex_width_data[w][2] + modex_height_data[h][2])
    out_word(sequ, #0300)

    -- allow access to CRTC protected registers
    out_byte(crtc, #11)
    out_byte(crtc+1, and_bits(in_byte(crtc+1), #7F))

    -- set horizontal and vertical registers
    out_word(crtc, modex_width_data[w][3..9])
    out_word(crtc, modex_height_data[h][3..10])
    out_word(crtc, {#0008,#0014,#E317})

    -- set the overscan color to 0
    if in_byte(ins1) then end if
    out_byte(attr, {#31, 0})

    set_width(width)
    SCREEN_W = width
    SCREEN_H = height
    
    if screen then -- free the old screen handle
        free(screen-24)
    end if
    screen = 24+allocate_proc(
        int_to_bytes(width) &        -- 0: clipping width
        int_to_bytes(height) &       -- 4: clipping height
        int_to_bytes(0) & int_to_bytes(0) &  -- 8: panning position
        int_to_bytes(bytes_per_line/4) &  -- 16: true width
        int_to_bytes(0) &
        fast_asm_mul(bytes_per_line/4) &-- 20: address function
        {#81,#C3} & int_to_bytes(video_mem) & -- add ebx, video_mem
        #C3)                       -- ret
    poke4(screen-4, screen)

    screen_blit = modex_blit_proc
    bits_per_pixel = 8
    bytes_per_pixel = 1
    mode_type = MODE_X
    
    return 0  -- success
end function


-----------------------------------------------------------------
-- VESA (SuperVGA) modes ----------------------------------------
-----------------------------------------------------------------

constant bank_proc = allocate(17)
global constant bank_regs = allocate(#32)

integer
    VESAVerbose, -- prints modes as it tests
    VESAOverride -- use incompatible modes (insufficient vram, wrong monitor)
VESAVerbose = 0
VESAOverride = 0

integer bank_granularity, bank_start
atom win_func_ptr

-- standard modes, only used if there is no mode list in rom
constant VESAModes = {
    {640,400,0,0,0,8,0,0,0}, -- 256
    {640,480,0,0,0,8,0,0,0}, -- 257
    {800,600,0,0,0,4,0,0,0}, -- 258
    {800,600,0,0,0,8,0,0,0}, -- 259
    {1024,768,0,0,0,4,0,0,0}, -- 260
    {1024,768,0,0,0,8,0,0,0}, -- 261
    {1280,1024,0,0,0,4,0,0,0}, -- 262
    {1280,1024,0,0,0,8,0,0,0}, -- 263
    {}, -- 264
    {}, -- 265
    {}, -- 266
    {}, -- 267
    {}, -- 268
    {320,200,0,0,0,15,0,0,0}, -- 269
    {320,200,0,0,0,16,0,0,0}, -- 270
    {320,200,0,0,0,24,0,0,0}, -- 271
    {640,480,0,0,0,15,0,0,0}, -- 272
    {640,480,0,0,0,16,0,0,0}, -- 273
    {640,480,0,0,0,24,0,0,0}, -- 274
    {800,600,0,0,0,15,0,0,0}, -- 275
    {800,600,0,0,0,16,0,0,0}, -- 276
    {800,600,0,0,0,24,0,0,0}, -- 277
    {1024,768,0,0,0,15,0,0,0}, -- 278
    {1024,768,0,0,0,16,0,0,0}, -- 279
    {1024,768,0,0,0,24,0,0,0}, -- 280
    {1280,1024,0,0,0,15,0,0,0}, -- 281
    {1280,1024,0,0,0,16,0,0,0}, -- 282
    {1280,1024,0,0,0,24,0,0,0}  -- 283
    }

-- code to convert the physical base address to the linear frame buffer address
constant getlfb = allocate_proc({
    #60,                    --    0: pusha
    #8B,#1D,#D4,#6A,#CC,#82,--    1: mov ebx, [@linear]
    #85,#DB,                --    7: test ebx, ebx
    #74,#0C,                --    9: jz batman
    #89,#D9,                --    B: mov ecx, ebx
    #C1,#EB,#10,            --    D: shr ebx, 16
    #B8,#01,#08,#00,#00,    --   10: mov eax, #801
    #CD,#31,                --   15: int #31
                            --   17: batman: 
    #BB,#00,#00,#00,#00,    --   17: mov ebx, PhysBasePtr (24)
    #89,#D9,                --   1C: mov ecx, ebx
    #C1,#EB,#10,            --   1E: shr ebx, 16
    #BE,#00,#00,#40,#00,    --   21: mov esi, 4194304
    #89,#F7,                --   26: mov edi, esi
    #C1,#EE,#10,            --   28: shr esi, 16
    #B8,#00,#08,#00,#00,    --   2B: mov eax, #800
    #CD,#31,                --   30: int #31
    #C1,#E3,#10,            --   32: shl ebx, 16
    #66,#89,#CB,            --   35: mov bx, cx
    #89,#1D,#D4,#6A,#CC,#82,--   38: mov [@linear], ebx
    #61,                    --   3E: popa
    #C3,                    --   3F: ret
    #00,#00,#00,#00})       --   40: linear: dd 0 (64)
poke4(getlfb + 3, getlfb + 64) -- @linear
poke4(getlfb + 58, getlfb + 64) -- @linear


global function vesa_mode(integer width, integer height, integer bpp)
--> attempts to set a VESA mode
--> width,height pairs may be (640,480),(800,600),(1024,768),(1280,1024)
-->   (other modes may be supported by your video hardware)
--> bpp may be 8, 15, 16, 24 or 32
-->   (some video cards support only 15 or 16, and 24 or 32, not both)
--> returns 0 on success, 1 otherwise
--> See also: force_vesa, print_vesa
    atom data, OEMStringPtr, VideoModePtr, ModeAttributes
    atom PhysBasePtr, OffScreenMemOffset, OffScreenMemSize
    integer bank_order, bank_64k
    sequence regs
    object result, VESASignature, VESAVersion, OEMString, Capabilities, VideoModes
    regs = repeat(0,10)
    regs[REG_AX] = #4F00
    data = allocate_low(256)
    regs[REG_ES] = floor(data / 16)
    regs[REG_DI] = and_bits(data, 15)
    regs = dos_interrupt(#10, regs)
    if regs[REG_AX] != #4F then
        return {}
    end if
    
    VESASignature = peek({data,4})
    VESAVersion = 0.01*peek(data+4)+peek(data+5)
    OEMStringPtr = peek(data+6) + 256*peek(data+7) + 16*peek(data+8) + 4096*peek(data+9)
    Capabilities = peek4u(data+10)
    VideoModePtr = peek(data+14) + 256*peek(data+15) + 16*peek(data+16) + 4096*peek(data+17)
    free_low(data)
    OEMString = ""
    while peek(OEMStringPtr) do
        OEMString = append(OEMString, peek(OEMStringPtr))
        OEMStringPtr = OEMStringPtr + 1
    end while
    VideoModes = {}
    while compare(peek({VideoModePtr,2}),{#FF,#FF}) do
        VideoModes = append(VideoModes, peek(VideoModePtr)+256*peek(VideoModePtr+1))
        VideoModePtr = VideoModePtr + 2
    end while

    if VESAVerbose then
        printf(1, "%s version %g \"%s\"\n", {VESASignature, VESAVersion, OEMString})
        if VESAVerbose = 2 then
            puts(1, "mode   W    H CwCh P Bpp  M   LFB \n")
        end if
    end if

    data = allocate_low(256)
    result = {}
    for mode = 1 to length(VideoModes) do
        regs = repeat(0,10)
        regs[REG_AX] = #4F01
        regs[REG_CX] = VideoModes[mode]
        regs[REG_ES] = floor(data / 16)
        regs[REG_DI] = and_bits(data, 15)
        regs = dos_interrupt(#10, regs)
        if regs[REG_AX] != #4F then
            free_low(data)
            return 1
        end if
        ModeAttributes = peek(data) + 256*peek(data+1)
        if 1-and_bits(ModeAttributes, 1) > VESAOverride then
            result = {}  -- mode available but not supported, because
                         -- insufficient vram or incompatible monitor
        elsif and_bits(ModeAttributes,2) then
            result = (peek(data+18) + 256*peek(data+19)) &
                (peek(data+20) + 256*peek(data+21)) & peek({data+22,7})
        elsif VideoModes[mode] >= 256 and VideoModes[mode] - 255 <= length(VESAModes) then
            result = VESAModes[VideoModes[mode]-255]
        else
            result = {}  -- not enough information
        end if
        if length(result) then
                
            if VESAVerbose = 2 then
                printf(1, "%3d%5dx%4d%2dx%2d%2d%3d%2d%2d%2d", VideoModes[mode] & result)
                if and_bits(ModeAttributes, #80) then
                    printf(1, " #%x\n", peek4u(data+#28))
                else
                    puts(1, "\n")
                end if
            end if
            if width = result[1] and height = result[2] and bpp = result[6] then
                bank_granularity = (peek(data + 4) + 256 * peek(data + 5)) * 1024
                win_granularity = 65536 / bank_granularity
                bytes_per_line = peek(data + #10) + 256 * peek(data + #11)
                bits_per_pixel = bpp
                bytes_per_pixel = 1 + (bpp>8) + (bpp>=24) + (bpp>=32)
                win_func_ptr = peek4u(data+12)
                SCREEN_W = width
                SCREEN_H = height

                regs[REG_AX] = #4F02
                regs[REG_BX] = VideoModes[mode]

                if and_bits(ModeAttributes, #80) then
                    -- linear frame buffer
                    regs[REG_BX] = regs[REG_BX] + #4000
                    PhysBasePtr = peek4u(data+#28)
                    OffScreenMemOffset = peek4u(data+#2C)
                    OffScreenMemSize = peek(data+#30) + 256 * peek(data+#31)

                    mode_type = LINEAR

                else
                    -- banked
                    video_mem = #A0000
                    mode_type = BANKED
                end if
                
                regs = dos_interrupt(#10, regs)
                result = regs[REG_AX]  -- success

                exit
            end if
        end if
    end for
    free_low(data)
    if atom(result) then

        if mode_type = LINEAR then

            poke4(getlfb + 24, PhysBasePtr)  -- convert physical base address
            call(getlfb)                     -- to linear frame buffer
            video_mem = peek4u(getlfb + 64)

            screen_blit = blit
        else

            poke4(bank_regs + #2A, win_func_ptr) -- CS:IP
            poke4(bank_regs + #10, 0) -- BX
            poke4(bank_regs + #14, 1) -- DX
            poke(bank_proc,
               {#60,                    -- pusha
                #B8,#02,#FF,#00,#00,    -- mov eax, #FF02
                #1E,                    -- push ds
                #07,                    -- pop es
                #BF}&int_to_bytes(bank_regs)&{-- mov edi, bank_regs
                #CD,#31,                -- int #31
                #61,                    -- popa
                #C3})                   -- ret
            bank_start = 0

            bank_64k = #10000 / bank_granularity
            bank_order = 1
            while power(2, bank_order) < bank_granularity do
                bank_order = bank_order + 1            
            end while

            poke(bank_blit_proc + 249, bank_order)
            poke4(bank_blit_proc + 262, bank_regs)
            poke(bank_blit_proc + 271, bank_order)
            poke4(bank_blit_proc + 377, bank_64k)
            poke4(bank_blit_proc + 406, bank_regs)

            screen_blit = bank_blit_proc

        end if

        if screen then -- free the old screen handle
            free(screen-24)
        end if

        screen = 24 + allocate_proc(
            int_to_bytes(width) & -- 0: clipping width
            int_to_bytes(height) &       -- 4: clipping height
            int_to_bytes(0) & int_to_bytes(0) &  -- 8: panning position
            int_to_bytes(bytes_per_line) &  -- 16: true width
            int_to_bytes(0) & 
            mul_ebx[bytes_per_pixel] &    -- 20: address function
            fast_asm_mul(bytes_per_line) &
            {#81,#C3} & int_to_bytes(video_mem) & -- add ebx, video_mem
            #C3)                       -- ret
        poke4(screen-4, screen)

    end if
    VESAVerbose = 0
    return sequence(result)
end function


global procedure force_vesa()
--> Forces vesa_mode to attempt modes that are flagged as not supported.
    VESAOverride = 1
end procedure

global procedure print_vesa()
--> Displays the VESA id and a list of modes
    VESAVerbose = 2
    VESAVerbose = vesa_mode(0,0,0)
    VESAVerbose = 0
end procedure


------------------------------------------------------------------------
-- GFX_MODE ------------------------------------------------------------
------------------------------------------------------------------------

global function gfx_mode(integer width, integer height, integer bpp)
--> Attempts to set a graphics mode of the desired width, height and color
--> depth.  Valid color depths (bpp) are: 8, 15, 16, 24, 32.
--> See mode_x and vesa_mode for supported resolutions.
--> Note: some 8 bpp, width 256 modes are supported as linear (non-modeX)
--> See also: make_rgb_map, restore_mode
    if width = 320 and height = 200 and bpp = 8 then
        if machine_func(5, 19) = 0 then  -- attempt graphics mode 19

            bits_per_pixel = 8
            bytes_per_pixel = 1
            bytes_per_line = 320
            mode_type = LINEAR
            video_mem = #A0000
            SCREEN_W = 320
            SCREEN_H = 200
            if screen then  -- free the old screen handle
                free(screen-24)
            end if
    
            screen = 24 + allocate_proc(
                int_to_bytes(320) & -- 0: clipping width
                int_to_bytes(200) &       -- 4: clipping height
                int_to_bytes(0) & int_to_bytes(0) &  -- 8: panning position
                int_to_bytes(320) &  -- 16: true width
                int_to_bytes(0) & 
                fast_asm_mul(bytes_per_line) &-- 20: address function
                {#81,#C3} & int_to_bytes(video_mem) & -- add ebx, video_mem
                #C3)                       -- ret
            poke4(screen-4, screen)
    
            screen_blit = blit

            -- set the overscan color to 0
            if in_byte(ins1) then end if
            out_byte(attr, {#31, 0})

            return 0
        end if
    elsif width = 256 and find(height,{200,224,240,256}) and bpp = 8 then
        if machine_func(5, 19) = 0 then  -- attempt graphics mode 19

            bits_per_pixel = 8
            bytes_per_pixel = 1
            bytes_per_line = 256
            set_width(bytes_per_line)

            -- allow access to CRTC protected registers
            out_byte(crtc, #11)
            out_byte(crtc+1, and_bits(in_byte(crtc+1), #7F))

            out_byte(misc, #23 + modex_width_data[1][2] + modex_height_data[3][2])

            -- set horizontal and vertical registers
            out_word(crtc, modex_width_data[1][3..9])
            if height = 224 then
                out_word(crtc, modex_height_data[2][3..10])
            elsif height = 240 then
                out_word(crtc, modex_height_data[3][3..10])
            elsif height = 256 then
                out_word(crtc, modex_height_data[4][3..10])
            end if
--            out_word(crtc, {#0008,#0014,#E317})

            mode_type = LINEAR
            video_mem = #A0000
            SCREEN_W = 256
            SCREEN_H = height
            if screen then  -- free the old screen handle
                free(screen-24)
            end if
    
            screen = 24 + allocate_proc(
                int_to_bytes(256) & -- 0: clipping width
                int_to_bytes(height) &       -- 4: clipping height
                int_to_bytes(0) & int_to_bytes(0) &  -- 8: panning position
                int_to_bytes(256) &  -- 16: true width
                int_to_bytes(0) & 
                fast_asm_mul(bytes_per_line) &-- 20: address function
                {#81,#C3} & int_to_bytes(video_mem) & -- add ebx, video_mem
                #C3)                       -- ret
            poke4(screen-4, screen)
    
            screen_blit = blit

            -- set the overscan color to 0
            if in_byte(ins1) then end if
            out_byte(attr, {#31, 0})

            return 0
        end if

    end if
    if vesa_mode(width, height, bpp) then
        if bpp != 8 then
            return 1
        end if
        return mode_x(width, height)
    end if
    return 0 
end function

global procedure restore_mode()
--> Restores the graphics mode the one present when ex.exe started.
--> Same as graphics_mode(-1) except it's not a function :-)
    if machine_func(5,-1) then end if
    mode_type = 0
end procedure


------------------------------------------------------------------------
-- VIRTUAL SCREENS -----------------------------------------------------
------------------------------------------------------------------------

constant   -- offsets into the screen structure
    screen_header_size = 40,
    screen_sub_code = 17,
    screen_animated_frame = -36,
    screen_hot_x = -32,
    screen_hot_y = -28,
    screen_width = -24,
    screen_height = -20,
    screen_pan_x = -16,
    screen_pan_y = -12,
    screen_actual_width = -8,
    screen_addr_func = -4

global function virtual_screen(integer width, integer height)
--> creates a virtual screen.  The handle is actually a memory address, the
--> screen is linear - you may peek and poke to your hearts content.
--> returns the handle (atom)
--> Use free_screen() to dispose of the virtual screen handle.
    atom handle, addr_func
    sequence addr_code
                      
    addr_code = fast_asm_mul(width) & mul_ebx[bytes_per_pixel]
    addr_func = width * height * bytes_per_pixel
    handle = allocate0(screen_header_size + addr_func + length(addr_code) + 7) + screen_header_size

    poke4(handle + screen_width, {width, height,0,0, width * bytes_per_pixel, handle + addr_func})
    poke(handle + addr_func, addr_code &
        {#81,#C3} & int_to_bytes(handle) & -- add ebx, vs_addr
        #C3)                       -- ret

    return handle
end function

global function sub_screen(sequence source)
--> creates a sub-screen on the virtual screen "source" at the position "x1,y1",
--> with the dimensions "width" and "height"
--> returns (atom) the handle to the sub-screen
--> *** do not poke around on the memory address returned ***
--> *** do not use sub-screens with pixel or get_pixel ***
--> Use free_screen() to dispose of the sub virtual screen handle.
    atom handle, addr_func, bytes_per_line
    bytes_per_line = peek4u(screen_actual_width+source[1])
    addr_func = 0
    handle = allocate0(screen_header_size + screen_sub_code) + screen_header_size

    poke4(handle + screen_width, source[4..5]&{0,0, bytes_per_line, handle + addr_func})
    poke(handle + addr_func,
        #81&#C3&int_to_bytes(source[2])&--    0: add ebx, xoff (2)
        #81&#C2&int_to_bytes(source[3])&--    6: add edx, yoff (8)
        #E9&int_to_bytes(peek4u(screen_addr_func+source[1])-handle-screen_sub_code))    --    C: jmp near offset
    if length(source) = 7 then
        poke4(handle + screen_hot_x, source[6..7])
    end if
    return handle
end function

global procedure free_screen(atom handle)
--> frees the memory associated with a virtual screen
    free(handle - screen_header_size)
end procedure

global procedure pan_screen(atom handle, sequence pos)
--> changes the relative position of blits to the virtual screen
--> pan_screen(screen, {50,50}) -- all sprites on the screen are subsequently
-->                      -- drawn 50 pixels up and to the left.
--> pan_screen affects the way sprites are drawn on this screen
--> See also: set_hot_spot
    poke4(handle + screen_pan_x, pos)
end procedure

global procedure set_hot_spot(atom handle, sequence pos)
--> changes the relative position of blits of the virtual screen
--> set_hot_spot(screen, {50,50}) -- this sprite is subsequently always
-->                      -- drawn 50 pixels up and to the left.
--> set_hot_spot affects the way this sprite is drawn on another screen
--> See also: pan_screen
    poke4(handle + screen_hot_x, pos)
end procedure

global procedure tweak_screen(atom handle)
--> Tweaks a screen so that zero bytes occur only on zero colored pixels.
--> This should remedy weird effects when using clear_copy/mix_copy in 15,16,
--> 24,32 bit-per-pixel modes.
--> Note: will not work with a subscreen.
    sequence data
    if bits_per_pixel = 8 then
        return        
    end if
    data = peek({handle, 
        peek4u(handle + screen_actual_width) * 
        peek4u(handle + screen_height)})
    if bits_per_pixel = 15 then
        for i = 1 to length(data)-1 by 2 do
            if data[i] or data[i+1] then
                if not data[i]   then data[i]   = 1 end if
                if not data[i+1] then data[i+1] = 1 end if
            end if
        end for
    elsif bits_per_pixel = 16 then
        for i = 1 to length(data)-1 by 2 do
            if data[i] or data[i+1] then
                if not data[i]   then data[i]   = 1 end if
                if not data[i+1] then data[i+1] = 1 end if
            end if
        end for
    elsif bits_per_pixel = 24 then
        for i = 1 to length(data)-2 by 3 do
            if data[i] or data[i+1] or data[i+2] then
                if not data[i]   then data[i]   = 1 end if
                if not data[i+1] then data[i+1] = 1 end if
                if not data[i+2] then data[i+2] = 1 end if
            end if
        end for
    elsif bits_per_pixel = 32 then
        for i = 1 to length(data)-3 by 4 do
            if data[i] or data[i+1] or data[i+2] then
                if not data[i]   then data[i]   = 1 end if
                if not data[i+1] then data[i+1] = 1 end if
                if not data[i+2] then data[i+2] = 1 end if
                data[i+3] = 1
            else
                data[i+3] = 0
            end if
        end for
    end if
    poke(handle, data)    

end procedure

-----------------------------------------------------------------
--RGB TABLE MAPPING----------------------------------------------
-----------------------------------------------------------------

atom rgb_map
     rgb_map = 0

global procedure make_rgb_map()
--> Enables true-color conversions in an 8-bit mode and creates the necessary
--> 32x32x32 lookup table using a fast recursive floodfill method.
--> If the palette changes, i.e. a call to set_all_palette, you must call
--> make_rgb_map() again afterward.  Only valid in 8-bit color modes.
--> See also: gfx_mode, load_bitmap, set_all_palette
    atom proc, rgb_diff, squares
    if bits_per_pixel != 8 then
        return
    end if
    if rgb_map = 0 then
        rgb_map = allocate0(32768)
    end if
    rgb_diff = allocate0(4 * 32768)
    poke4(rgb_diff, repeat(400, 32768))
    squares = allocate0(4 * 64)
    for i = 0 to 63 do
        poke4(squares + i*4, i*i)
    end for
    proc = allocate_proc( {
        #60,                    --    0: pusha
        #31,#C0,                --    1: xor eax, eax
        #BA,#C7,#03,#00,#00,    --    3: mov edx, 967
        #EE,                    --    8: out dx, al
        #31,#C0,                --    9: loop1: xor eax, eax
        #BA,#C9,#03,#00,#00,    --    B: mov edx, 969
        #EC,                    --   10: in al, dx
        #D1,#E8,                --   11: shr eax, 1
        #89,#C7,                --   13: mov edi, eax
        #C1,#E7,#05,            --   15: shl edi, 5
        #EC,                    --   18: in al, dx
        #D1,#E8,                --   19: shr eax, 1
        #01,#C7,                --   1B: add edi, eax
        #C1,#E7,#05,            --   1D: shl edi, 5
        #EC,                    --   20: in al, dx
        #D1,#E8,                --   21: shr eax, 1
        #01,#C7,                --   23: add edi, eax
        #31,#D2,                --   25: xor edx, edx
        #31,#C9,                --   27: xor ecx, ecx
        #31,#DB,                --   29: xor ebx, ebx
        #31,#F6,                --   2B: xor esi, esi
        #E8,#0A,#00,#00,#00,    --   2D: call near start
        #FE,#05,#B2,#78,#CA,#82,--   32: inc byte ptr [@current_col]
        #75,#CF,                --   38: jnz loop1
        #61,                    --   3A: popa
        #C3,                    --   3B: ret
        #C6,#87,#00,#00,#00,#00,#00,--   3C: start: mov byte ptr [edi+rgb_map], current_col (62) (66)
        #2B,#B2,#00,#00,#00,#00,--   43: sub esi, dword ptr [squares + edx] (69)
        #83,#C2,#04,            --   49: add edx,4
        #03,#B2,#00,#00,#00,#00,--   4C: add esi, dword ptr [squares + edx] (78)
        #F7,#C7,#00,#7C,#00,#00,--   52: test edi,#7C00
        #74,#21,                --   58: jz red0
        #81,#EF,#00,#04,#00,#00,--   5A: sub edi,1024
        #39,#34,#BD,#00,#00,#00,#00,--   60: cmp [rgb_diff+edi*4], esi (99)
        #7E,#0C,                --   67: jle red0x
        #89,#34,#BD,#00,#00,#00,#00,--   69: mov [rgb_diff+edi*4], esi (108)
        #E8,#C7,#FF,#FF,#FF,    --   70: call near start
        #81,#C7,#00,#04,#00,#00,--   75: red0x: add edi,1024
        #89,#F8,                --   7B: red0: mov eax, edi
        #35,#00,#7C,#00,#00,    --   7D: xor eax, #7C00
        #A9,#00,#7C,#00,#00,    --   82: test eax,#7C00
        #74,#21,                --   87: jz red63
        #81,#C7,#00,#04,#00,#00,--   89: add edi,1024
        #39,#34,#BD,#00,#00,#00,#00,--   8F: cmp [rgb_diff+edi*4], esi (146)
        #7E,#0C,                --   96: jle red63x
        #89,#34,#BD,#00,#00,#00,#00,--   98: mov [rgb_diff+edi*4], esi (155)
        #E8,#98,#FF,#FF,#FF,    --   9F: call near start
        #81,#EF,#00,#04,#00,#00,--   A4: red63x: sub edi,1024
        #2B,#B2,#00,#00,#00,#00,--   AA: red63: sub esi, dword ptr [squares + edx] (172)
        #83,#EA,#04,            --   B0: sub edx,4
        #03,#B2,#00,#00,#00,#00,--   B3: add esi, dword ptr [squares + edx] (181)
        #2B,#B1,#00,#00,#00,#00,--   B9: sub esi, dword ptr [squares + ecx] (187)
        #83,#C1,#04,            --   BF: add ecx,4
        #03,#B1,#00,#00,#00,#00,--   C2: add esi, dword ptr [squares + ecx] (196)
        #F7,#C7,#E0,#03,#00,#00,--   C8: test edi,#03E0
        #74,#1B,                --   CE: jz gre0
        #83,#EF,#20,            --   D0: sub edi,32
        #39,#34,#BD,#00,#00,#00,#00,--   D3: cmp [rgb_diff+edi*4], esi (214)
        #7E,#0C,                --   DA: jle gre0x
        #89,#34,#BD,#00,#00,#00,#00,--   DC: mov [rgb_diff+edi*4], esi (223)
        #E8,#54,#FF,#FF,#FF,    --   E3: call near start
        #83,#C7,#20,            --   E8: gre0x: add edi,32
        #89,#F8,                --   EB: gre0: mov eax, edi
        #35,#E0,#03,#00,#00,    --   ED: xor eax, #03E0
        #A9,#E0,#03,#00,#00,    --   F2: test eax,#03E0
        #74,#1B,                --   F7: jz gre63
        #83,#C7,#20,            --   F9: add edi,32
        #39,#34,#BD,#00,#00,#00,#00,--   FC: cmp [rgb_diff+edi*4], esi (255)
        #7E,#0C,                --  103: jle gre63x
        #89,#34,#BD,#00,#00,#00,#00,--  105: mov [rgb_diff+edi*4], esi (264)
        #E8,#2B,#FF,#FF,#FF,    --  10C: call near start
        #83,#EF,#20,            --  111: gre63x: sub edi,32
        #2B,#B1,#00,#00,#00,#00,--  114: gre63: sub esi, dword ptr [squares + ecx] (278)
        #83,#E9,#04,            --  11A: sub ecx,4
        #03,#B1,#00,#00,#00,#00,--  11D: add esi, dword ptr [squares + ecx] (287)
        #2B,#B3,#00,#00,#00,#00,--  123: sub esi, dword ptr [squares + ebx] (293)
        #83,#C3,#04,            --  129: add ebx,4
        #03,#B3,#00,#00,#00,#00,--  12C: add esi, dword ptr [squares + ebx] (302)
        #F7,#C7,#1F,#00,#00,#00,--  132: test edi,#001F
        #74,#17,                --  138: jz blu0
        #4F,                    --  13A: dec edi
        #39,#34,#BD,#00,#00,#00,#00,--  13B: cmp [rgb_diff+edi*4], esi (318)
        #7E,#0C,                --  142: jle blu0x
        #89,#34,#BD,#00,#00,#00,#00,--  144: mov [rgb_diff+edi*4], esi (327)
        #E8,#EC,#FE,#FF,#FF,    --  14B: call near start
        #47,                    --  150: blu0x: inc edi
        #89,#F8,                --  151: blu0: mov eax, edi
        #83,#F0,#1F,            --  153: xor eax, #001F
        #A9,#1F,#00,#00,#00,    --  156: test eax,#001F
        #74,#17,                --  15B: jz blu63
        #47,                    --  15D: inc edi
        #39,#34,#BD,#00,#00,#00,#00,--  15E: cmp [rgb_diff+edi*4], esi (353)
        #7E,#0C,                --  165: jle blu63x
        #89,#34,#BD,#00,#00,#00,#00,--  167: mov [rgb_diff+edi*4], esi (362)
        #E8,#C9,#FE,#FF,#FF,    --  16E: call near start
        #4F,                    --  173: blu63x: dec edi
        #2B,#B3,#00,#00,#00,#00,--  174: blu63: sub esi, dword ptr [squares + ebx] (374)
        #83,#EB,#04,            --  17A: sub ebx,4
        #03,#B3,#00,#00,#00,#00,--  17D: add esi, dword ptr [squares + ebx] (383)
        #C3})                   --  183: ret
    poke4(proc + 52, proc + 66) -- @current_col
    poke4(proc + 62, rgb_map)
    poke(proc + 66, 0) -- current_col)
    poke4(proc + 69, squares)
    poke4(proc + 78, squares)
    poke4(proc + 99, rgb_diff)
    poke4(proc + 108, rgb_diff)
    poke4(proc + 146, rgb_diff)
    poke4(proc + 155, rgb_diff)
    poke4(proc + 172, squares)
    poke4(proc + 181, squares)
    poke4(proc + 187, squares)
    poke4(proc + 196, squares)
    poke4(proc + 214, rgb_diff)
    poke4(proc + 223, rgb_diff)
    poke4(proc + 255, rgb_diff)
    poke4(proc + 264, rgb_diff)
    poke4(proc + 278, squares)
    poke4(proc + 287, squares)
    poke4(proc + 293, squares)
    poke4(proc + 302, squares)
    poke4(proc + 318, rgb_diff)
    poke4(proc + 327, rgb_diff)
    poke4(proc + 353, rgb_diff)
    poke4(proc + 362, rgb_diff)
    poke4(proc + 374, squares)
    poke4(proc + 383, squares)
    call(proc)
    free(proc)
    free(squares)
    free(rgb_diff)
end procedure

global function rgb(object r, object g, object b)
--> convert 24-bit rgb's into current pixel depth
    sequence result
    integer index
    if atom(r) then r = {r} end if
    if atom(g) then g = {g} end if
    if atom(b) then b = {b} end if
    result = repeat(0, length(r) * bytes_per_pixel)
    index = 1
    if bits_per_pixel = 8 then
        if not rgb_map then
            make_rgb_map()
        end if
        for i = 1 to length(r) do
            result[i] = peek(rgb_map + floor(r[i]/8)*1024+floor(g[i]/8)*32+floor(b[i]/8))
        end for
    elsif bits_per_pixel = 15 then
        for x = 1 to length(r) do
            result[index] = floor(b[x] / 8) + and_bits(g[x] * 4, #E0)
            result[index+1] = floor(g[x] / 64) + and_bits(r[x] / 2, #7C)
            index = index + 2
        end for
    elsif bits_per_pixel = 16 then
        for x = 1 to length(r) do
            result[index] = floor(b[x] / 8) + and_bits(g[x] * 8, #E0)
            result[index+1] = floor(g[x] / 32) + and_bits(r[x], #F8)
            index = index + 2
        end for
    elsif bits_per_pixel = 24 then
        for x = 1 to length(r) do
            result[index] = b[x]
            result[index+1] = g[x]
            result[index+2] = r[x]
            index = index + 3
        end for
    elsif bits_per_pixel = 32 then
        for x = 1 to length(r) do
            result[index] = b[x]
            result[index+1] = g[x]
            result[index+2] = r[x]
            result[index+3] = 0
            index = index + 4
        end for
    end if
    return result
end function

global function rgb_pal(sequence Palette)
--> convert 24-bit palette entries into current pixel depth
    sequence result
    integer index
    index = 1
    result = repeat(0,length(Palette) * bytes_per_pixel)
    if bits_per_pixel = 8 then
        if not rgb_map then
            make_rgb_map()
        end if
        for i = 1 to length(Palette) do
            result[i] = peek(rgb_map +
                floor(Palette[i][1] / 8) * 1024 +
                floor(Palette[i][2] / 8) * 32 +
                floor(Palette[i][3] / 8))
        end for
    elsif bits_per_pixel = 15 then
        for x = 1 to length(Palette) do
            result[index] = floor(Palette[x][3] / 8) + and_bits(Palette[x][2] * 4, #E0)
            result[index+1] = floor(Palette[x][2] / 64) + and_bits(Palette[x][1] / 2, #7C)
            index = index + 2
        end for
    elsif bits_per_pixel = 16 then
        for x = 1 to length(Palette) do
            result[index] = floor(Palette[x][3] / 8) + and_bits(Palette[x][2] * 8, #E0)
            result[index+1] = floor(Palette[x][2] / 32) + and_bits(Palette[x][1], #F8)
            index = index + 2
        end for
    elsif bits_per_pixel = 24 then
        for x = 1 to length(Palette) do
            result[index] = Palette[x][3]
            result[index+1] = Palette[x][2]
            result[index+2] = Palette[x][1]
            index = index + 3
        end for
    elsif bits_per_pixel = 32 then
        for x = 1 to length(Palette) do
            result[index] = Palette[x][3]
            result[index+1] = Palette[x][2]
            result[index+2] = Palette[x][1]
            result[index+3] = 0
            index = index + 4
        end for
    end if
    return result
end function

------------------------------------------------------------------------
--PIXEL ROUTINES--------------------------------------------------------
------------------------------------------------------------------------

---------sets the active VGA page using VESA
global procedure set_page(integer page)       --VESA interrupt interface
  sequence regs
  regs=repeat(0,10)
  regs[REG_AX] = #4F05                      --VESA interrupt to set bank
  regs[REG_BX] = #0000                      --affect window 0
  regs[REG_DX] = page         --set page
  regs = dos_interrupt(#10, regs)           --call the interrupt
end procedure


global procedure color_convert(integer flag)
--> toggles color conversions for pixel, get_pixel, line, etc.
--> with color_convert(1) (default), pixel data must be in #RRGGBB hex 
-->     format, #FF0000=RED #00FF00=GREEN #0000FF=BLUE
--> with color_convert(0), pixel data is simply poked into memory
--> (Note: this is the same as "raw_mode = not flag")
    raw_mode = not flag    
end procedure

global procedure pixel(object data, sequence xy)
--> draws a pixel on the screen or virtual screen (xy[3] may = virtual_screen)
--> pixel data is in #RRGGBB hex format: #FF0000=RED #00FF00=GREEN #0000FF=BLUE
--> (unless global var raw_mode = 1, then pixel data will NOT be converted)
--> See also: color_convert, get_pixel, draw_line
--> xy can take {x,y} or {x,y,virtual screen}
--> *** no clipping is performed due to speed constraints ***

    atom offset, len, x, c
    sequence buf
    if length(xy) = 3 then
        offset = xy[3] + xy[2] * peek4u(screen_actual_width+xy[3]) + xy[1] * bytes_per_pixel
    else 
        offset = video_mem + xy[2] * bytes_per_line + xy[1] * bytes_per_pixel
    end if
    if mode_type = MODE_X then
        x = and_bits(xy[1],3)
        offset = video_mem + xy[2] * bytes_per_line

        --floor(xy[1] / 4)
        return
    end if
    if atom(data) then 
        data = {data} 
    elsif length(data) = 0 then 
        return 
    end if
    if mode_type = LINEAR or length(xy) = 3 then
        if raw_mode then
            poke(offset, data)
        elsif bits_per_pixel = 32 then
            poke4(offset, data)
        elsif bits_per_pixel = 24 then
            for i = 1 to length(data) do
                poke(offset, floor(data[i] / {1,256,65536}))
                offset = offset + 3 
            end for
        elsif bits_per_pixel = 15 then
            for i = 1 to length(data) do
                c = data[i]
                poke(offset, and_bits(floor(c / 8),#1F) + and_bits(c / 64, #E0))
                poke(offset+1, and_bits(floor(c / 16384),#7) + and_bits(c / 131072, #F8))
                offset = offset + 2
            end for
        elsif bits_per_pixel = 16 then
            for i = 1 to length(data) do
                c = data[i]
                poke(offset, and_bits(floor(c / 8),#1F) + and_bits(c / 32, #E0))
                poke(offset+1, and_bits(floor(c / 8192),#7) + and_bits(c / 65536, #F8))
                offset = offset + 2
            end for
        else
            for i = 1 to length(data) do
                c = data[i]
                mem_copy(offset, rgb_map + 
                    and_bits(floor(c / 512), #7C00) +
                    and_bits(floor(c / 64), #3E0) +
                    and_bits(floor(c / 8), #1F),
                    1)
                offset = offset + 1
            end for
        end if
        return
    end if

    if not raw_mode then
        
        buf = repeat(0, bytes_per_pixel * length(data))
        if bits_per_pixel = 32 then
            for i = 1 to length(data) do
                buf[i*4-3..i*4] = floor(data[i] / {1,65536,256,1})
            end for
        elsif bits_per_pixel = 24 then
            for i = 1 to length(data) do
                buf[i*3-2..i*3] = floor(data[i] / {1,256,65536})
            end for
        elsif bits_per_pixel = 15 then
            for i = 1 to length(data) do
                c = data[i]
                buf[i*2-1] = and_bits(floor(c / 8),#1F) + and_bits(c / 64, #E0)
                buf[i*2] = and_bits(floor(c / 16384),#3) + and_bits(c / 131072, #7C)
            end for
        elsif bits_per_pixel = 16 then
            for i = 1 to length(data) do
                c = data[i]
                buf[i*2-1] = and_bits(floor(c / 8),#1F) + and_bits(c / 32, #E0)
                buf[i*2] = and_bits(floor(c / 8192),#7) + and_bits(c / 65536, #F8)
            end for
        else
            for i = 1 to length(data) do
                c = data[i]
                buf[i] = peek(rgb_map + 
                    and_bits(floor(c / 512), #7C00) +
                    and_bits(floor(c / 64), #3E0) +
                    and_bits(floor(c / 8), #1F))
            end for
        end if
    
        data = buf
    end if

    if mode_type = LINEAR then
        poke(offset, data)
        return
    end if

    offset = offset - video_mem
    if offset < bank_start or offset > bank_start + 65535 then
        --poke4(bank_regs + #14, floor(offset / bank_granularity))
        bank_start = floor(offset / bank_granularity)
        set_page(bank_start)
        --poke4(bank_regs + #14, bank_start)
        --call(bank_proc)
        bank_start = bank_start * bank_granularity
    end if
    len = length(data)
    if offset + len > bank_start + 65535 then
        x = bank_start + 65536 - offset
        poke(video_mem + offset - bank_start, data[1..x])
        bank_start = floor((offset+len) / bank_granularity)
        set_page(bank_start)
        --poke4(bank_regs + #14, bank_start)
        --call(bank_proc)
        bank_start = bank_start * bank_granularity
        poke(video_mem + offset - bank_start + x, data[x+1..len])
    else
        poke(video_mem + offset - bank_start, data)
    end if
end procedure

global function get_pixel(sequence xy)
--> gets pixels from the screen or a virtual screen (xy[4] may = virtual_screen)
--> xy can take {x,y} or {x,y,count} or {x,y,count,virtual screen}
--> pixels returned are in #RRGGBB format
--> (unless global var raw_mode = 1, then pixel data will NOT be converted)
--> See also: color_convert, pixel, draw_line
--> *** no clipping is performed due to speed constraints ***
    atom offset, len, x
    sequence pix, buf
    len = bytes_per_pixel
    if length(xy) >= 3 then len = len * xy[3] end if
    if length(xy) = 4 then 
        offset = xy[2] * peek4u(screen_actual_width+xy[4]) + xy[1] * bytes_per_pixel
        pix = peek({xy[4] + offset, len})
    else
        offset = xy[2] * bytes_per_line + xy[1] * bytes_per_pixel
        if mode_type = LINEAR then
            if bits_per_pixel = 32 and not raw_mode then
                if length(xy) = 2 then
                    return peek4u(video_mem + offset)
                else
                    return peek4u({video_mem + offset, xy[3]})
                end if
            end if
            pix = peek({video_mem + offset, len})
        else
            if offset < bank_start or offset > bank_start + 65535 then
                bank_start = floor(offset / bank_granularity)
                set_page(bank_start)
                --poke4(bank_regs + #14, bank_start)
                --call(bank_proc)
                bank_start = bank_start * bank_granularity
            end if
            if offset + len > bank_start + 65535 then
                x = bank_start + 65536 - offset
                pix = peek({video_mem + offset - bank_start, x})
                bank_start = floor((offset+len) / bank_granularity)
                set_page(bank_start)
                --poke4(bank_regs + #14, bank_start)
                --call(bank_proc)
                bank_start = bank_start * bank_granularity
                pix = pix & peek({video_mem + offset - bank_start + x, len-x})
            else
                pix = peek({video_mem + offset - bank_start, len})
            end if
        end if
    end if        
    if raw_mode then
        return pix
    end if
    buf = repeat(0, len / bytes_per_pixel)    
    if bits_per_pixel = 32 then
        for i = 1 to length(buf) do
            buf[i] = pix[i*4-3] + 256*pix[i*4-2] + 65536*pix[i*4-1]
        end for
    elsif bits_per_pixel = 24 then
        for i = 1 to length(buf) do
            buf[i] = pix[i*3-2] + 256*pix[i*3-1] + 65536*pix[i*3]
        end for
    elsif bits_per_pixel = 16 then
        for i = 1 to length(buf) do
            buf[i] = 8 * and_bits(pix[i*2-1],#1F) +  -- blue
                     32 * and_bits(pix[i*2-1],#E0) + -- green
                     8192 * and_bits(pix[i*2],#07) + 
                     65536 * and_bits(pix[i*2],#F8)  -- red
        end for
    elsif bits_per_pixel = 15 then
        for i = 1 to length(buf) do
            buf[i] = 8 * and_bits(pix[i*2-1],#1F) +  -- blue
                     16 * and_bits(pix[i*2-1],#E0) +  -- green
                     4096 * and_bits(pix[i*2],#03) + 
                     131072 * and_bits(pix[i*2],#7C)  -- red
        end for
    else
        for i = 1 to length(buf) do
            out_byte(967, pix[i])   -- lazy lazy lazy, reading from pal regs
            buf[i] = 262144*in_byte(969) + 1024*in_byte(969) + 4*in_byte(969)
        end for        
    end if
    if length(xy) = 2 then
        buf = buf[1]
    end if
    return buf
end function

global procedure display_image(sequence pos, sequence image)
--> display a 2-d sequence on the screen (pos[3] may = virtual_screen)
--> pixel image is in #RRGGBB hex format: #FF0000=RED #00FF00=GREEN #0000FF=BLUE
--> *** no clipping is performed due to speed constraints ***
    for y = 1 to length(image) do
        pixel(image[y], pos)
        pos[2] = pos[2] + 1
    end for
end procedure

global procedure Blit(sequence src, sequence dest)
--> Blit({source_handle,source_x,source_y,source_w,source_h},{dest_handle,dest_x,dest_y})
--> slower Blit, used for debugging/comparing the banked blit
--> *** Please don't use this **** for testing purposes only ***
    atom offset, source, len, soff, x, bank_limit
    soff = peek4u(screen_actual_width+src[1])
    source = src[1] + src[3] * soff + src[2] * bytes_per_pixel
    len = src[4] * bytes_per_pixel

    if dest[1] = screen and mode_type = BANKED then
        offset = dest[3] * bytes_per_line + dest[2] * bytes_per_pixel
        bank_start = floor(offset / bank_granularity)
        set_page(bank_start)
        --poke4(bank_regs + #14, bank_start)
        --call(bank_proc)
        bank_start = bank_start * bank_granularity
        bank_limit = video_mem + #10000 - len
        offset = offset + video_mem - bank_start
        
        for i = 1 to src[5] do
            if offset > bank_limit then
                x = offset-bank_limit
                if x < len then
                    mem_copy(offset, source, len-x)
                end if
                bank_start = bank_start + #10000
                set_page(bank_start)
                --poke4(bank_regs + #14, floor(bank_start/bank_granularity))
                --call(bank_proc)
                offset = #A0000
                if x < len then
                    mem_copy(offset, source + (len-x), x)
                    offset = offset - (len-x)
                else
                    offset = offset - (len-x)
                    mem_copy(offset, source, len)
                end if
                
--            if offset + len > bank_start + 65535 then
--                x = bank_start + 65536 - offset
--                mem_copy(video_mem + offset - bank_start, source, x)
--                bank_start = floor((offset+len) / bank_granularity)
--                poke4(bank_regs + #14, bank_start)
--                call(bank_proc)
--                bank_start = bank_start * bank_granularity
--                mem_copy(video_mem + offset - bank_start + x, source+x, len-x)
            else
                mem_copy(offset, source, len)
            end if
            source = source + soff
            offset = offset + bytes_per_line
        end for

    elsif mode_type = MODE_X then
        -- too hard. skip it    
    else -- modetype = linear
        if dest[1] = screen then
            x = bytes_per_line
            offset = video_mem + x * dest[3] + dest[2] * bytes_per_pixel
        else
            x = peek4u(screen_actual_width + dest[1])
            offset = dest[1] + x * dest[3] + dest[2] * bytes_per_pixel
        end if
        for i = 1 to src[5] do
            mem_copy(offset, source, len)
            source = source + soff
            offset = offset + x
        end for
    end if
end procedure



------------------------------------------------------------------------
--LOAD_BITMAP-----------------------------------------------------------
------------------------------------------------------------------------

constant BMPFILEHDRSIZE = 14
constant OLDHDRSIZE = 12, NEWHDRSIZE = 40
constant EOF = -1

-- error codes returned by read_bitmap(), save_bitmap() and save_screen()
global constant BMP_SUCCESS = 0,
                BMP_OPEN_FAILED = 1,
                BMP_UNEXPECTED_EOF = 2,
                BMP_UNSUPPORTED_FORMAT = 3,
                BMP_INVALID_MODE = 4
         
integer use_bitmap_palette_flag
    use_bitmap_palette_flag = 0
global procedure use_bitmap_palette()
    use_bitmap_palette_flag = 1
end procedure


integer fn, error_code

function get_word()
-- read 2 bytes
    integer lower, upper
    lower = getc(fn)
    upper = getc(fn)
    if upper = EOF then
        error_code = BMP_UNEXPECTED_EOF
    end if
    return upper * 256 + lower
end function

function get_dword()
-- read 4 bytes
    integer lower, upper
    lower = get_word()
    upper = get_word()
    return upper * 65536 + lower
end function

function get_c_block(integer num_bytes)
-- read num_bytes bytes
    sequence s
    
    s = repeat(0, num_bytes)
    for i = 1 to num_bytes do
        s[i] = getc(fn)
    end for
    if s[length(s)] = EOF then
        error_code = BMP_UNEXPECTED_EOF
    end if
    return s
end function

function get_rgb(integer set_size)
-- get red, green, blue palette values
    integer red, green, blue
    
    blue = getc(fn)
    green = getc(fn)
    red = getc(fn)
    if set_size = 4 then
        if getc(fn) then
        end if
    end if
    return {red, green, blue}
end function

function get_rgb_block(integer num_dwords, integer set_size)
-- reads palette 
    sequence s

    s = {}
    for i = 1 to num_dwords do
        s = append(s, get_rgb(set_size))
    end for
    if s[length(s)][3] = EOF then
        error_code = BMP_UNEXPECTED_EOF
    end if
    return s
end function

function row_bytes(atom BitCount, atom Width)
-- number of bytes per row of pixel data
    return floor(((BitCount * Width) + 31) / 32) * 4
end function

global function load_bitmap(sequence file_name)
--> read a bitmap (.BMP) file into a new virtual screen
--> Bitmaps are automatically converted to the current color depth.
--> In 256-color modes, call use_bitmap_palette() to set the current 
--> palette to the bitmap's palette, otherwise colors will be matched 
--> to the current palette.
--> A graphics mode must be set before attempting load_bitmap().
--> returns {virtual screen handle, width, height}
--> Use free_screen() to dispose of the virtual screen handle.
--> See also: make_rgb_map, use_bitmap_palette
    atom Size
    integer Type, Xhot, Yhot, Planes, BitCount, byte
    atom Width, Height, Compression, OffBits, SizeHeader, 
         SizeImage, XPelsPerMeter, YPelsPerMeter, ClrUsed,
         ClrImportant, NumColors
    sequence Bits, Palette
    atom two_d_vs, vs_ptr

    error_code = 0
    fn = open(file_name, "rb")
    if fn = -1 then
        return BMP_OPEN_FAILED
    end if
    Type = get_word()
    Size = get_dword()
    Xhot = get_word()
    Yhot = get_word()
    OffBits = get_dword()
    SizeHeader = get_dword()

    if SizeHeader = NEWHDRSIZE then
        Width = get_dword()
        Height = get_dword()
        Planes = get_word()
        BitCount = get_word()
        Compression = get_dword()
        if Compression != 0 then
            return BMP_UNSUPPORTED_FORMAT
        end if
        SizeImage = get_dword()
        XPelsPerMeter = get_dword()
        YPelsPerMeter = get_dword()
        ClrUsed = get_dword()
        ClrImportant = get_dword()
        NumColors = (OffBits - SizeHeader - BMPFILEHDRSIZE) / 4
        if BitCount > 8 then
            if not rgb_map and bits_per_pixel = 8 then
                make_rgb_map()
            end if
            Palette = {}        
        elsif NumColors < 2 or NumColors > 256 then
            return BMP_UNSUPPORTED_FORMAT
        else
            Palette = get_rgb_block(NumColors, 4)
        end if
    elsif SizeHeader = OLDHDRSIZE then 
        Width = get_word()
        Height = get_word()
        Planes = get_word()
        BitCount = get_word()
        NumColors = (OffBits - SizeHeader - BMPFILEHDRSIZE) / 3
        SizeImage = row_bytes(BitCount, Width) * Height
        Palette = get_rgb_block(NumColors, 3) 
    else
        return BMP_UNSUPPORTED_FORMAT
    end if
    if Planes != 1 or Height <= 0 or Width <= 0 
    or not find(BitCount, {1,2,4,8,24}) then
        return BMP_UNSUPPORTED_FORMAT
    end if

    if use_bitmap_palette_flag and bits_per_pixel = 8 then
        use_bitmap_palette_flag = 0
        if length(Palette) then
            out_byte(968,0)         -- load the palette registers with Palette
            for i = 1 to length(Palette) do
                out_byte(969, floor(Palette[i] / 4))
                Palette[i] = i-1
            end for
            
            make_rgb_map()
        elsif not rgb_map then
            make_rgb_map()
        end if
    else
        if not rgb_map and bits_per_pixel = 8 then
            make_rgb_map()
        end if
        for i = 1 to length(Palette) do
            Palette[i] = rgb_pal({Palette[i]})
        end for
    end if

    two_d_vs = virtual_screen(Width, Height)
    Bits = repeat(0, row_bytes(BitCount, Width))
    for i = 1 to Height do
        for j = 1 to length(Bits) do
            Bits[j] = getc(fn)
        end for
        vs_ptr = two_d_vs + Width * (Height-i) * bytes_per_pixel
        if BitCount = 1 then
            for j = 1 to floor(Width / 8) do
                byte = Bits[j]
                for k = 0 to 7 * bytes_per_pixel by -bytes_per_pixel do
                    poke(vs_ptr + k, Palette[and_bits(byte, 1)+1])
                    byte = floor(byte / 2)
                end for
                vs_ptr = vs_ptr + 8 * bytes_per_pixel
            end for
        elsif BitCount = 2 then
            for j = 1 to floor(Width / 4) do
                byte = Bits[j]
                for k = 0 to 3 * bytes_per_pixel by -bytes_per_pixel do
                    poke(vs_ptr + k, Palette[and_bits(byte, 3)+1])
                    byte = floor(byte / 4)
                end for
                vs_ptr = vs_ptr + 4 * bytes_per_pixel
            end for
        elsif BitCount = 4 then
            for j = 1 to floor(Width / 2) do
                byte = Bits[j]
                poke(vs_ptr, Palette[floor(byte/16)+1])
                vs_ptr = vs_ptr + bytes_per_pixel
                poke(vs_ptr, Palette[and_bits(byte, 15)+1])
                vs_ptr = vs_ptr + bytes_per_pixel
            end for
        elsif BitCount = 8 then
            for j = 1 to Width do
                poke(vs_ptr, Palette[Bits[j]+1])
                vs_ptr = vs_ptr + bytes_per_pixel
            end for
        elsif BitCount = 24 then
            if bits_per_pixel = 8 and rgb_map then
                for b = 1 to Width*3 by 3 do
                    poke(vs_ptr, peek(rgb_map +
                        floor(Bits[b+2]/8)*1024+
                        floor(Bits[b+1]/8)*32+
                        floor(Bits[b]/8)))
                    vs_ptr = vs_ptr + 1
                end for
            elsif bits_per_pixel = 15 then
                for b = 1 to Width*3 by 3 do
                    poke(vs_ptr, floor(Bits[b] / 8) + and_bits(Bits[b+1] * 4, #E0))
                    poke(vs_ptr+1, floor(Bits[b+1] / 64) + and_bits(Bits[b+2] / 2, #7C))
                    vs_ptr = vs_ptr + 2
                end for
            elsif bits_per_pixel = 16 then
                for b = 1 to Width*3 by 3 do
                    poke(vs_ptr, floor(Bits[b] / 8) + and_bits(Bits[b+1] * 8, #E0))
                    poke(vs_ptr+1, floor(Bits[b+1] / 32) + and_bits(Bits[b+2], #F8))
                    vs_ptr = vs_ptr + 2
                end for
            elsif bits_per_pixel = 24 then
                for b = 1 to Width*3 by 3 do
                    poke(vs_ptr, Bits[b..b+2])
                    vs_ptr = vs_ptr + 3
                end for
            elsif bits_per_pixel = 32 then
                for b = 1 to Width*3 by 3 do
                    poke(vs_ptr, Bits[b..b+2])
                    vs_ptr = vs_ptr + 4
                end for
            end if
        end if
    end for
    close(fn)
    if Bits[length(Bits)] = EOF then
        free_screen(two_d_vs)
        return BMP_UNEXPECTED_EOF        
    end if
    return {two_d_vs, Width, Height}
end function

global procedure save_bitmap24(sequence file_name, object image)
--> saves a 24-bpp bitmap 
--> image is a 2-d sequence of integers in #RRGGBB format
--> or image can be a virtual screen handle
    integer fn
    sequence line
    fn = open(file_name, "wb")
    if fn = -1 then
        return
    end if
    if atom(image) then
        line = {}
        for y = 0 to peek4u(image+screen_height)-1 do
            line = append(line, get_pixel({0,y,peek4u(image+screen_width),image}))
        end for
        image = line
    end if
    line = repeat(0, row_bytes(24,length(image[1])))
    puts(fn, "BM"& -- type
        int_to_bytes(14+40+length(image)*length(line))&-- size
        0&0&0&0& -- hotx,hoty
        54&0&0&0& -- offbits
        40&0&0&0& -- sizeheader
        int_to_bytes(length(image[1]))& -- width
        int_to_bytes(length(image))& -- height
        1&0& -- planes
        24&0& -- bitcount
        0&0&0&0& -- compression
        0&0&0&0& -- sizeimage
        64&11&0&0& -- xpelspermeter
        64&11&0&0& -- ypelspermeter
        0&0&0&0& -- clrused
        0&0&0&0) -- clrimportant
    for y = length(image) to 1 by -1 do
        for x = 1 to length(image[1]) do
            line[x*3-2..x*3] = floor(image[y][x] / {1,256,65536})
        end for
        puts(fn, line)
    end for
    close(fn)        
end procedure

global procedure save_bitmap8(sequence file_name, object image)
--> saves a 8-bpp bitmap 
--> image is a 2-d sequence of integers in 256-color format
--> or image can be a virtual screen handle
    integer fn, width
    sequence line, pal
    fn = open(file_name, "wb")
    if fn = -1 then
        return
    end if
    if atom(image) then
        line = {}
        width = peek4u(image+screen_width)
        for y = 0 to peek4u(image+screen_height)-1 do
            line = append(line, peek({image+y*width,width}))
        end for
        image = line
    end if
    line = repeat(0, row_bytes(8,length(image[1])))
    puts(fn, "BM"& -- type
        int_to_bytes(14+40+length(image)*length(line)+1024)&-- size
        0&0&0&0& -- hotx,hoty
        54&4&0&0& -- offbits
        40&0&0&0& -- sizeheader
        int_to_bytes(length(image[1]))& -- width
        int_to_bytes(length(image))& -- height
        1&0& -- planes
        8&0& -- bitcount
        0&0&0&0& -- compression
        0&0&0&0& -- sizeimage
        64&11&0&0& -- xpelspermeter
        64&11&0&0& -- ypelspermeter
        0&1&0&0& -- clrused
        0&1&0&0) -- clrimportant
    out_byte(967,0)
    for i = 1 to 256 do
        pal = {in_byte(969), 0}
        pal = in_byte(969) & pal
        pal = in_byte(969) & pal
        puts(fn, pal*255/63)
    end for
    line = repeat(0, length(line) - length(image[1]))
    for y = length(image) to 1 by -1 do
        puts(fn, image[y] & line)
    end for
    close(fn)        
end procedure


------------------------------------------------------------------------
--COMMAND LISTS---------------------------------------------------------
------------------------------------------------------------------------

sequence safe_handles
safe_handles = {}
type new_safe_handle(atom handle)
    safe_handles = append(safe_handles, handle)    
    return 1
end type
type safe_handle(atom handle)
    integer i
    i = find(handle, safe_handles)
    if not i then
        restore_mode()
        printf(2, "\nattempt to use invalid sprite handle #%x\n", {handle})
    end if
    return i
end type
type free_safe_handle(atom handle)
    integer i
    i = find(handle, safe_handles)
    safe_handles = safe_handles[1..i-1] & safe_handles[i+1..length(safe_handles)]
    return i
end type


-- command structure
global constant
     command_code_pointer             = 0, 
     next_command                     = 4,
     prev_command                     = 8,
     dest_z                           = 12,
     source_handle                    = 16,
     dest_vs_handle                   = 20,
     dest_x                           = 24,
     dest_y                           = 28,
     mixing_table_pointer             = 32,
     sprite_flags                     = 36,
     command_size                     = 40

constant scratch_command = allocate0(command_size)
constant scratch_screen = allocate0(screen_header_size + screen_sub_code) + screen_header_size
poke(scratch_screen, {#81,#C3,0,0,0,0,#81,#C2,0,0,0,0,#E9,0,0,0,0})
poke4(scratch_screen + screen_addr_func, scratch_screen)


global constant first_command = allocate_proc(
   {#60,                    --    0: pusha
    #90,                    --    1: nop
    #90,                    --    2: nop
    #BB,#00,#00,#00,#00,    --    3: mov ebx,next_command (4)
    #FF,#23,#00,#00,        --    8: jmp near dword ptr [ebx]
    #00,#00,#00,#80})       -- dest_z = -max_int

global constant last_command = allocate_proc(
   {#00,#00,#00,#00,        -- command_code_pointer
    #61,                    --    4: popa
    #C3,                    --    5: ret
    #00,#00,
    #00,#00,#00,#00,        --    8: prev_command
    #FF,#FF,#FF,#7F})       -- dest_z = max_int

poke4(first_command + next_command, last_command)
poke4(last_command + prev_command, first_command)
poke4(last_command, last_command + 4)
poke4(scratch_command + next_command, last_command)

global function change_list(sequence new_lists)
--> This supports multiple command lists.  new_lists accepts {} to clear the
--> command list, or accepts a sequence previously returned by change_list.
--> change_list always returns a two-element sequence containing the 
--> configuration of the command list before it is changed.  
    sequence old_lists
    old_lists =
        peek4u(first_command + next_command) &
        peek4u(last_command + prev_command)
    if length(new_lists) < 2 then
        -- clear the list
        poke4(first_command + next_command, last_command)
        poke4(last_command + prev_command, first_command)
    else
        poke4(first_command + next_command, new_lists[1])
        poke4(last_command + prev_command, new_lists[2])
    end if
    return old_lists        
end function

global procedure move_command(safe_handle handle, sequence disp)
--> adds disp to the position of a sprite command handle
    poke4(handle+dest_x, disp + peek4s({handle+dest_x,2}))
end procedure

global function get_dest_pos(safe_handle handle)
--> returns {dest_x, dest_y} of the command handle
    return peek4s({handle + dest_x, 2})
end function

global procedure set_dest_pos(safe_handle handle, sequence pos)
--> sets the position of a sprite command handle
--> pos can take: {x,y} -- only sets the position, command list depth unchanged
-->               {x,y,z} -- sets position and depth in the command list,
-->                       -- adding it to the command list if neccessary
--> See also: remove_command() to get it off the command list
    atom next, prev
    integer z
    if length(pos) = 3 then
        next = peek4u(handle + next_command)
        prev = peek4u(handle + prev_command)
        if next then
            poke4(next + prev_command, prev)
        end if
        if prev then
            poke4(prev + next_command, next)
        end if
        
        z = floor(pos[3])
        
        next = peek4u(first_command + next_command)
        while z > peek4s(next + dest_z) do
            next = peek4u(next + next_command)
        end while
        
        prev = peek4u(next + prev_command)
        poke4(prev + next_command, handle)
        poke4(next + prev_command, handle)
        poke4(handle + next_command, next)
        poke4(handle + prev_command, prev)

        poke4(handle + dest_z, z)
    end if
    poke4(handle + dest_x, pos[1..2])
end procedure

global function new_command(atom kind, object source, sequence dest)
--> creates a new command in memory of type kind, and places it in the command
--> list if dest_z is present.  Returned is the command handle.
-->  kind can take: blit, clear_blit, mixer_blit, screen_blit, wait_retrace
-->  source can take a screen handle,
-->           {screen_handle,x,y,width,height},
-->           {screen_handle,x,y,width,height,hot_x,hot_y},
-->        or a rle-sprite handle only if kind is rle_blit.
-->  dest can take {screen_handle, dest_x, dest_y, dest_z} (put in command list)
-->             or {screen_handle, dest_x, dest_y}   (isn't put in command list)  
--> Note: if you use "screen" as the dest screen handle, you should only
-->  use screen_blit for kind.
--> Use free_command() to dispose of the command handle.
-->
--> See also: set_dest_pos, move_command, set_source_handle, remove_command,
-->           free_command, set_mix_table, set_rle_flags

    new_safe_handle handle
    atom attached_screen
    if atom(source) then
        handle = allocate0(command_size)
        poke4(handle + source_handle, source)
    else
        handle = allocate0(command_size + screen_header_size + screen_sub_code)
        attached_screen = handle + command_size + screen_header_size
        poke(attached_screen, {#81,#C3,0,0,0,0,#81,#C2,0,0,0,0,#E9,0,0,0,0})
        poke4(attached_screen + 13,
            peek4u(screen_addr_func + source[1])
            - attached_screen - screen_sub_code)
        poke4(attached_screen + 2, source[2])
        poke4(attached_screen + 8, source[3])
        poke4(attached_screen + screen_width, source[4])
        poke4(attached_screen + screen_height, source[5])
        poke4(attached_screen + screen_actual_width,
            peek4u(source[1] + screen_actual_width))
        poke4(attached_screen + screen_addr_func, attached_screen)
            
        poke4(handle + source_handle, attached_screen)

--        poke4(handle + source_handle, handle + command_size)
--        poke4(handle + 40, source[1] & 0 & source[2..length(source)])
    end if
    poke4(handle + command_code_pointer, kind+6)
    poke4(handle + dest_vs_handle, dest[1])

    set_dest_pos(handle, dest[2..length(dest)])
    return handle
end function

global procedure do_command(atom kind, object source, sequence dest)
--> does a single command with no messing with the command list
-->  kind can take: blit, clear_blit, mixer_blit, screen_blit, wait_retrace
-->  source can take a screen handle,
-->           {screen_handle,x,y,width,height},
-->           {screen_handle,x,y,width,height,hot_x,hot_y},
-->        or a rle-sprite handle only if kind is rle_blit.
-->  dest must take {screen_handle, dest_x, dest_y}
    if atom(source) then
        poke4(scratch_command + source_handle, source)
    else
        poke4(scratch_screen + 13,
            peek4u(screen_addr_func + source[1])
            - scratch_screen - screen_sub_code)
        poke4(scratch_screen + 2, source[2])
        poke4(scratch_screen + 8, source[3])
        poke4(scratch_screen + screen_width, source[4])
        poke4(scratch_screen + screen_height, source[5])
        poke4(scratch_screen + screen_actual_width,
            peek4u(source[1] + screen_actual_width))
            
        poke4(scratch_command + source_handle, scratch_screen)

--        poke4(scratch_command + command_size, source[1] & 0 & source[2..length(source)])
    end if
    
    poke4(scratch_command + dest_vs_handle, dest[1])
    poke4(scratch_command + dest_x, dest[2..3])
    call(kind)
end procedure


global procedure set_source(safe_handle handle, atom source)
--> Sets the source (screen handle, or rle-sprite) of a command.
--> See also: new_source, make_rle_sprite
    poke4(handle + source_handle, source)
end procedure

global procedure set_mix_table(safe_handle handle, atom mix_table)
--> Sets the mixing table associated with a sprite handle
--> mix_table must take an atom from allocate(#10000)
    poke4(handle + mixing_table_pointer, mix_table)
end procedure

global procedure set_bit_mask(safe_handle handle, atom mask)
--> Sets the bit mask for a rle sprite.  This mask is AND'ed with the value
--> used with the hit_sprite function.  If the result is nonzero and the
--> sprites touch, a successful hit occurs.  
    poke4(handle + 36, mask)
end procedure

global procedure execute_commands()
--> Execute all the commands in the command list.  This will render all the
--> commands in the order from lowest to highest dest_z.
    call(first_command)
end procedure

global procedure remove_command(safe_handle handle)
--> Remove the command handle from the command list. The the memory is not
--> freed, so handle may still be used later on.

    atom next, prev
    next = peek4u(handle + next_command)
    prev = peek4u(handle + prev_command)
    if next then
        poke4(next + prev_command, prev)
    end if
    if prev then
        poke4(prev + next_command, next)
    end if
    poke4(handle + next_command, 0)
    poke4(handle + prev_command, 0)
end procedure

global procedure free_command(atom handle)
--> Free the memory associated with the command handle and removes it from
--> the command list if neccessary.  Further use of the value in handle is
--> not recommended, and advised to be set to zero after this function call.
    free_safe_handle junk
    remove_command(handle)
    free(handle)
    junk = handle
end procedure




-- rle sprite handle structure
--   dest y offset                      0
--   next animated frame                4
--   run count (unsigned word)          8 (run + 0)
--   run x offset (signed word)         10 (run + 2)
--   run data                           12 (run + 4)
--   ...

sequence clearcolor
clearcolor = {}

global procedure clear_color(object c)
--> Sets the clear color (in #RRGGBB) to be used with make_rle_sprite.
    if atom(c) then
        clearcolor = rgb_pal({and_bits(255, c / {65536,256,1})})
    else
        clearcolor = c
    end if
end procedure

global function make_rle_sprite(object source)
--> Creates a run-length encoded sprite source handle from the a virtual
--> screen or source sequence.  Set the global variable clear_color (#RRGGBB)
--> to determine which pixels will be transparent. (default #000000)
--> This kind of sprite has the potential to be faster than using clear_blit
--> because this function precalculates the see-through portions of the image.
-->  Source can take a screen handle (use the whole virtual screen),
-->        {screen_handle,x,y,width,height},
-->     or {screen_handle,x,y,width,height,hot_x,hot_y}
-->  An atom is returned containing the rle-sprite handle.
--> Use free() to dispose of a rle sprite handle.
--> See also: new_command, set_source

    atom addr, width, hot_x, hot_y
    integer on, last_len
    sequence data, temp
    if atom(source) then
        source = {source, 0,0, 
            peek4u(source + screen_width), peek4u(source + screen_height)}
    end if
    width = peek4u(source[1] + screen_actual_width)
    addr = source[1] + source[2] * bytes_per_pixel + source[3] * width 
    if length(source) = 7 then
        hot_x = source[6] * bytes_per_pixel
        hot_y = source[7]
    else
        hot_x = 0
        hot_y = 0
    end if

    data = int_to_bytes(hot_y) & {0,0,0,0}
    last_len = 0
    for y = 1 to source[5] do
        on = 0
        for x = 0 to source[4] * bytes_per_pixel - 1 by bytes_per_pixel do
            temp = peek({addr+x,bytes_per_pixel})
            if compare(clearcolor, temp) then
                if on then
                    data[on..on+1] =
                        and_bits(255, floor(
                        (bytes_per_pixel + data[on] + data[on+1]*256)
                        / {1, 256}))
                else
                    on = length(data) + 1
                    data = data & {bytes_per_pixel,0,(x-hot_x),floor((x-hot_x)/256)}
                end if
                data = data & peek({addr+x, bytes_per_pixel})
            else
                on = 0
            end if
        end for
        if last_len = length(data) then
            data[length(data)-1..length(data)] =
                and_bits(255, floor(
                    (1 + data[length(data)-1] + data[length(data)]*256)
                    / {1, 256}))
        else
            data = data & {0,0,1,0}
            last_len = length(data)
        end if
        addr = addr + width
    end for
    return allocate_proc(data & {0,0,0,0})
end function

constant resultptr = allocate(4)
atom rle_hit_proc

global function hit_sprite(atom sprite1, atom bit_mask)
--> Finds the first rle sprite whose pixels overlap rle sprite command handle
--> and whose bit_mask and'ed with bit_mask is non_zero.  Returns the bit 
--> mask of the hit sprite.  Use bit_mask to discern between different types
--> of sprites, i.e. power-ups, bad guys, bullets, etc.  A sprite id can also
--> be encoded in the lower bits of the bit mask for finding which sprite hit.
--> See also: set_bit_mask
    poke4(rle_hit_proc + 22, sprite1)
    poke4(rle_hit_proc + 65, bit_mask)
    call(rle_hit_proc)
    return peek4u(resultptr)
end function


------------------------------------------------------------------------
-- RAW MACHINE CODES :-) -----------------------------------------------
------------------------------------------------------------------------

rle_hit_proc = allocate_proc( {
#60,#C7,#05,#DB,#40,#E7,#82,#00,#00,#00,#00,#C7,#05,#00,#00,#00,#00,#00,#00,#00,
#00,#BE,#00,#00,#00,#00,#B9,#00,#00,#00,#00,#8B,#49,#04,#81,#F9,#00,#00,#00,#00,
#75,#0D,#C7,#05,#00,#00,#00,#00,#00,#00,#00,#00,#61,#C3,#90,#89,#0D,#DB,#40,#E7,
#82,#8B,#41,#24,#A9,#00,#00,#00,#00,#74,#CE,#A2,#00,#00,#00,#00,#8B,#7E,#1C,#31,
#D2,#8B,#46,#14,#51,#8B,#4E,#18,#FF,#50,#FC,#89,#0D,#69,#41,#E7,#82,#59,#8B,#76,
#10,#03,#3E,#83,#C6,#08,#C1,#E7,#10,#8B,#69,#1C,#31,#D2,#8B,#41,#14,#51,#8B,#49,
#18,#FF,#50,#FC,#89,#0D,#97,#41,#E7,#82,#59,#8B,#49,#10,#03,#29,#83,#C1,#08,#C1,
#E5,#10,#89,#CB,#31,#C9,#31,#D2,#39,#FD,#7C,#2E,#8B,#06,#66,#89,#C1,#8D,#74,#31,
#04,#85,#C9,#74,#15,#C1,#F8,#10,#05,#00,#00,#00,#00,#A3,#B5,#41,#E7,#82,#39,#FD,
#7C,#10,#7F,#E0,#EB,#3A,#85,#C0,#0F,#84,#53,#FF,#FF,#FF,#01,#C7,#EB,#D2,#8B,#03,
#66,#89,#C2,#8D,#5C,#1A,#04,#85,#D2,#74,#15,#C1,#F8,#10,#05,#00,#00,#00,#00,#A3,
#BA,#41,#E7,#82,#39,#FD,#7C,#E2,#7F,#B2,#EB,#0C,#85,#C0,#0F,#84,#25,#FF,#FF,#FF,
#01,#C5,#EB,#D2,#B8,#00,#00,#00,#00,#2D,#00,#00,#00,#00,#7D,#08,#F7,#D8,#39,#C8,
#7C,#08,#EB,#90,#39,#D0,#7C,#02,#EB,#B8,#61,#C3})
poke4(rle_hit_proc + 3, rle_hit_proc + 27) -- @command
poke4(rle_hit_proc + 7, first_command)
poke4(rle_hit_proc + 13, resultptr)
poke4(rle_hit_proc + 36, last_command)
poke4(rle_hit_proc + 44, resultptr)
poke4(rle_hit_proc + 57, rle_hit_proc + 27) -- @command
poke4(rle_hit_proc + 72, resultptr)
poke4(rle_hit_proc + 93, rle_hit_proc + 169) -- @x1
poke4(rle_hit_proc + 126, rle_hit_proc + 215) -- @x2
poke4(rle_hit_proc + 174, rle_hit_proc + 245) -- @sprpos1
poke4(rle_hit_proc + 220, rle_hit_proc + 250) -- @sprpos2

rle_blit_proc = allocate_proc( {
#60,#BB,#00,#00,#00,#00,#8B,#43,#04,#A3,#9F,#2E,#CD,#82,#8B,#53,#1C,#8B,#73,#10,
#2B,#16,#8B,#46,#04,#85,#C0,#74,#03,#89,#43,#10,#83,#C6,#08,#FC,#31,#C9,#8B,#43,
#14,#2B,#50,#F4,#8B,#68,#F8,#89,#2D,#59,#2E,#CD,#82,#89,#2D,#8F,#2E,#CD,#82,#8B,
#5B,#18,#2B,#58,#F0,#52,#FF,#50,#FC,#5A,#53,#31,#DB,#FF,#50,#FC,#5A,#29,#DA,#52,
#53,#31,#DB,#8B,#50,#EC,#FF,#50,#FC,#89,#1D,#98,#2E,#CD,#82,#31,#DB,#31,#D2,#FF,
#50,#FC,#89,#1D,#3F,#2E,#CD,#82,#5A,#5B,#3B,#15,#98,#2E,#CD,#82,#7D,#78,#31,#C9,
#EB,#02,#89,#EE,#31,#C9,#8B,#06,#66,#89,#C1,#83,#C6,#04,#85,#C9,#74,#4E,#8D,#2C,
#31,#81,#FA,#00,#00,#00,#00,#7C,#E5,#C1,#F8,#10,#01,#D8,#7D,#08,#01,#C1,#7E,#DA,
#29,#C6,#31,#C0,#8D,#3C,#01,#81,#EF,#00,#00,#00,#00,#7E,#04,#29,#F9,#7E,#C7,#8D,
#3C,#10,#89,#F8,#F7,#D8,#25,#03,#00,#00,#00,#29,#C1,#7E,#0F,#91,#F3,#A4,#89,#C1,
#25,#03,#00,#00,#00,#C1,#E9,#02,#F3,#A5,#01,#C1,#F3,#A4,#EB,#A2,#C1,#E8,#10,#74,
#11,#81,#C2,#00,#00,#00,#00,#48,#75,#F7,#81,#FA,#00,#00,#00,#00,#7C,#8E,#BB,#00,
#00,#00,#00,#FF,#23})
poke4(rle_blit_proc + 2, scratch_command)
poke4(rle_blit_proc + 10, rle_blit_proc + 239) -- @next_command
poke4(rle_blit_proc + 49, rle_blit_proc + 169) -- @bytes_per_line
poke4(rle_blit_proc + 55, rle_blit_proc + 223) -- @bytes_per_line2
poke4(rle_blit_proc + 91, rle_blit_proc + 232) -- @virtual_screen_end
poke4(rle_blit_proc + 104, rle_blit_proc + 143) -- @virtual_screen_start
poke4(rle_blit_proc + 112, rle_blit_proc + 232) -- @virtual_screen_end

bank_blit_proc = allocate_proc( {
#60,#BB,#00,#00,#00,#00,#8B,#43,#04,#A3,#39,#0B,#5D,#83,#8B,#7B,#14,#89,#3D,#65,
#0B,#5D,#83,#8B,#73,#10,#8B,#46,#DC,#85,#C0,#74,#03,#89,#43,#10,#8B,#56,#E8,#8B,
#43,#18,#2B,#47,#F0,#31,#C9,#2B,#46,#E0,#7D,#08,#01,#C2,#7E,#5C,#29,#C1,#31,#C0,
#A3,#6A,#0B,#5D,#83,#89,#0D,#47,#0B,#5D,#83,#F7,#D8,#03,#47,#E8,#39,#C2,#7F,#02,
#89,#D0,#83,#F8,#00,#7E,#3D,#A3,#42,#0B,#5D,#83,#A3,#60,#0B,#5D,#83,#8B,#56,#EC,
#8B,#43,#1C,#2B,#47,#F4,#31,#C9,#2B,#46,#E4,#7D,#08,#01,#C2,#7E,#1F,#29,#C1,#31,
#C0,#A3,#72,#0B,#5D,#83,#89,#0D,#4F,#0B,#5D,#83,#F7,#D8,#03,#47,#EC,#39,#C2,#7F,
#02,#89,#D0,#83,#F8,#00,#7F,#07,#BB,#00,#00,#00,#00,#FF,#23,#89,#C5,#B9,#00,#00,
#00,#00,#BB,#00,#00,#00,#00,#8B,#46,#F8,#BA,#00,#00,#00,#00,#FF,#56,#FC,#29,#C8,
#89,#DE,#A3,#05,#0C,#5D,#83,#B9,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#BB,#00,#00,
#00,#00,#8B,#47,#F8,#BA,#00,#00,#00,#00,#FF,#57,#FC,#29,#C8,#A3,#0B,#0C,#5D,#83,
#89,#0D,#24,#0C,#5D,#83,#BA,#00,#00,#0B,#00,#29,#CA,#89,#15,#ED,#0B,#5D,#83,#89,
#DA,#81,#EA,#00,#00,#0A,#00,#C1,#EA,#00,#B8,#02,#FF,#00,#00,#89,#15,#41,#0C,#5D,
#83,#BF,#00,#00,#00,#00,#89,#57,#14,#C1,#E2,#00,#CD,#31,#29,#D3,#89,#DF,#F7,#DB,
#81,#E3,#03,#00,#00,#00,#29,#D9,#7D,#04,#01,#CB,#31,#C9,#89,#1D,#E7,#0B,#5D,#83,
#88,#CB,#C1,#E9,#02,#80,#E3,#03,#89,#0D,#F6,#0B,#5D,#83,#89,#1D,#FD,#0B,#5D,#83,
#EB,#00,#B9,#00,#00,#00,#00,#81,#FF,#00,#00,#00,#00,#7D,#24,#F3,#A4,#B9,#00,#00,
#00,#00,#F3,#A5,#B9,#00,#00,#00,#00,#F3,#A4,#81,#C6,#00,#00,#00,#00,#81,#C7,#00,
#00,#00,#00,#4D,#75,#D4,#E9,#21,#FF,#FF,#FF,#81,#05,#41,#0C,#5D,#83,#00,#00,#00,
#00,#89,#FB,#B9,#00,#00,#00,#00,#2B,#1D,#ED,#0B,#5D,#83,#29,#D9,#7E,#02,#F3,#A4,
#B8,#02,#FF,#00,#00,#BF,#00,#00,#00,#00,#C7,#47,#14,#00,#00,#00,#00,#CD,#31,#BF,
#00,#00,#0A,#00,#01,#CB,#29,#CF,#89,#D9,#F3,#A4,#EB,#AD})
poke4(bank_blit_proc + 2, scratch_command)
poke4(bank_blit_proc + 10, bank_blit_proc + 149) -- @next_command
poke4(bank_blit_proc + 19, bank_blit_proc + 193) -- @dest_vs
poke4(bank_blit_proc + 61, bank_blit_proc + 198) -- @dest_x
poke4(bank_blit_proc + 67, bank_blit_proc + 163) -- @source_x
poke4(bank_blit_proc + 88, bank_blit_proc + 158) -- @source_w
poke4(bank_blit_proc + 93, bank_blit_proc + 188) -- @source_w2
poke4(bank_blit_proc + 122, bank_blit_proc + 206) -- @dest_y
poke4(bank_blit_proc + 128, bank_blit_proc + 171) -- @source_y
poke4(bank_blit_proc + 183, bank_blit_proc + 353) -- @source_add
poke4(bank_blit_proc + 216, bank_blit_proc + 359) -- @dest_add
poke4(bank_blit_proc + 222, bank_blit_proc + 384) -- @line_length
poke4(bank_blit_proc + 235, bank_blit_proc + 329) -- @bank_limit
poke4(bank_blit_proc + 257, bank_blit_proc + 413) -- @bank_pos
poke4(bank_blit_proc + 296, bank_blit_proc + 323) -- @line1
poke4(bank_blit_proc + 310, bank_blit_proc + 338) -- @line2
poke4(bank_blit_proc + 316, bank_blit_proc + 345) -- @line3
poke4(bank_blit_proc + 373, bank_blit_proc + 413) -- @bank_pos
poke4(bank_blit_proc + 390, bank_blit_proc + 329) -- @bank_limit

blit_proc = allocate_proc( {
#60,#BB,#00,#00,#00,#00,#8B,#43,#04,#A3,#89,#75,#4E,#83,#8B,#7B,#14,#89,#3D,#B5,
#75,#4E,#83,#8B,#73,#10,#8B,#46,#DC,#85,#C0,#74,#03,#89,#43,#10,#8B,#56,#E8,#8B,
#43,#18,#2B,#47,#F0,#31,#C9,#2B,#46,#E0,#7D,#08,#01,#C2,#7E,#5C,#29,#C1,#31,#C0,
#A3,#BA,#75,#4E,#83,#89,#0D,#97,#75,#4E,#83,#F7,#D8,#03,#47,#E8,#39,#C2,#7F,#02,
#89,#D0,#83,#F8,#00,#7E,#3D,#A3,#92,#75,#4E,#83,#A3,#B0,#75,#4E,#83,#8B,#56,#EC,
#8B,#43,#1C,#2B,#47,#F4,#31,#C9,#2B,#46,#E4,#7D,#08,#01,#C2,#7E,#1F,#29,#C1,#31,
#C0,#A3,#C2,#75,#4E,#83,#89,#0D,#9F,#75,#4E,#83,#F7,#D8,#03,#47,#EC,#39,#C2,#7F,
#02,#89,#D0,#83,#F8,#00,#7F,#07,#BB,#00,#00,#00,#00,#FF,#23,#89,#C5,#B9,#00,#00,
#00,#00,#BB,#00,#00,#00,#00,#8B,#46,#F8,#BA,#00,#00,#00,#00,#FF,#56,#FC,#29,#C8,
#89,#DE,#A3,#2A,#76,#4E,#83,#B9,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#BB,#00,#00,
#00,#00,#8B,#47,#F8,#BA,#00,#00,#00,#00,#FF,#57,#FC,#29,#C8,#89,#DF,#A3,#30,#76,
#4E,#83,#75,#13,#0B,#05,#2A,#76,#4E,#83,#75,#0B,#89,#C8,#F7,#E5,#BD,#01,#00,#00,
#00,#89,#C1,#F7,#DB,#81,#E3,#03,#00,#00,#00,#29,#D9,#7D,#04,#01,#CB,#31,#C9,#89,
#1D,#14,#76,#4E,#83,#88,#CB,#C1,#E9,#02,#80,#E3,#03,#89,#0D,#1B,#76,#4E,#83,#89,
#1D,#22,#76,#4E,#83,#EB,#00,#B9,#00,#00,#00,#00,#F3,#A4,#B9,#00,#00,#00,#00,#F3,
#A5,#B9,#00,#00,#00,#00,#F3,#A4,#81,#C6,#00,#00,#00,#00,#81,#C7,#00,#00,#00,#00,
#4D,#75,#DC,#E9,#4C,#FF,#FF,#FF})
poke4(blit_proc + 2, scratch_command)
poke4(blit_proc + 10, blit_proc + 149) -- @next_command
poke4(blit_proc + 19, blit_proc + 193) -- @dest_vs
poke4(blit_proc + 61, blit_proc + 198) -- @dest_x
poke4(blit_proc + 67, blit_proc + 163) -- @source_x
poke4(blit_proc + 88, blit_proc + 158) -- @source_w
poke4(blit_proc + 93, blit_proc + 188) -- @source_w2
poke4(blit_proc + 122, blit_proc + 206) -- @dest_y
poke4(blit_proc + 128, blit_proc + 171) -- @source_y
poke4(blit_proc + 183, blit_proc + 310) -- @source_add
poke4(blit_proc + 218, blit_proc + 316) -- @dest_add
poke4(blit_proc + 226, blit_proc + 310) -- @source_add
poke4(blit_proc + 261, blit_proc + 288) -- @line1
poke4(blit_proc + 275, blit_proc + 295) -- @line2
poke4(blit_proc + 281, blit_proc + 302) -- @line3

clear_blit_proc = allocate_proc( {
#60,#BB,#00,#00,#00,#00,#8B,#43,#04,#A3,#65,#47,#5D,#83,#8B,#7B,#14,#89,#3D,#91,
#47,#5D,#83,#8B,#73,#10,#8B,#46,#DC,#85,#C0,#74,#03,#89,#43,#10,#8B,#56,#E8,#8B,
#43,#18,#2B,#47,#F0,#31,#C9,#2B,#46,#E0,#7D,#08,#01,#C2,#7E,#5C,#29,#C1,#31,#C0,
#A3,#96,#47,#5D,#83,#89,#0D,#73,#47,#5D,#83,#F7,#D8,#03,#47,#E8,#39,#C2,#7F,#02,
#89,#D0,#83,#F8,#00,#7E,#3D,#A3,#6E,#47,#5D,#83,#A3,#8C,#47,#5D,#83,#8B,#56,#EC,
#8B,#43,#1C,#2B,#47,#F4,#31,#C9,#2B,#46,#E4,#7D,#08,#01,#C2,#7E,#1F,#29,#C1,#31,
#C0,#A3,#9E,#47,#5D,#83,#89,#0D,#7B,#47,#5D,#83,#F7,#D8,#03,#47,#EC,#39,#C2,#7F,
#02,#89,#D0,#83,#F8,#00,#7F,#07,#BB,#00,#00,#00,#00,#FF,#23,#89,#C5,#B9,#00,#00,
#00,#00,#BB,#00,#00,#00,#00,#8B,#46,#F8,#BA,#00,#00,#00,#00,#FF,#56,#FC,#29,#C8,
#89,#DE,#A3,#04,#48,#5D,#83,#B9,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#BB,#00,#00,
#00,#00,#8B,#47,#F8,#BA,#00,#00,#00,#00,#FF,#57,#FC,#29,#C8,#89,#DF,#A3,#0A,#48,
#5D,#83,#F7,#DB,#81,#E3,#03,#00,#00,#00,#29,#D9,#7D,#04,#01,#CB,#31,#C9,#89,#1D,
#E5,#47,#5D,#83,#88,#CB,#80,#E3,#03,#C1,#E9,#02,#75,#08,#01,#1D,#E5,#47,#5D,#83,
#89,#CB,#89,#0D,#FC,#47,#5D,#83,#89,#1D,#58,#48,#5D,#83,#EB,#00,#B9,#00,#00,#00,
#00,#85,#C9,#74,#0D,#8A,#06,#84,#C0,#74,#02,#88,#07,#46,#47,#49,#75,#F3,#81,#C1,
#00,#00,#00,#00,#75,#14,#81,#C6,#00,#00,#00,#00,#81,#C7,#00,#00,#00,#00,#4D,#75,
#D3,#E9,#4E,#FF,#FF,#FF,#8B,#06,#85,#C0,#74,#31,#89,#C3,#C1,#EB,#10,#84,#C0,#74,
#10,#84,#E4,#74,#42,#84,#DB,#74,#43,#84,#FF,#74,#47,#89,#07,#EB,#18,#47,#84,#E4,
#74,#02,#88,#27,#47,#84,#DB,#74,#02,#88,#1F,#47,#84,#FF,#74,#02,#88,#3F,#83,#EF,
#03,#83,#C7,#04,#83,#C6,#04,#49,#75,#C0,#81,#C1,#00,#00,#00,#00,#74,#A4,#8A,#06,
#84,#C0,#74,#02,#88,#07,#46,#47,#49,#EB,#F1,#88,#07,#47,#EB,#CC,#88,#07,#47,#88,
#27,#47,#EB,#CB,#88,#07,#47,#88,#27,#47,#88,#1F,#83,#EF,#02,#EB,#C8})
poke4(clear_blit_proc + 2, scratch_command)
poke4(clear_blit_proc + 10, clear_blit_proc + 149) -- @next_command
poke4(clear_blit_proc + 19, clear_blit_proc + 193) -- @dest_vs
poke4(clear_blit_proc + 61, clear_blit_proc + 198) -- @dest_x
poke4(clear_blit_proc + 67, clear_blit_proc + 163) -- @source_x
poke4(clear_blit_proc + 88, clear_blit_proc + 158) -- @source_w
poke4(clear_blit_proc + 93, clear_blit_proc + 188) -- @source_w2
poke4(clear_blit_proc + 122, clear_blit_proc + 206) -- @dest_y
poke4(clear_blit_proc + 128, clear_blit_proc + 171) -- @source_y
poke4(clear_blit_proc + 183, clear_blit_proc + 308) -- @source_add
poke4(clear_blit_proc + 218, clear_blit_proc + 314) -- @dest_add
poke4(clear_blit_proc + 240, clear_blit_proc + 277) -- @line1
poke4(clear_blit_proc + 256, clear_blit_proc + 277) -- @line1
poke4(clear_blit_proc + 264, clear_blit_proc + 300) -- @line2
poke4(clear_blit_proc + 270, clear_blit_proc + 392) -- @line3

mixer_blit_proc = allocate_proc( {
#60,#BB,#00,#00,#00,#00,#8B,#43,#04,#A3,#ED,#71,#5D,#83,#8B,#7B,#14,#89,#3D,#3A,
#72,#5D,#83,#8B,#73,#10,#8B,#46,#DC,#85,#C0,#74,#03,#89,#43,#10,#8B,#56,#E8,#8B,
#43,#18,#2B,#47,#F0,#31,#C9,#2B,#46,#E0,#7D,#08,#01,#C2,#7E,#5C,#29,#C1,#31,#C0,
#A3,#3F,#72,#5D,#83,#89,#0D,#1C,#72,#5D,#83,#F7,#D8,#03,#47,#E8,#39,#C2,#7F,#02,
#89,#D0,#83,#F8,#00,#7E,#3D,#A3,#17,#72,#5D,#83,#A3,#35,#72,#5D,#83,#8B,#56,#EC,
#8B,#43,#1C,#2B,#47,#F4,#31,#C9,#2B,#46,#E4,#7D,#08,#01,#C2,#7E,#1F,#29,#C1,#31,
#C0,#A3,#47,#72,#5D,#83,#89,#0D,#24,#72,#5D,#83,#F7,#D8,#03,#47,#EC,#39,#C2,#7F,
#02,#89,#D0,#83,#F8,#00,#7F,#07,#BB,#00,#00,#00,#00,#FF,#23,#89,#C5,#8B,#43,#20,
#A3,#A2,#72,#5D,#83,#A3,#07,#73,#5D,#83,#A3,#1A,#73,#5D,#83,#A3,#29,#73,#5D,#83,
#A3,#3C,#73,#5D,#83,#A3,#48,#73,#5D,#83,#B9,#00,#00,#00,#00,#BB,#00,#00,#00,#00,
#8B,#46,#F8,#BA,#00,#00,#00,#00,#FF,#56,#FC,#29,#C8,#89,#DE,#A3,#B7,#72,#5D,#83,
#B9,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#BB,#00,#00,#00,#00,#8B,#47,#F8,#BA,#00,
#00,#00,#00,#FF,#57,#FC,#29,#C8,#89,#DF,#A3,#BD,#72,#5D,#83,#F7,#DB,#31,#C0,#81,
#E3,#03,#00,#00,#00,#29,#D9,#7D,#04,#01,#CB,#31,#C9,#89,#1D,#90,#72,#5D,#83,#88,
#CB,#80,#E3,#03,#C1,#E9,#02,#75,#08,#01,#1D,#90,#72,#5D,#83,#89,#CB,#89,#0D,#AF,
#72,#5D,#83,#89,#1D,#F7,#72,#5D,#83,#EB,#00,#B9,#00,#00,#00,#00,#85,#C9,#74,#15,
#8A,#06,#84,#C0,#74,#0A,#8A,#27,#8A,#80,#00,#00,#00,#00,#88,#07,#46,#47,#49,#75,
#EB,#81,#C1,#00,#00,#00,#00,#75,#14,#81,#C6,#00,#00,#00,#00,#81,#C7,#00,#00,#00,
#00,#4D,#75,#CB,#E9,#23,#FF,#FF,#FF,#8B,#16,#85,#D2,#74,#1D,#8B,#1F,#84,#D2,#75,
#3D,#84,#F6,#75,#47,#C1,#CA,#10,#84,#D2,#75,#51,#C1,#CB,#10,#84,#F6,#75,#5B,#C1,
#CB,#10,#89,#1F,#83,#C6,#04,#83,#C7,#04,#49,#75,#D4,#81,#C1,#00,#00,#00,#00,#74,
#B8,#8A,#06,#84,#C0,#74,#0A,#8A,#27,#8A,#80,#00,#00,#00,#00,#88,#07,#46,#47,#49,
#EB,#E9,#88,#DC,#88,#D0,#84,#F6,#8A,#98,#00,#00,#00,#00,#74,#B9,#88,#FC,#88,#F0,
#C1,#CA,#10,#8A,#B8,#00,#00,#00,#00,#84,#D2,#74,#AF,#C1,#CB,#10,#88,#D0,#88,#DC,
#84,#F6,#8A,#98,#00,#00,#00,#00,#74,#A5,#88,#F0,#88,#FC,#8A,#B8,#00,#00,#00,#00,
#C1,#CB,#10,#89,#1F,#EB,#99})
poke4(mixer_blit_proc + 2, scratch_command)
poke4(mixer_blit_proc + 10, mixer_blit_proc + 149) -- @next_command
poke4(mixer_blit_proc + 19, mixer_blit_proc + 226) -- @dest_vs
poke4(mixer_blit_proc + 61, mixer_blit_proc + 231) -- @dest_x
poke4(mixer_blit_proc + 67, mixer_blit_proc + 196) -- @source_x
poke4(mixer_blit_proc + 88, mixer_blit_proc + 191) -- @source_w
poke4(mixer_blit_proc + 93, mixer_blit_proc + 221) -- @source_w2
poke4(mixer_blit_proc + 122, mixer_blit_proc + 239) -- @dest_y
poke4(mixer_blit_proc + 128, mixer_blit_proc + 204) -- @source_y
poke4(mixer_blit_proc + 161, mixer_blit_proc + 330) -- @table1
poke4(mixer_blit_proc + 166, mixer_blit_proc + 431) -- @table2
poke4(mixer_blit_proc + 171, mixer_blit_proc + 450) -- @table3
poke4(mixer_blit_proc + 176, mixer_blit_proc + 465) -- @table4
poke4(mixer_blit_proc + 181, mixer_blit_proc + 484) -- @table5
poke4(mixer_blit_proc + 186, mixer_blit_proc + 496) -- @table6
poke4(mixer_blit_proc + 216, mixer_blit_proc + 351) -- @source_add
poke4(mixer_blit_proc + 251, mixer_blit_proc + 357) -- @dest_add
poke4(mixer_blit_proc + 275, mixer_blit_proc + 312) -- @line1
poke4(mixer_blit_proc + 291, mixer_blit_proc + 312) -- @line1
poke4(mixer_blit_proc + 299, mixer_blit_proc + 343) -- @line2
poke4(mixer_blit_proc + 305, mixer_blit_proc + 415) -- @line3

modex_blit_proc = allocate_proc( {
#60,#BB,#00,#00,#00,#00,#8B,#43,#04,#A3,#C5,#E2,#5D,#83,#8B,#7B,#14,#89,#3D,#F0,
#E2,#5D,#83,#8B,#73,#10,#8B,#46,#DC,#85,#C0,#74,#03,#89,#43,#10,#8B,#56,#E8,#8B,
#43,#18,#2B,#47,#F0,#31,#C9,#2B,#46,#E0,#7D,#08,#01,#C2,#7E,#5C,#29,#C1,#31,#C0,
#A3,#F5,#E2,#5D,#83,#89,#0D,#D3,#E2,#5D,#83,#F7,#D8,#03,#47,#E8,#39,#C2,#7F,#02,
#89,#D0,#83,#F8,#00,#7E,#3D,#A3,#CE,#E2,#5D,#83,#A3,#EB,#E2,#5D,#83,#8B,#56,#EC,
#8B,#43,#1C,#2B,#47,#F4,#31,#C9,#2B,#46,#E4,#7D,#08,#01,#C2,#7E,#1F,#29,#C1,#31,
#C0,#A3,#FD,#E2,#5D,#83,#89,#0D,#DB,#E2,#5D,#83,#F7,#D8,#03,#47,#EC,#39,#C2,#7F,
#02,#89,#D0,#83,#F8,#00,#7F,#07,#BB,#00,#00,#00,#00,#FF,#23,#89,#C5,#B9,#00,#00,
#00,#00,#BB,#00,#00,#00,#00,#8B,#46,#F8,#BA,#00,#00,#00,#00,#FF,#56,#FC,#8D,#73,
#F0,#A3,#FD,#E3,#5D,#83,#B9,#00,#00,#00,#00,#BF,#00,#00,#00,#00,#BB,#00,#00,#00,
#00,#8B,#47,#F8,#BA,#00,#00,#00,#00,#C1,#EB,#02,#FF,#57,#FC,#8D,#7B,#FC,#A3,#03,
#E4,#5D,#83,#C1,#E9,#04,#89,#0D,#2D,#E3,#5D,#83,#89,#0D,#60,#E3,#5D,#83,#89,#0D,
#94,#E3,#5D,#83,#89,#0D,#C8,#E3,#5D,#83,#EB,#00,#B9,#00,#00,#00,#00,#BA,#C4,#03,
#00,#00,#66,#B8,#02,#01,#66,#EF,#89,#F3,#89,#FA,#81,#C3,#10,#00,#00,#00,#8A,#63,
#0C,#8A,#43,#08,#C1,#E0,#10,#81,#C2,#04,#00,#00,#00,#8A,#63,#04,#8A,#03,#49,#89,
#02,#75,#E1,#B9,#00,#00,#00,#00,#BA,#C4,#03,#00,#00,#66,#B8,#02,#02,#66,#EF,#8D,
#5E,#01,#89,#FA,#81,#C3,#10,#00,#00,#00,#8A,#63,#0C,#8A,#43,#08,#C1,#E0,#10,#81,
#C2,#04,#00,#00,#00,#8A,#63,#04,#8A,#03,#49,#89,#02,#75,#E1,#B9,#00,#00,#00,#00,
#BA,#C4,#03,#00,#00,#66,#B8,#02,#04,#66,#EF,#8D,#5E,#02,#89,#FA,#81,#C3,#10,#00,
#00,#00,#8A,#63,#0C,#8A,#43,#08,#C1,#E0,#10,#81,#C2,#04,#00,#00,#00,#8A,#63,#04,
#8A,#03,#49,#89,#02,#75,#E1,#B9,#00,#00,#00,#00,#BA,#C4,#03,#00,#00,#66,#B8,#02,
#08,#66,#EF,#8D,#5E,#03,#89,#FA,#81,#C3,#10,#00,#00,#00,#8A,#63,#0C,#8A,#43,#08,
#C1,#E0,#10,#81,#C2,#04,#00,#00,#00,#8A,#63,#04,#8A,#03,#49,#89,#02,#75,#E1,#81,
#C6,#00,#00,#00,#00,#81,#C7,#00,#00,#00,#00,#4D,#0F,#85,#1E,#FF,#FF,#FF,#E9,#B1,
#FE,#FF,#FF})
poke4(modex_blit_proc + 2, scratch_command)
poke4(modex_blit_proc + 10, modex_blit_proc + 149) -- @next_command
poke4(modex_blit_proc + 19, modex_blit_proc + 192) -- @dest_vs
poke4(modex_blit_proc + 61, modex_blit_proc + 197) -- @dest_x
poke4(modex_blit_proc + 67, modex_blit_proc + 163) -- @source_x
poke4(modex_blit_proc + 88, modex_blit_proc + 158) -- @source_w
poke4(modex_blit_proc + 93, modex_blit_proc + 187) -- @source_w2
poke4(modex_blit_proc + 122, modex_blit_proc + 205) -- @dest_y
poke4(modex_blit_proc + 128, modex_blit_proc + 171) -- @source_y
poke4(modex_blit_proc + 182, modex_blit_proc + 461) -- @source_add
poke4(modex_blit_proc + 219, modex_blit_proc + 467) -- @dest_add
poke4(modex_blit_proc + 228, modex_blit_proc + 253) -- @line1
poke4(modex_blit_proc + 234, modex_blit_proc + 304) -- @line2
poke4(modex_blit_proc + 240, modex_blit_proc + 356) -- @line3
poke4(modex_blit_proc + 246, modex_blit_proc + 408) -- @line4

blit       = blit_proc
clear_blit = clear_blit_proc
mixer_blit = mixer_blit_proc
rle_blit   = rle_blit_proc

wait_retrace = allocate_proc( {
#60,#BB,#00,#00,#00,#00,#8B,#5B,#04,#BA,#DA,#03,#00,#00,#EC,#A8,#08,#75,#FB,#EC,
#A8,#08,#74,#FB,#FF,#23})
poke4(wait_retrace + 2, scratch_command)

wait_retrace = wait_retrace


-----------------------------------------------------------------
-- PALETTE STUFF ------------------------------------------------
-----------------------------------------------------------------

global procedure set_all_palette(sequence pal)
--> set the 256 color palette with {#RRGGBB,...}
--> only useful in 8-bit color modes
--> you probably should call make_rgb_map() immediately afterward 
    out_byte(968, 0)
    for i = 1 to length(pal) do
        out_byte(969, floor(pal[i] / {262144, 1024, 4}))
    end for
end procedure

global function get_all_palette()
--> get the 256 color palette with {#RRGGBB,...}
--> only useful in 8-bit color modes
    sequence pal
    out_byte(967, 0)
    pal = repeat(0, 256)
    for i = 1 to 256 do
        pal[i] = 262144 * in_byte(969)
               + 1024 * in_byte(969)
               + 4*in_byte(969)
    end for
    return pal
end function

------------------------------------------------------------------------

global procedure draw_line(atom vs, object color, sequence p)
--> draws a line (or polygon) on virtual screen vs
--> p is a minimum length 2 sequence of points: {{x0,y0},{x1,y1},...,{xn,yn}}
--> See also: color_convert, pixel, get_pixel
    integer x, y, w, h, dx, dy, count, point
    atom scr_addr, addr, bytes_per_line

    if vs = screen then scr_addr = video_mem
    else scr_addr = vs end if
    bytes_per_line = peek4u(vs + screen_actual_width)

    if not raw_mode then
        if bits_per_pixel = 32 then
            color = floor(color / {1,65536,256,1})
        elsif bits_per_pixel = 24 then
            color = floor(color / {1,256,65536})
        elsif bits_per_pixel = 15 then
            color = { and_bits(floor(color / 8),#1F) + and_bits(color / 64, #E0),
                and_bits(floor(color / 16384),#3) + and_bits(color / 131072, #7C)}
        elsif bits_per_pixel = 16 then
            color = { and_bits(floor(color / 8),#1F) + and_bits(color / 32, #E0),
                and_bits(floor(color / 8192),#7) + and_bits(color / 65536, #F8)}
        else
            color = peek(rgb_map + 
                and_bits(floor(color / 512), #7C00) +
                and_bits(floor(color / 64), #3E0) +
                and_bits(floor(color / 8), #1F))
        end if
    end if

    point = 1

    while point < length(p) do
        x = p[point][1]
        y = p[point][2]
        addr = scr_addr + y*bytes_per_line + x*bytes_per_pixel
        point = point + 1
        dx = p[point][1] - x
        dy = p[point][2] - y
        if dx < 0 then
            w = -dx+1
        else
            w = dx+1
        end if
        dx = (dx>0)-(dx<0)
        if dy < 0 then
            h = -dy+1
        else
            h = dy+1
        end if
        dy = (dy>0)-(dy<0)
    
        dy = dy * bytes_per_line
        dx = dx * bytes_per_pixel
    
        if w + h = 2 then
            pixel(color, p[1])
        elsif w >= h then
            count = floor(h/2)
            for i = 1 to w do
                poke(addr, color)
                addr = addr + dx
                count = count + h
                if count >= w then
                    addr = addr + dy
                    count = count - w
                end if
            end for
        else
            count = floor(w/2)
            for i = 1 to h do
                poke(addr, color)
                addr = addr + dy
                count = count + w
                if count >= h then
                    addr = addr + dx
                    count = count - h
                end if
            end for
        end if
    end while
end procedure



------------------------------------------------------------------------
-- FONTS based on Jiri Babor's fonts
------------------------------------------------------------------------

global constant
    BOLD = 1,
    ITALIC = 2, 
    UNDERLINE = 4,
    DIMMED = 8

sequence char, char_widths
integer char_height, baseline_height, text_x, text_y
char = {}
text_x = 0
text_y = 0

sequence all_fonts
all_fonts = {}

global procedure rom_font(integer width, integer height)
--> select a rom font
-->  width may be 8 or 9 
-->  height may be 8, 14 or 16
--> Note: width 9 doesn't work with height 8
    sequence regs, charaddr
    integer font_buf, bits, on, mask, x

    regs = repeat(0, 10)
    regs[REG_AX] = #1130
    if width = 8 and height = 8 then
        regs[REG_BX] = #0300  
    elsif (width = 8 or width = 9) and height = 14 then
        regs[REG_BX] = #0200
    elsif (width = 8 or width = 9) and height = 16 then
        regs[REG_BX] = #0600
    else
        return
    end if

    regs = dos_interrupt(#10, regs)
    font_buf = regs[REG_ES] * #10 + regs[REG_BP]
    charaddr = repeat(0, 256)
    for c = 1 to 256 do
        charaddr[c] = c * height
    end for
    charaddr = charaddr + font_buf

    char_widths = repeat(width, 256)
    char_height = height
    baseline_height = height

    if width = 9 then
        if height = 14 then
            regs[REG_BX] = #0500
        elsif height = 16 then
            regs[REG_BX] = #0700
        end if
        regs = dos_interrupt(#10, regs)
        font_buf = regs[REG_ES] * #10 + regs[REG_BP]
        while peek(font_buf) do
            charaddr[peek(font_buf)] = font_buf + 1
            font_buf = font_buf + 1 + height
        end while
    end if
            
    char = repeat({}, 256)
    for newAscii = 1 to 256 do
        for y = 1 to height do
            bits = peek(charaddr[newAscii] + y-1)
            on = 0  x = 8  mask = 1
            while mask < bits do
                if and_bits(bits, mask) then
                    if on then
                        char[newAscii][on] = char[newAscii][on] + 1
                        char[newAscii][on-2] = x
                    else
                        char[newAscii] = char[newAscii] & {x,y,0}
                        on = length(char[newAscii])
                    end if
                elsif on then
                    on = 0
                end if
                x = x - 1
                mask = mask * 2
            end while
        end for
    end for
end procedure

global procedure add_pitch(integer pitch)
--> add pitch to horizontal font character spacing
    char_widths = char_widths + pitch
end procedure


global function build_text_image(sequence chars, integer foreground, object background, integer attributes)
--> returns a 2-d image sequence of chars
--> foreground and background are colors, same format as pixel, get_pixel, etc.
--> attributes is an or'ed value of BOLD, ITALIC, UNDERLINE, or DIMMED
    integer x, y, l, dx, fbold
    sequence ch, image, ioffset
    
    if not length(char) then
        rom_font(8,16)        
    end if
    
    ioffset = repeat(0, char_height)
    if and_bits(attributes, ITALIC) then
        for i = 1 to char_height do
            ioffset[i] = floor((char_height - i) * 0.25 )
        end for
    end if

    fbold = and_bits(attributes, BOLD) != 0
    l = 0
    for i = 1 to length(chars) do
        l = l + char_widths[chars[i]] + fbold
    end for
    image = repeat(repeat(background, l+ioffset[1]), char_height)

    if and_bits(attributes, UNDERLINE) then
        x = 1 + ioffset[baseline_height]
        image[baseline_height][x..x+l-1] = foreground
    end if
    
    if and_bits(attributes, DIMMED) then
        dx = 0
        for c = 1 to length(chars) do
            ch = char[chars[c]]
            for i = 1 to length(ch)-2 by 3 do
                y = ch[i+1]
                x = ch[i] + dx + ioffset[y]
                for xx = x+and_bits(x+y,1) to x+ch[i+2]+fbold by 2 do
                    image[y][xx] = foreground
                end for
            end for
            dx = dx + char_widths[chars[c]] + fbold
        end for
    else
        dx = 0
        for c = 1 to length(chars) do
            ch = char[chars[c]]
            for i = 1 to length(ch)-2 by 3 do
                y = ch[i+1]
                x = ch[i] + dx + ioffset[y]
                image[y][x..x+ch[i+2]+fbold] = foreground
            end for
            dx = dx + char_widths[chars[c]] + fbold
        end for
    end if
    return image
end function

global procedure draw_text(sequence pos, sequence chars, integer foreground, integer background, integer attributes)
--> write chars on the screen at pos
--> attributes is an or'ed value of BOLD, ITALIC, UNDERLINE, or DIMMED
--> foreground and background are colors, same format as pixel, get_pixel, etc.
--> background may be -1 for transparent
--> Note: to write to a virtual screen, use
-->       display_image({x,y,vs}, build_text_image(...))
    integer x, y, l, dx, fbold
    sequence ch, image, ioffset

    if not length(char) then
        rom_font(8,16)        
    end if

    ioffset = repeat(0, char_height)
    if and_bits(attributes, ITALIC) then
        for i = 1 to char_height do
            ioffset[i] = floor((char_height - i) * 0.25 ) 
        end for
    end if

    fbold = and_bits(attributes, BOLD) != 0
    l = 0
    for i = 1 to length(chars) do
        l = l + char_widths[chars[i]] + fbold
    end for
    image = repeat(repeat(background, l), char_height)
    if background = -1 then
        for i = 1 to char_height do
            image[i] = get_pixel((pos + {ioffset[i],i-1}) & l)
        end for
    end if

    if and_bits(attributes, UNDERLINE) then
        image[baseline_height] = repeat(foreground, l)
    end if
    
    if and_bits(attributes, DIMMED) then
        dx = 0
        for c = 1 to length(chars) do
            ch = char[chars[c]]
            for i = 1 to length(ch)-2 by 3 do
                y = ch[i+1]
                x = ch[i] + dx
                for xx = x+and_bits(x+y,1) to x+ch[i+2]+fbold by 2 do
                    image[y][xx] = foreground
                end for
            end for
            dx = dx + char_widths[chars[c]] + fbold
        end for
    else
        dx = 0
        for c = 1 to length(chars) do
            ch = char[chars[c]]
            for i = 1 to length(ch)-2 by 3 do
                y = ch[i+1]
                x = ch[i] + dx
                image[y][x..x+ch[i+2]+fbold] = foreground
            end for
            dx = dx + char_widths[chars[c]] + fbold
        end for
    end if
    for i = 1 to char_height do
        pixel(image[i], pos + {ioffset[i],i-1})
    end for
    text_x = text_x + dx
end procedure


global procedure fselect(integer f)
--> select a loaded font using the value returned by fload
    char = all_fonts[f][2]
    char_widths = all_fonts[f][3]
    char_height = all_fonts[f][4]
    baseline_height = all_fonts[f][5]
end procedure


global function fload(sequence file_name)
--> load font file into memory and make it the current selection
    integer fn,bw,fc,lc, version, on
    sequence cs, bits
    -- if font is already loaded, do not do it again, just select it
    file_name=file_name + ('A'-'a')*((file_name>='a') and (file_name<='z'))
    for n = 1 to length(all_fonts) do
        if compare(all_fonts[n][1], file_name) = 0 then
            fselect(n)
            return n
        end if
    end for

        fn=open(file_name, "rb")
        if fn=-1 then
            puts(1, "Font load error: " & file_name & " not found !\n")
            abort(1) 
        end if
        fc=getc(fn)
        lc=getc(fn)
        bw=getc(fn)
        char_height=getc(fn)
        version=getc(fn)
        baseline_height=getc(fn)                  -- baseline height
        if baseline_height=0 then baseline_height=char_height end if    -- pre version 3 fonts
        char_widths=repeat(0, lc)
        char=repeat({}, lc)
        for i=fc to lc do 
            if i then 
                char_widths[i] = getc(fn) 
            else
                on = getc(fn)  
            end if 
        end for
        for i=fc to lc do
            cs={}
            for row=1 to char_height do
                bits={}
                for col=1 to bw do 
                    bits=bits & and_bits(getc(fn),{128,64,32,16,8,4,2,1})
                end for
                on = 0  
                for x = 1 to length(bits) do
                    if bits[x] then
                        if on then
                            cs[on] = cs[on] + 1
                        else
                            cs = cs & {x,row,0}
                            on = length(cs)
                        end if
                    elsif on then
                        on = 0
                    end if
                end for       
            end for
            if i then
                char[i] = cs
            end if
        end for
        close(fn)
        all_fonts=append(all_fonts, {file_name, char, char_widths, char_height, baseline_height})
    return length(all_fonts)   -- make the last loaded font current

end function  -- fload



----------------------------------------------------------------------------
-- stupid graphics.e and puts,printf emulation -----------------------------
----------------------------------------------------------------------------

procedure oldputs(integer fn, object data)
    puts(fn, data)
end procedure

integer fore_color, back_color
fore_color = BRIGHT_WHITE
back_color = BLACK

global procedure clear_screen()
--> Clears the screen to the color set by bg_color and moves cursor to (0,0)
    text_x = 0
    text_y = 0
    display_image({0,0}, repeat(repeat(back_color, SCREEN_W), SCREEN_H))
end procedure

global procedure fg_color(integer color)
--> Set the foreground color (in the format #RRGGBB)
    fore_color = color        
end procedure

global procedure bg_color(integer color)
--> Set the background color (in the format #RRGGBB)
    back_color = color        
end procedure

global procedure position(atom row, atom column)
--> Positions the (invisible) text cursor at the indicated row and column.
    if not length(char) then
        rom_font(8,16)        
    end if
    text_x = floor(column * char_widths[' '])
    text_y = floor(row * char_height)
end procedure

global function get_position()
--> Returns the position of the text cursor as {row, column}
    if not length(char) then
        rom_font(8,16)        
    end if
    return {floor(text_y / char_height), floor(text_x / char_widths[' '])}
end function

global procedure puts(integer fn, object data)
--> puts redefined to take advantage of internal font system
--> Note: does not scroll or wrap if the cursor moves off the screen.
--> See also: printf
    integer cr
    if fn != 1 or mode_type = 0 then
        oldputs(fn, data)
    else
        if atom(data) then
            data = sprintf("%g", {data})
        end if
        while 1 do
            if find('\n', data) then
                cr = find('\n', data)
                draw_text({text_x, text_y}, data[1..cr-1], fore_color, back_color, 0)
                data = data[cr+1..length(data)]
                text_x = 0
                text_y = text_y + 16
            elsif find('\r', data) then
                cr = find('\r', data)
                draw_text({text_x, text_y}, data[1..cr-1], fore_color, back_color, 0)
                data = data[cr+1..length(data)]
                text_x = 0
            else
                draw_text({text_x, text_y}, data, fore_color, back_color, 0)
                exit
            end if
        end while
    end if
end procedure

global procedure printf(integer fn, sequence fmt, object data)
--> printf redefined to take advantage of internal font system
--> See also: puts
    puts(fn, sprintf(fmt, data))
end procedure

