--Working expression parser

global constant debug = 0

--procedure text_mode() end procedure --uncomment this if vesa20.e absent

sequence fp_precision
include get.e
include asm.e
--include graphics.e --for text_color()

include strnum.e

with trace
without warning
without type_check

-----------------------------------------------------------------------------

--Stack functions
sequence stack
--st_push : push object onto stack
procedure st_push(object item)
  stack &= {item}
end procedure
--st_pop : pop object off stack
function st_pop()
  object item
  item = stack[length(stack)]
  stack = stack[1..length(stack) - 1]
  return item
end function  
--st_empty : is stack empty?
function st_empty()
  return length(stack) = 0
end function  
--st_top : returns top object without popping
function st_top()
  return stack[length(stack)]
end function  

-----------------------------------------------------------------------------
--Support routines
--is_num : confirms a character is recognisably part of a number
function is_num(integer char)
  return (char >= '0' and char <= '9') or find(char, ".")
end function
--numeric : confirms a string indeed contains just numbers
function numeric(sequence string)
--  puts(1, " "&string&":")
  for index = 1 to length(string) do
    if sequence(string[index]) or not(is_num(string[index])) then return 0 end if
  end for
  return 1
end function    
--is_white : confirms a character is whitespace
function is_white(integer char)
  return find(char, {8, 13, 32})
end function  
--strip : remove whitespace from string
function strip(sequence string)
  integer index
  index = 1
  while index <= length(string) do
    if is_white(string[index]) then
      string = string[1..index - 1] & string[index + 1..length(string)]
    else
      index += 1
    end if
  end while
  return string
end function      
--error : halt (suddenly) on any errors
procedure error(sequence msg)
  puts(1, " --ERROR--\n" & msg & "!\nCannot continue...\n")
  abort(1)
end procedure  

-----------------------------------------------------------------------------
--Math routines
global constant 
         pi = 3.141592653589753,
         natural = 2.718281828459,              --e
         l10t = log(10),
         log_nat = log(natural) / l10t,      --log(e)
         l2t = 3.321928094887362,  --log2(10)
         l2e = 1.442695040888964,  --log2(e)
         lg2 = 0.3010299956639812,  --log10(2)
         ln2 = 0.6931471805599453   --loge(2)
         
function sgn(object r) return (r >= 0) - (r < 0) end function

function abs(object r) return sgn(r) * r end function

--reduce them name conflicts
function my_arcsin(atom r)                --sin^-1
  if abs(r) > 1 then r = sgn(r) end if        -- real
  if r = 0 then return 0
  elsif abs(r) = 1 then return (pi / 2) * sgn(r)
  else return arctan(r / sqrt(1 - r * r)) end if
end function
function my_arccos(atom r)                --cos^-1
  if abs(r) > 1 then r = 1 end if             -- real
  if r = 0 then return pi / 2
  elsif abs(r) = 1 then return 0
  else return arctan(sqrt(1 - r * r) / r) end if
end function

global function arctan2(object x, object y)   --mod. arctan  :sequence ok
  object a                                    -- real
  x = (x = 0) * 0.00000001 + x
  a = arctan(y / x)
  a = a + ((x < 0 and y >= 0) * 3.14159625)
  a = a - ((x < 0 and y < 0) * 3.14159625)
  return a
end function
                                              

-----------------------------------------------------------------------------
--Core routines

--globals
constant NAME = 1, TYPE = 2, VALUE = 3, ADDRESS = 4, REFERENCES=5
constant OPERATOR  = 0, 
         CONSTANT  = 1,
         VARIABLE  = 2, 
         FUNCTION  = 3, 
         OPERAND   = 4, 
         ASSIGN    = 5,
         STACKPOP  = 6,
         FLUSH     = 7,
         ERROR     = -1,
         INSTK     = 1,
         INCOM     = 2,
         DATA_SEG  = 3

--Variable handler preamble

sequence variable_types, type_names, special_vars
special_vars = {"CONTROL_WORD"}
type_names={}

-----------------------------------------------------------------------------
--Formula Parser


constant operators = {      '^', '*', '/', '+', '-', '(', '~', ',', '=', ':'},
         opt_trans = {        1,   0,   1,   0,   1},
         pri_list  = {{  0,   3,   2,   2,   1,   1,   0,   0,   0,   0},
                      {  4,   4,   2,   2,   1,   1,   4,   4,   1,   0}}
--priority : returns operational priority of operator item
function priority(sequence item, integer in) 
  return pri_list[in][find(item[1], operators) + 1]
end function
--post_flush : translates items into postfix output
function post_flush(sequence item)
  if item[1] = '=' then                         --Assignment operator?
    item = {{ASSIGN, item[2]}}                  -- translate to output
  elsif item[1] = '^' then                            --Symbolic operator?
    item = {{FUNCTION, "power", item[3]}}       -- translate to output
  elsif item[1] = ',' then                            --Symbolic operator?
    item = {{FLUSH, ""}}       -- translate to output
  elsif item[1] != FUNCTION then                            --Symbolic operator?
    item = {{OPERATOR, item[1], item[3]}}       -- translate to output
  else item = {item}
  end if
  return item                                   --return output
end function
--stack_flush : empties the stack and outputs as postfix
function stack_flush()
  object post, item
  post = {}
  while not st_empty() do                   --Operators still in stack?
    item = st_top()                         --check next item
    if item[1] = '(' then                   --Open brace?
      return 0                              --Error, there was no close
    end if
    post &= post_flush(st_pop())            --flush item into output
  end while
  return post
end function  
--reorder param : required to reverse the parameter order for functions
function reorder_param(sequence infix, integer index)
  integer brace
  sequence comma, new
  brace = 1  comma = {index}
  for search = index to length(infix) do
    if infix[search] = '(' then brace -= 1 
    elsif infix[search] = ')' then brace += 1
    elsif infix[search] = ',' then comma &= {search}
    end if
    if brace = 1 then brace = search comma &= {search} exit end if
  end for    
  if brace < 1 then return {-1, "Parameter error: Function braces not closed"} end if
  if length(comma) < 3 then return {0, infix} end if
  for count = 1 to length(comma) - 1 do
    st_push(infix[comma[count] + 1..comma[count + 1] - 1])
  end for
  new = ""
  for count = 1 to length(comma) - 1 do
    new &= st_pop() & ","
  end for
  infix = infix[1..index] & new[1..length(new) - 1] & infix[brace..length(infix)]  
--puts(1, infix)
  return {0, infix}
end function
--infix_to_postfix : first pass of expression parser
--  Takes a mathematical expression in the form described in readme.txt
--  and converts it to extended postfix format, i.e: "a b c / + x ="  for 
--  use with postfix_assembler().  Output is a sequence in the form:
--xx  {ERROR, OUTPUT} where if ERROR < 0 then OUTPUT is a string describing
--xx  some error, else OUTPUT contains a sequence in the form:
--xx  {{TYPE, DATA}, {TYPE, DATA}, ..}  where each TYPE is one of the 
--xx  constants operator, literal or variable and DATA is either the name or
--xx  character being identified.  Error checking of postfix is not necessary 
--  if it is to be used with postfix_assembler() as errors get passed on. 
global function infix_to_postfix(sequence infix)
  object item, postfix, temp
  integer index, len, numargs, last_type, last_char
  index = 0 postfix = {} numargs = 0
  last_type = OPERATOR                               --init last type flag
  item = {0, {}}
  stack = {}
  infix = strip(infix)                          --strip whitespace
  while index < length(infix) do                          --scan input
    index += 1                                  --next character
    last_char = item[1]                               --update last char buffer
    item = {infix[index], {}}                   --get next char
    if item[1] = ':' then                       --Seperator?
      --Flush delayed operators on encountering seperator ':'
      temp = stack_flush()                      --flush pending operators
      if atom(temp) then                        --Nesting error?
        return {-1, "Formula syntax error: ')' expected"}
      end if
      postfix &= temp                           --cat items to output
    elsif find(item[1], operators) then         --Is an operator?
      --Find operators, arrange by priority and delay for later
      if item[1] = ',' then
        while (not st_empty()) do
          temp = st_top()
          if temp[1] = ',' or temp[1] = '(' then exit end if
          postfix &= post_flush(st_pop())
        end while  
--?{postfix, index}
--?stack
        st_push(item) 
        postfix &= post_flush(item)
        last_type = OPERATOR
      else
        --Is unary minus?
        if item[1] = '-' and (last_type = OPERATOR) then 
          item[1] = '~'                           --substitute specific symbol
          numargs = 1
        else numargs = 2 end if  
        while (not st_empty())                    --flush higher priority ops
              and (priority(st_top(), INSTK) >= priority(item, INCOM)) do
          postfix &= post_flush(st_pop())         --add operators to output
        end while
        st_push(item & numargs)                   --push current operator
        last_type = OPERATOR                           --set last flag
        numargs = 0
      end if
    elsif item[1] = ')' then                    --Closing brace?
      --Unnest braces and check for associated functions
      temp = st_top()
      numargs = 0
      while 1 do                                --start unnesting
        if st_empty() then                      --Not found..
          return {-1, "Formula syntax error: '(' expected"}
        end if  
        item = st_pop()                         --pop item off stack
        if item[1] = '(' then                   --Corresponding open brace?
          if not st_empty() then                --Stack not empty?
            temp = st_top()                     --look at stack
            if temp[1] = FUNCTION then          --and function next on stack?
              temp = st_pop()
              temp[3] += numargs
              postfix &= post_flush(temp)--add function to output
            end if
          end if
          numargs = 0
          exit                                  --all unnested, exit while
        elsif item[1] = ',' then
          numargs += 1
        else
          postfix &= post_flush(item)             --add operators to output
        end if
      end while
      last_type = FUNCTION                            --brace behaves this way..
    else
      --Read function name, variable name or literal number value
      while 1 do                                --start scanning input
        item[2] &= infix[index]                 --cat input char to item 
        if index = length(infix)                --Scanned off end, or next
           or find(infix[index + 1], operators) -- is operator or closing
           or infix[index + 1] = ')' then       -- brace => end of scan.
          if index < length(infix) - 1          --Not scanned of end, and
             and infix[index + 1] = '(' then    --Next is '(' => Function
            --Function() found, delay for later
            item[1] = FUNCTION                      --function flag
            if infix[index + 2] != ')' then
              infix = reorder_param(infix, index + 1)
              if infix[1] = ERROR then return infix[2] end if
              infix = infix[2]
            end if  
            st_push(item & (infix[index + 2] != ')')) --ok, push item with numargs
            last_type = FUNCTION                     --set last flag
          elsif index < length(infix)           --Not scanned of end, and
             and infix[index + 1] = '=' then    --Next is '=' => Assign
            --Assign= found, delay for later
            item[1] = '='                       --assignment flag
            index += 1                          --move past '=' operator!
            st_push(item & 1)                   --ok, push item with numargs
            last_type = OPERATOR                     --set last flag
          else                                  -- => Variable or literal
            --Operand found, drop straight into output stream

--            puts(1, " " & item[2]) print(1,numeric(item[2]))

            if not numeric(item[2]) then           --Not a literal..
              --Operand is variable name
              item[1] = VARIABLE                --flag a variable
              postfix &= {item}                 --ok, add directly to output
            else                                --Literal value
              --Operand is literal value
              item = value(item[2])                --get value
              if item[1] = GET_FAIL then        --Cant read..
                return {-1, "Formula parsing error: Cannot read constant value '" & item & "'"}
              end if
              --find any unary negatives and apply now
              if last_type = OPERATOR then
                temp = st_top()
                if temp[1] = '~' then
                  temp = st_pop()
                  item[2] = -item[2]
                end if
              end if    
              item[1] = CONSTANT                 --flag a CONSTANT
              postfix &= {item}                 --ok, add to output
            end if
            last_type = OPERAND                      --set last flag
          end if
          exit                                  --end of scan, exit while
        end if
        index += 1                              --move to next character
      end while
    end if
  end while
  temp = stack_flush()                          --flush pending operators
  if atom(temp) then                            --Nesting error?
    return {-1, "Formula syntax error: ')' expected"}
  end if
  postfix &= temp                               --cat items to output
  return {0, postfix}                           --return postfix output
end function

-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
include I80x87.e
-----------------------------------------------------------------------------
--Function and operator tables

--Real type number operator code
constant operator_table = {{}, o_mul1, o_div1, o_add1, o_sub1}

--function table
constant function_table = {{"neg", {1, 1}, f_neg1},         --neg()
                           {"sqr", {1, 1}, f_sqr1},   --sqr()
                           {"sqrt", {1, 1}, f_sqrt1},       --sqrt()
                           {"exp", {1, 1}, f_exp1},         --exp()
                           {"log", {1, 1}, f_log1},        --log()
                           {"ln", {1, 1},  f_ln1},         --ln()
                           {"power", {2, 1}, f_pwr1},       --power()
                           {"sgn", {1, 1}, f_sgn1},         --sgn()
                           {"abs", {1, 1}, f_abs1},         --abs()
                           {"sin", {1, 1}, f_sin1},         --sin()
                           {"cos", {1, 1}, f_cos1},         --cos()
                           {"sincos", {1, 2}, f_sincos1},   --sincos()
                           {"tan", {1, 1}, f_tan1},         --tan()
                           {"sec", {1, 1}, f_sec1},         --sec()
                           {"cosec", {1, 1}, f_cosec1},         --cosec()
                           {"cotan", {1, 1}, f_cotan1},      --cotan()
                           {"arcsin", {1, 1}, f_arcsin1},  --arcsin()
                           {"arccos", {1, 1}, f_arccos1},       --arccos()
                           {"arctan", {1, 1}, f_arctan1},    --arctan()
                           {"arctan2", {2, 1}, f_arctan21},       --arctan2()
                           {"sinh", {1, 1}, f_sinh1},       --sinh()
                           {"cosh", {1, 1}, f_cosh1},       --cosh()
                           {"tanh", {1, 1}, f_tanh1},       --tanh()
                           {"cotanh", {1, 1}, f_cotanh1},     --cotanh()
                           {"mod", {2, 1}, f_mod1},         --mod()
                           {"rnd", {0, 1}, f_rnd1},         --rnd()
                           {"int", {1, 1}, f_int1}}         --int()

sequence function_list
function_list={}
for count = 1 to length(function_table) do
  function_list &= {function_table[count][1]}
end for

-----------------------------------------------------------------------------
--Postfix evaluator

--actual functions used by the evaluator
function fn_neg(atom n) return -n end function  
function fn_sqr(atom n) return n * n end function  
function fn_sqrt(atom n) return sqrt(n) end function  
function fn_exp(atom n) return power(natural, n) end function  
function fn_log(atom n) return log(n) / l10t end function  
function fn_ln(atom n) return log(n) end function  
function fn_power(atom b, atom a) return power(a, b) end function  
function fn_sgn(atom n) return sgn(n) end function  
function fn_abs(atom n) return abs(n) end function  
function fn_sin(atom n) return sin(n) end function  
function fn_cos(atom n) return cos(n) end function  
function fn_sincos(atom n) return {sin(n), cos(n)} end function  
function fn_tan(atom n) return tan(n) end function  
function fn_sec(atom n) return 1 / cos(n) end function  
function fn_cosec(atom n) return 1 / sin(n) end function  
function fn_cotan(atom n) return 1 / tan(n) end function  
function fn_arcsin(atom n) return my_arcsin(n) end function  
function fn_arccos(atom n) return my_arccos(n) end function  
function fn_arctan(atom n) return arctan(n) end function  
function fn_arctan2(atom b, atom a) return arctan2(a, b) end function  
function fn_sinh(atom n) return (fn_exp(n) - fn_exp(-n)) / 2 end function  
function fn_cosh(atom n) return (fn_exp(n) + fn_exp(-n)) / 2 end function  
function fn_tanh(atom n) return fn_sinh(n) / fn_cosh(n) end function  
function fn_cotanh(atom n) return fn_cosh(n) / fn_sinh(n) end function  
function fn_mod(atom b, atom a) return remainder(a, b) end function
function fn_rnd() return (rand(1073741823) / 1073741823) * 2 - 1 end function
function fn_int(atom n) return floor(n) end function
--Turn postfix into results!
function postfix_eval(sequence postfix, sequence vars)
  object item, temp, var_names, fn, ft, results
  if postfix[1] = -1 then return postfix end if   --Reject bad postfix
  postfix = postfix[2]
  var_names = {}  results = {}
  for count = 1 to length(vars[VARIABLE]) do
    var_names &= {vars[VARIABLE][count][NAME]}
  end for
  for count = 1 to length(postfix) do
    item = postfix[count]
    if item[1] = CONSTANT then                 --Constant value..
      st_push(item[2])                                     --push it
    elsif item[1] = ASSIGN then             --Assign..
      temp = find(item[2], var_names)                        --look it up
      if temp then                                      --Found?
        results &= st_pop()
        vars[VARIABLE][temp][VALUE] = results[length(results)]
      else  
        return {-1, "Evaluation error: Variable '" & item[2] & "' not declared"}   --no, error
      end if
    elsif item[1] = VARIABLE then             --Variable..
      temp = find(item[2], var_names)                        --look it up
      if temp then                                      --Found?
        st_push(vars[VARIABLE][temp][VALUE])                          --yes, push it
      else  
        return {-1, "Evaluation error: Variable '" & item[2] & "' not declared"}   --no, error
      end if
    elsif item[1] = FUNCTION then                        --Named function
      fn = routine_id("fn_" & item[2])
      if fn != -1 then                 --Look it up..
        ft = find(item[2], function_list)
        if function_table[ft][2][1] = 0 then
          if function_table[ft][2][2] = 1 then
            st_push(call_func(fn, {})) --execute
          else
            temp = call_func(fn, {}) --execute
            st_push(temp[1])  st_push(temp[2])
          end if  
        elsif function_table[ft][2][1] = 1 then
          if function_table[ft][2][2] = 1 then
            st_push(call_func(fn, {st_pop()})) --execute
          else
            temp = call_func(fn, {st_pop()}) --execute
            st_push(temp[1])  st_push(temp[2])
          end if  
        elsif function_table[ft][2][1] = 2 then
          if function_table[ft][2][2] = 1 then
            temp = st_pop()
            st_push(call_func(fn, {st_pop(), temp})) --execute
          else
            temp = st_pop()
            st_push(call_func(fn, {st_pop(), temp})) --execute
            st_push(temp[1])  st_push(temp[2])
          end if
        end if    
      else
        return {-1, "Evaluatin Error: Unknown function '" & item[2] & "()' "}     --not found, error
      end if
    elsif item[1] = OPERATOR then
      if item[2] = '~' then                               --Unary negative..
        st_push( -st_pop())                           
      elsif item[2] = '+' then                               --Add..
        temp = st_pop()                                   --(watch order)
        st_push(st_pop() + temp)
      elsif item[2] = '-' then                               --Sub..
        temp = st_pop()
        st_push(st_pop() - temp)
      elsif item[2] = '*' then                               --Mul..
        temp = st_pop()
        st_push(st_pop() * temp)
      elsif item[2] = '/' then                               --Div..
        temp = st_pop()
        st_push(st_pop() / temp)
      elsif item[2] = '^' then                               --Pwr..
        temp = st_pop()
        st_push(power(st_pop(), temp))
      else ?item
      end if
    elsif item[1] != FLUSH then ?item
    end if
  end for
  if debug then return {{}, results}     --return dummy for debugging 
  else return {vars, results} end if    --else update variable values
end function      

-----------------------------------------------------------------------------
--Variable declaraton parser and variable tools

global procedure set_fp_precision(integer bits)
  type_names = {}
  if bits = 32 or bits = 4 then fp_precision = {"dword ptr", 4}
  elsif bits = 64 or bits = 8 then fp_precision = {"qword ptr", 8}
  elsif bits = 80 or bits = 10 then fp_precision = {"qword ptr", 10}
  else 
    text_mode() 
    puts(1,"Unrecognised precision specifier in set_fp_precision()\n") 
    abort(0)
  end if  
  variable_types = {{"constant", {1, "fixed", 0, ""}}, --constant must be 1st
                    {"byte", {1,"byte ptr", 1, "_byte"}},
                    {"word", {1, "word ptr", 2, "_word"}},
                    {"int", {1, "dword ptr", 4, "_dword"}},
                    {"fixpt", {1, "dword ptr", 4, "_fixpt"}},
                    {"real", {1} & {1, 1} * fp_precision & {"_real"}},
                    {"float4", {1, "dword ptr", 4, "_float4"}},
                    {"float8", {1, "qword ptr", 8, "_float8"}},
                    {"float10", {1, "tbyte ptr", 10, "_float10"}},
                    {"complex", {2} & {1, 2} * fp_precision & {"_complex"}},
                    {"quaternion", {4} & {1, 4} * fp_precision & {"_quat"}}}
  if length(type_names) = 0 then
    for count = 1 to length(variable_types) do
      type_names &= {variable_types[count][1]}
    end for
  end if
end procedure

function find_variables_by_type(sequence list)
  integer char, type_id, index, var_flag
  sequence tag, name, vars, val
  type_id = 0 tag = {} vars = {} val = {} index = 1 var_flag = 0
  --add trailing whitespace to overcome shortcomings
--  list &= " "
  --scan list
  while index <= length(list) do
    char = list[index]
    if not(is_white(char)) or var_flag then
      if not (type_id or var_flag or 
              index >= length(list) or 
              find(char, ",=:\n")) then 
        tag &= char
        if is_white(list[index + 1]) then
          type_id = find(tag, type_names)
          if type_id = 0 then
            return {-1, "Declaration Error : Type '" & tag & "' not known\n"}
          end if
          tag = {} name = {}
        end if
      elsif type_id then
        if find(char, ",=:\n") then 
          return {-1, "Declaration Error : Missing variable name\n"}
        elsif find(char, "()*/+-^") then
          return {-1, "Declaration Error : Illegal characters in variable name\n"}
        end if  
        name &= char
        if index = length(list) or
           is_white(list[index + 1]) or
           find(list[index + 1], ",=:\n") then
          --vars &= {name, type, value, address, references} 
          vars &= {{name, type_id, 0, 0, 0}} 
          type_id = 0  name = {}  val = {}
        end if
      elsif var_flag then
        if find(char, ",:\n") and length(val) = 0 then 
          return {-1, "Definition Error : Missing literal value for '" & vars[length(vars)][NAME] & "'\n"}
        end if  
        if index = length(list) or
           (is_white(char) and length(val) > 0) or
           find(char, ",=:\n") then
          if index = length(list) then val &= char end if
          val = postfix_eval(infix_to_postfix(
                vars[length(vars)][NAME] & " = " & val), {{}, vars, 0})
          if atom(val[1]) then return val end if
          if debug then vars[length(vars)][VALUE] = val[2]
          else vars = val[1][2] end if
          var_flag = 0  name = {}
        end if
        val &= char
      end if
      if char = ',' then type_id = vars[length(vars)][2] end if
      if char = '=' then var_flag = 1 end if
    end if
    index += 1
  end while
  return vars
end function 

global function find_var_index(sequence vars, object val, integer field)
  integer found
  if atom(vars[1]) then return vars end if
  found = 0
  for count = 1 to length(vars) do
    if compare(val, vars[count][field]) = 0 then
      found = count
    end if  
  end for
  return found
end function  

global function read_variables(sequence list)
  sequence item, vars
  vars = fpu_constants
  list = find_variables_by_type(list)
  if length(list) > 0 and atom(list[1]) then return list end if
  for index = 1 to length(list) do
    item = list[index]
    for count = 1 to length(special_vars) do
      if compare(item[NAME], special_vars[count]) = 0 then
        item[REFERENCES] += 1
      end if    
    end for    
    if find_var_index(vars[VARIABLE], item[NAME], NAME) then
      return {-1, "Repetition Error: multiple declarations of variable '" & item[NAME] & "'"}
    end if
    vars[VARIABLE] &= {item}
  end for
  return vars
end function  

procedure poke_var(atom data_seg, sequence var)
  call_proc(routine_id("poke" & variable_types[var[TYPE]][2][4]), 
                        {var[ADDRESS] + data_seg, var[VALUE]})
end procedure

function peek_var(atom data_seg, sequence var)
  return call_func(routine_id("peek" & variable_types[var[TYPE]][2][4]), 
                        {var[ADDRESS] + data_seg})
end function

integer is_allocated is_allocated = 0
global function get_variable_address(sequence vars, sequence name)
  atom found, num, val
  sequence ret
  if (not is_allocated) or atom(vars[1]) then return 0 end if
  if atom(name[1]) then name = {name} end if
  ret = {} num = length(name)
  for count = 1 to num do
    found = find_var_index(vars[VARIABLE], name[count], NAME)
    if found then 
      val = vars[DATA_SEG] + vars[VARIABLE][found][ADDRESS]
      if num = 1 then return val else ret &= {val} end if
    else return 0 end if
  end for  
  return ret
end function

global function set_variable(sequence vars, sequence name, object value)
  integer found, num
  if atom(vars[1]) then return vars end if
  if atom(name[1]) then name = {name} end if
  if atom(value) then value = {value} end if
  num = length(name)
  for count = 1 to num do
    found = find_var_index(vars[VARIABLE], name[count], NAME)
    if found then
      vars[VARIABLE][found][VALUE] = value[count]
      if is_allocated then
        poke_var(vars[DATA_SEG], vars[VARIABLE][found])
      end if  
    else
      return {-1, "set_variable() Error : Undeclared variable '" & name[count] & "'\n"}
    end if  
  end for
  return vars
end function

global function get_variable(sequence vars, sequence name)
  integer found, num
  sequence ret
  atom val
  if atom(vars[1]) then return vars end if
  if atom(name[1]) then name = {name} end if
  num = length(name)
  ret = {}
  for count = 1 to num do
    found = find_var_index(vars[VARIABLE], name[count], NAME)
    if found then
      if is_allocated then
        val = peek_var(vars[DATA_SEG], vars[VARIABLE][found])
      else
        val = vars[VARIABLE][found][VALUE]
      end if    
      if num = 1 then return val else ret &= {val} end if
    else
      return "get_variable() Error : Undeclared variable '" & name[count] & "'\n"
    end if  
  end for
  return ret
end function

global function allocate_variables(sequence vars)
  integer size
  atom data
  if atom(vars[1]) then return vars end if
  size = 0                                           
  for typ = CONSTANT to VARIABLE do
    for count = 1 to length(vars[typ]) do
      if vars[typ][count][TYPE] != CONSTANT then
        vars[typ][count][ADDRESS] = size
        size += variable_types[vars[typ][count][TYPE]][2][3]
      end if
    end for  
  end for
  data = allocate(size)
  vars[DATA_SEG] = data
  for typ = CONSTANT to VARIABLE do
    for count = 1 to length(vars[typ]) do
      if vars[typ][count][TYPE] != CONSTANT then
        poke_var(data, vars[typ][count]) 
      end if
    end for  
  end for
  is_allocated = 1
  return vars
end function  

global procedure resolve_variables(atom address, sequence vars)
  if atom(vars[1]) then
    text_mode()
    puts(1, vars[2] & '\n') 
    abort(0)
  end if
  for typ = CONSTANT to VARIABLE do     --resolve variable references
    for count = 1 to length(vars[typ]) do
      if --vars[typ][count][REFERENCES] > 0 and
         vars[typ][count][TYPE] != CONSTANT then
        resolve_param(address, vars[typ][count][NAME], 
                      vars[typ][count][ADDRESS] + 
                      vars[DATA_SEG])
      end if
    end for
  end for                      
end procedure

global procedure free_variables(sequence vars)
  if atom(vars[1]) then return end if
  free(vars[DATA_SEG])
  is_allocated = 0
end procedure

-----------------------------------------------------------------------------
--Postfix Compiler

--construct_function : insert function code with references
atom func_offs_ind  func_offs_ind = 0
function construct_function(sequence name)
  sequence code
  integer offs, index, flag
  flag = 0
  index = find(name, function_list)             --Look up function..
  --Function writing code here
  code = function_table[index][3]           --straight from table
  --individualize labels
  for count = 1 to length(code) do
    offs = find('#', code[count])
    if offs then
      flag = 1
      if offs = length(code[count]) then        --insert at end
        code[count] = code[count][1..offs - 1] & 
                      sprintf("%d", func_offs_ind)
      else                                      --insert inside
        code[count] = code[count][1..offs - 1] & 
                      sprintf("%d", func_offs_ind) & 
                      code[count][offs + 1..length(code[count])]
      end if
    end if
  end for
  func_offs_ind += flag                    
  return code  
end function                                               
--register_reference : finds a register reference and returns value
function register_reference(object name)
  integer data
  if sequence(name) and (length(name) = 5 or length(name) = 6)     --match name
     and compare(upper(name[1..3]), "ST[") = 0 and name[5] = ']' then
    if not(length(name) = 6 and name[6] != '!') then  
      data = name[4] - '0'                      --get absolute position
      if data >= 0 and data <= 7 then           --sanity check
        return data                             --return register value
      end if
    end if
  end if
  return -1                                     --not a register reference
end function      
--resolve_name : finds and resolves name references
function resolve_name(sequence item, sequence vars)
  integer found, id
  --index the item
  id = item[1]
  if id = STACKPOP or id = ASSIGN then id = VARIABLE end if  
  found = find_var_index(vars[id], item[2], 
           (id = CONSTANT) * VALUE + 
           (id = VARIABLE) * NAME)
  if found = 0 then
    if id = CONSTANT then
    --introduce new values
      vars[id] &= {{sprintf("CONST%d", length(vars[id]) + 1), 6, item[2], 0, 1}}
      item[2] = length(vars[id])
    else
      item = {ERROR, "Fatal error : Variable '" & item[2] & "' not declared"}
    end if
  else 
    vars[id][found][REFERENCES] += 1
    item[2] = found
  end if
  return {item, vars}
end function  

--construct_name : constructs an operand tail from name
integer stack_use, stackuse_boost
function construct_name(sequence item, sequence vars)
  sequence pop, var, name, int
  integer abs_pos, rel_pos, len
  int = "" --no integer prefix
  pop = ""                                      --no pop prefix 
  abs_pos = register_reference(item[2])         --see if it's a register
  if abs_pos = -1 then 
    item = resolve_name(item, vars) 
    vars = item[2]  item = item[1]
  end if
  if item[1] = ERROR then return item end if
----Pop prefixer not working?
  if item[1] = ASSIGN or abs_pos != -1 then     --Is item assign or register?
    var = vars[VARIABLE][item[2]]
    len = length(var[NAME])                       --get length of name
    if len > 1 and var[NAME][len] = '!' then      --Pop operator at end?
      pop = "P"                                 --set pop prefix
      vars[VARIABLE][item[2]][NAME] = 
         vars[VARIABLE][item[2]][NAME][1..len - 1]             --remove pop operator
    end if
  end if
------------------
  if abs_pos != -1 then                         --Was it a register?
    if abs_pos > stackuse_boost then            --boost the stack use count
      stackuse_boost = abs_pos 
    end if
-- relative register addressing not working
--    rel_pos = (abs_pos + stack_use - 1) --* (abs_pos > 0) --find position
    name = {0, " ST(" & abs_pos + '0' & ")", int} --alter name
--    if length(pop) > 0 then item[1] = STACKPOP end if
  else--if item[1] != CONSTANT then                    --Memory reference..
    if item[1] = ASSIGN then
      var = vars[VARIABLE][item[2]]
      if var[TYPE] = CONSTANT then
        return {-1, "Assignment error: Cannot alter FPU Constant '" & var[NAME] & "'"}
      end if
    else
      var = vars[item[1]][item[2]]
    end if
    if var[TYPE] = CONSTANT then
      if item[1] = CONSTANT then
        if var[VALUE] = 0 then name = {var[TYPE], "Z", int}
        elsif var[VALUE] = 1 then name = {var[TYPE], "1", int}
        else return {-1, "Variable Error: Unusable FPU Constant"}
        end if
      else  
        name = {var[TYPE], upper(var[NAME]), int}
      end if
    else  
      if var[TYPE] < 6 then int = "I" end if  --integer type
      name = {var[TYPE], pop & " " & variable_types[var[TYPE]][2][2] & 
              "[" & var[NAME] & "]", int}
    end if
  end if
  return {name, vars}
end function  
--ten_byte() - determines if a variable is 80 bit f.p.
function split(integer var_type) 
  return compare(variable_types[var_type][2][2][1..5], "tbyte") = 0 or
         var_type < 6
         --compare(variable_types[var_type][2][2][1..5], "fixed") = 0
end function  
--pasm_flush : postfix assembler operand stack cleanup
function pasm_flush(object info, sequence vars)
  object item, varname, code, front, revfix
  code = {}
  item = st_pop()                               --fetch current item
  if sequence(info) then                --Operator : Non-default action?
    --Construct an opcode from info
    front = operator_table[info[1]] & info[2]   --Build opcode
  end if  
  if sequence(item) then                        --Fresh operand?
    if atom(info) then                          --Default for flush
      front = "FLD" 
    end if
    if item[1] = CONSTANT or 
       item[1] = VARIABLE or 
       item[1] = STACKPOP then--Introduce variable..
      varname = construct_name(item, vars)                   --resolve any registers
      if atom(varname[1]) then return varname end if
      vars = varname[2]  varname = varname[1]
      revfix = ""
      if item[1] = STACKPOP then
        if sequence(info) then
          if split(varname[1]) then
            if opt_trans[info[1]]  then--Non-transposable?
              if compare(info[2], "R") = 0 then--already reversed?
                front = front[1..length(front) - 1] --unreverse the original
              else  
                revfix = "R"                        --use reverse version
              end if
            end if
            code &= {"F" & varname[3] & "LD" & varname[2] & "\n", front & revfix & "P ST(1), ST(0) \n"}
          else
            code &= {front & "P" & varname[2] & " \n"}
          end if  
        --else
          --do nothing?
          --code &= {front & varname[2] & " \n", "FFREE" & item[2] & " \n"}
        end if    
      else
        if split(varname[1]) and sequence(info) then
          if opt_trans[info[1]] then                 --Non-transposable?
            if compare(info[2], "R") = 0 then--already reversed?
              front = front[1..length(front) - 1] --unreverse the original
            else  
              revfix = "R"                        --use reverse version
            end if
          end if
          code &= {"F" & varname[3] & "LD" & varname[2] & "\n", front & revfix & "P ST(1), ST(0) \n"}
        else
          code &= {front & varname[2] & " \n"}
        end if  
      end if  
    end if
  elsif sequence(info) then                     --Used (undefined) operand
    --Operand is already in stack and action is to be taken
    code = {front & "P ST(1), ST(0)" & " \n"}   --Fetch from stack
  end if
  return {code, vars}    
end function

function flush_err(sequence flush)
  if length(flush) > 0 then if atom(flush[1]) then if flush[1] = ERROR then
        return 1
  end if end if end if
  return 0
end function
--postfix_compiler : second pass of expression parser
--  Takes a mathematical expression in extended postfix format and 
--  constructs a sequence of assembler-level coprocessor instructions.
--  Returns the sequence: {STACK, OUTPUT} where if STACK < 0, OUTPUT 
--  contains a string describing some error, else STACK contains the 
--  number of internal coprocessor stack registers required and OUTPUT
--  contains the code in the form {"instruction", "instruction", ..}.
global function postfix_compiler(sequence postin, sequence vars)
  object item, temp, first, second, flush, var
  integer op, stackmax, found 
  sequence code, revfix, args
  if postin[1] = -1 then return postin end if   --Reject bad postfix
  code = {}  stackmax = 0  stackuse_boost = 0
  postin = postin[2]
  for count = 1 to length(postin) do
    --update stack use counter
    stack_use = 0                                   --set stack counter
    for ptr = 1 to length(stack) do               --scan through stack
      if atom(stack[ptr]) then stack_use += 1  end if   --if used, inc counter
    end for
    --get postfix item
    item = postin[count]                        --get next item
    --operand..
    if item[1] = CONSTANT or item[1] = VARIABLE then --Any operand..
      ---------------------
      --Item is an operand..
      --Any kind of operand gets delayed until needed
      st_push(item)                             --delay it
    --operator..
    else                                        --Any operator..
      -----------------------
      --Item is an operator..
      --error checking
      if st_empty() then                        --run out of operands?
        if item[1] = ASSIGN then                --Assign to variable?
          return {-1, "Variable assignment Error: No operand for '" & item[2] & "='"}
        end if
      end if  
      --Any kind of operator is performed immediately using delayed operands
      if sequence(item[2]) then                 --Named process..
        ---------------------
        --Item is a process..
        --Named process, function() or assign=
        if item[1] = ASSIGN then                --Assign to variable?
          ------------------------
          --Process is an assign..
          flush = pasm_flush(0, vars)                --check operand
          if flush_err(flush) then return flush end if
          vars = flush[2]  flush = flush[1]
          code &= flush
          item = construct_name(item, vars)             --resolve name references
          if atom(item[1]) then return item end if
          vars = item[2]  item = item[1]
          code &= {"F" & item[3] & "STP" & item[2] & " \n"}             --add code
        elsif item[1] = FLUSH then
          flush = pasm_flush(0, vars)                --check operand
          if flush_err(flush) then return flush end if
          vars = flush[2]  flush = flush[1]
          code &= flush
          
        else                                    --Unary function..
          -------------------------
          --Process is a function..
          temp = find(item[2], function_list)
          if temp then  --Look up function..
            temp = function_table[temp][2][1]  --expected number of args
            --check arguments
            if item[3] != temp then
              args = {'a'}
              for c = 2 to temp do
                args &= ", " & c + 96
              end for
              return {-1, 
              sprintf("Function usage Error: Bad number of arguments (%d) to '%s(", 
               {item[3], item[2]}) & args & ")'"}
            else
              --insert code for function
              for c = 1 to temp do              --for each argument..
                flush = pasm_flush(0, vars)                --check operand
                if flush_err(flush) then return flush end if
                vars = flush[2]  flush = flush[1]
                code &= flush            --  check operand
              end for
              code &= construct_function(item[2]) --add code
            end if  
          else                                  --Not found..
            return {-1, "Function identification Error: Unknown function '" & item[2] & "()' "}
          end if
        end if    
      -------------------------------
      --Item is a symbolic operator..
      elsif item[2] = '~' then                  --Unary negative..
        ------------------------------------------------
        --Item is the unary negative substitute symbol..
        flush = pasm_flush(0, vars)                --check operand
        if flush_err(flush) then return flush end if
        vars = flush[2]  flush = flush[1]
        code &= flush                --check operand
        code &= {"FCHS \n"}
      else                                      --Binary operator..
        ------------------------------------
        --Item is symbolic binary operator..
        revfix = ""                             --reversing prefix
        op = find(item[2], operators)           --look it up
        second = st_pop()                         --grab 2nd operand
        if st_empty() then                      --run out of operands?!
          return {-1, "Formula syntax error: not enough operands for '" & item[2] & "'"}
        end if  
        --Re-order operands when stacked
        if atom(second) then                      --Already stacked? (stale)
          if opt_trans[op] then                 --Non-transposable?
            revfix = "R"                        --use reverse version
          end if
          st_push(second)                         --put second as 1st operand
          flush = pasm_flush(0, vars)                --check operand
          vars = flush[2]  flush = flush[1]
        else
        --introduce new operands
          found = 0
          ------------------------------------------------------------
          --optimization : operands are reversed to take advantage of
          --               fast < 80 bit operands. 
          temp = second
          if temp[1] = CONSTANT and (temp[2] = 1 or temp[2] = 0) then
            temp[1] = VARIABLE
          end if  
          if temp[1] = VARIABLE then    
            var = find_var_index(vars[second[1]], second[2], (2 - second[1]) * 2 + 1)
            if var = 0 then return {-1, "Reference error: Variable '" & second[2] & "' not found"} end if
            if split(vars[second[1]][var][TYPE]) then --tbyte?
              first = st_top()                   --ok, get first operand
              if sequence(first) then            --yes, ain't stacked.
                temp = first
                if temp[1] = CONSTANT and (temp[2] = 1 or temp[2] = 0) then
                  temp[1] = VARIABLE
                end if
                if temp[1] = VARIABLE then  
                  var = find_var_index(vars[first[1]], first[2], (2 - first[1]) * 2 + 1)
                  if var = 0 then return {-1, "Reference error: Variable '" & second[2] & "' not found"} end if
                  if not split(vars[first[1]][var][TYPE]) then --not tbyte?
                    if opt_trans[op] then                 --Non-transposable?
                      revfix = "R"                        --use reverse version
                    end if
                    found = 1
                    st_push(second)                         --put second as 1st operand
                    flush = pasm_flush(0, vars)                --check operand
                    if flush_err(flush) then return flush end if
                    vars = flush[2]  flush = flush[1]
                  end if
                end if
              end if
            end if
          end if        
          ------------------------------------------------------------
          if not found then
            flush = pasm_flush(0, vars)                --check operand
            if flush_err(flush) then return flush end if
            vars = flush[2]  flush = flush[1]
            st_push(second)                         
          end if
        end if
        code &= flush
--?{op, revfix}
        flush = pasm_flush({op, revfix}, vars)                --check operand
        if flush_err(flush) then return flush end if
        vars = flush[2]  flush = flush[1]
        code &= flush        --flush operator
      end if
      st_push(0)                                --Flag used operand
    end if
    --Update stack usage tracker
    if stack_use > stackmax then stackmax = stack_use end if
  end for
  return {stackmax + stackuse_boost, code, vars}
end function

--display Postfix in a colourful and kinda readable format
global procedure show_post(sequence p)
  if p[1] = -1 then return end if
  puts(1, "Postfix:  ")
  p = p[2]
  for c = 1 to length(p) do
    machine_proc(9, p[c][1] + 9)
    if p[c][1] = CONSTANT then
      puts(1, " ")
      print(1, p[c][2])
    elsif p[c][1] = ASSIGN then
      puts(1, " ")
      puts(1, p[c][2] & "=")
    elsif p[c][1] != FLUSH then
      puts(1, " ")
      puts(1, p[c][2])
      if p[c][1] = OPERATOR or p[c][1] = FUNCTION then
        printf(1, "(%d)", p[c][3])
      end if
    end if
  end for       
  machine_proc(9, 7)
  puts(1, "     \n")
end procedure       

--display used variables with their values
global procedure show_vars(sequence v)
  sequence frm
  puts(1, "     \nVariables:   ")
  machine_proc(9, 15)
  for c = 1 to length(v[VARIABLE]) do
--    if v[VARIABLE][c][REFERENCES] > 0 then
      if v[VARIABLE][c][TYPE] > 4 then frm = "%s:%s=%f, " 
      else frm = "%s:%s=%d, " end if
      if v[VARIABLE][c][TYPE] != CONSTANT then
        printf(1, frm,
                {type_names[v[VARIABLE][c][TYPE]],
                v[VARIABLE][c][NAME],
                peek_var(v[DATA_SEG], v[VARIABLE][c])})
      else
        printf(1, frm,
                {type_names[v[VARIABLE][c][TYPE]],
                v[VARIABLE][c][NAME],
                v[VARIABLE][c][VALUE]})
      end if            
--    end if
  end for
  machine_proc(9, 7)
  puts(1, "     \n")
end procedure       

--make an include file of variables with their values
global procedure include_vars(atom fn, sequence vars)
  sequence str
  integer pos, size
  pos = 0 size = 0
  puts(fn, "\nglobal constant\n ")
  for typ = CONSTANT to VARIABLE do
    for count = 1 to length(vars[typ]) do
      if vars[typ][count][TYPE] != CONSTANT then
        if size then puts(fn, ",") end if
        str = sprintf(" %s = %d", {upper(vars[typ][count][NAME]), pos})
        pos += variable_types[vars[typ][count][TYPE]][2][3]
        size += length(str) + 1
        if size >= 77 then 
          puts(fn, "\n ") 
          size = length(str) + 1
        end if
        puts(fn, str)  
      end if
    end for  
  end for
  puts(fn, "\n\n--Allocate data segment\nglobal atom data\n")
  printf(fn, "data = allocate(%d)\n", pos)
  printf(fn, "mem_set(data, 0, %d)\n", pos)
  puts(fn, "\n--Initialise data members\n")
  for typ = CONSTANT to VARIABLE do
    for count = 1 to length(vars[typ]) do
      if vars[typ][count][TYPE] != CONSTANT then
        if vars[typ][count][VALUE] then 
          printf(fn, "poke" & variable_types[vars[typ][count][TYPE]][2][4]
                            & "(data + %s, %f)\n",
                            {upper(vars[typ][count][NAME]), vars[typ][count][VALUE]})
        end if
      end if
    end for  
  end for
end procedure       

--display constants with their values
global procedure show_const(sequence v)
  puts(1, "Constants:   ")
  machine_proc(9, 15)
  for c = 1 to length(v[CONSTANT]) do
    if v[CONSTANT][c][TYPE] != CONSTANT then
      printf(1, "%s=%d, ", {v[CONSTANT][c][NAME], peek_var(v[DATA_SEG], v[CONSTANT][c])})
    end if
  end for
  machine_proc(9, 7)
  puts(1, "     \n")
end procedure       

--formula into eu_asm code, returns {vars, asm} or {-1, "error msg"}
--this is for quick & dirty repeated inline compilations in eu_asm code.
--WARNING-- does not allocate variables!! do that before code is called
--does not make use of reference counts.
global function code_formula(sequence vars, sequence formula)
  object post, code, asm
  post = infix_to_postfix(formula)      --turn it into postfix
  code = postfix_compiler(post, vars)         --assemble into code
  if code[1] = ERROR then return code end if  
  asm = ""
  for count = 1 to length(code[2]) do
    asm &= code[2][count]               --string code together
  end for  
  return {code[3], asm}
end function  

--formula into memory, allocates variables and return {vars, handle}
--or {ERROR, "error msg"}.  this is for individual functional units.
function parse_formula(sequence vars, sequence formula)
  object post, code, asm, variables, eval
  atom address
  if debug then puts(1, "Formula:   " & formula & "\n") end if
  variables = read_variables(vars)
  if atom(variables[1]) then return variables end if
  post = infix_to_postfix(formula)      --turn it into postfix
  code = postfix_compiler(post, variables)         --assemble into code
  if code[1] = ERROR then return code end if
  variables = allocate_variables(code[3]) --allocate all variables
  if debug then 
    show_post(post) 
    show_vars(variables) 
    show_const(variables) 
  end if 
  asm = ""
  code[2] =  {"PUSHAD \n",              --runtime wrapper
              "FINIT \n",
              "FLDCW [CONTROL_WORD] \n"}
              & code[2] &
             {"FINIT \n", 
              "POPAD \n",
              "RET \n"} 
  if debug then
--    printf(1, "\n-- Formula OK, Stack required: %d of 8 --\n", code[1])
  end if
  for count = 1 to length(code[2]) do
    if debug then puts(1, "* " & code[2][count]) end if
    asm &= code[2][count]               --string code together
  end for  
  if debug then 
    eval = postfix_eval(post, variables) 
    if atom(eval[1]) then return eval end if
    puts(1, "Expected evaluation: ") 
    for count = 1 to length(eval[2]) do 
      printf(1, "%1.16f, ", eval[2][count]) 
    end for
  end if 
  code = get_asm(asm)                --assemble code
  address = allocate(length(code))  poke(address, code)
  for typ = CONSTANT to VARIABLE do     --resolve variable references
    for count = 1 to length(variables[typ]) do
      if variables[typ][count][REFERENCES] > 0 and
         variables[typ][count][TYPE] != CONSTANT then
        resolve_param(address, variables[typ][count][NAME], 
                      variables[typ][count][ADDRESS] + 
                      variables[DATA_SEG])
      end if
    end for
  end for                      
  return {variables, address}
end function  

set_fp_precision(64)--default precision
--------------------------------------------------------------------------

--debugging nonsense
sequence formula, vars
object eval
sequence result
atom address

--for the rnd() test
--include graphics.e  object vid, col, cols  cols = repeat(0, 640)

if debug = 1 then
  --declare variables
  vars = "word CONTROL_WORD = 3967 \n" &
       "float8 a = 1, b = 0.1, c = 0.8, x = 4, y = 2, z = -1 \n" &
       "float8 x2 = 51, y2 = 100"
  --declare imaginative formula
  --formula = "x2=(1/a+(cos(b/2*3)+c-(2/sin(tan(11)+12))))"
  --formula = "x2=b/2*3+c-(2/sin(tan(11)+12))"
  --formula = "x2=pi + l2t + l2e + lg2 + ln2 "
  --formula = "x2 = sqr(x) - sqr(y) : y2 = 2 * x * y"
  --formula = "x2 = arctan2((1 + 100), 2 + 1)"
  --formula = "x2 = arctan2(cos(100) * sin(20) * 50, 24)"
  --formula = "x2 = ((1 + 10) ^ 1 + 4)"
  --formula = "x2 = -84 ^ 2 : y2 = 2 ^ 8 "
  --formula = "x2 = 2 + (3 * sqr(a - a / x) + cos(tan(sin(cotan(cosec(c - 7)))))) / y"
  --formula = "x2 = abs(tan(a)) / sqr(cos(c) / c) + 2.7"
  --formula = "x2 = arccos(a)"
  --formula = "x2 = 1 + y - a * sqr(x) : y2 = b * x" -- henon
  --formula = "x2 = y - sgn(x) * cos(sqr(ln(abs(b * x - c)))) * arctan(sqr(ln(abs(c * x - b)))) : y2 = a - x : x = x2 : y = y2"
  --formula = "x2 = rnd()"
  --formula = "x2 = arctan2(a, b) / 2 "
  --formula = "c = 2 : x2 = cos(c) * z : y2 = sin(c) * z"
  --formula = "x2 = a * y - b * x "
  formula = "x2 = int(6.9)"
  result = parse_formula(vars, formula)
  if atom(result[1]) then
    puts(1, "\n" & result[2] & "\n")
    abort(1)
  end if
  
  
  --do it
  call(result[2])

  --show ALL the vars
  --show_vars(result[1]) 

  --get specific variables
  eval = {get_variable(result[1], "x2"),
          get_variable(result[1], "y2")
          }
  puts(1, "\nActual evaluation: ") 
  for count = 1 to length(eval) do printf(1, "%1.16f, ", eval[count]) end for
  puts(1, "\n")

  --rnd() test
  --vid = graphics_mode(#12)
  --for count = 1 to 30000 do
  --  if get_key() != -1 then exit end if
  --  call(result[2])
  --  col = (get_variable(result[1], "x2") / 2 + .5) * 640
  --  cols[col + 1] += 1
  --  pixel(15, {col, 479 - cols[col + 1]})
  --end for  

end if                    


