--include vesa20.e
include neil.e
global procedure text_mode() restore_mode() end procedure --uncomment this if vesa20.e absent

integer bpp, fractal_style
sequence max

max = {640, 480} bpp = 16
fractal_style = 10 --41

constant iter_step = 1000

global constant 
         pi = 3.141592653589753

--------------------------------------------------------------------------
--Engine (machine level stuff)
include engine.e  

-----------------------------------------------------------------------------
--Fractals
include fracasm.e

----------------------------------------------------------------------------
-- Code compilation, assembly and data allocation
atom hitmap, pix_list, video_len
--seed random number generator
poke4(data + RAND_SEED, rand(1073741823)) 

----------------------------------------------------------------------------
--Data initialization

--ok, set mode now...
if vesa_mode(max[1], max[2], bpp) then puts(1, "No modes!\n") abort(0) end if
video_len = bytes_per_line * max[2]

--hitmap allocation
hitmap = allocate(video_len / bytes_per_pixel)
mem_set(hitmap, 0, video_len / bytes_per_pixel)

--pixlist allocation
pix_list = allocate(video_len / bytes_per_pixel * 4)
mem_set(pix_list, 0, video_len / bytes_per_pixel * 4)

--set once stuff
poke4(data + REAL_SCREEN, video_mem)
poke4(data + SCREEN_SIZE, video_len)
poke4(data + PIXELS_PER_LINE, bytes_per_line / bytes_per_pixel)
poke(data + BITS_PER_PIXEL, bits_per_pixel)
poke(data + GRANULARITY, win_granularity)
poke4(data + XSIZE, max[1])
poke4(data + YSIZE, max[2])
poke4(data + HITMAP, hitmap)
poke4(data + PIX_LIST, pix_list)

----------------------------------------------------------------------------
--Wedge code

procedure insert_formula(integer style, integer method, integer projection)
  poke(shuffle_insert, params[style][SHUFFLE])
  poke(fractal_insert, params[style][ORBIT])
  poke(method_insert, methods[method][2])
  poke(project_insert, projections[projection][2])
end procedure

integer method, projection
procedure init(integer style)
  object delta, values, size
  values = call_func(params[style][INIT], {params[style]})

  --coefficients
  for ct = 1 to length(values[COEF]) do
    poke_float8(data + A + (ct - 1) * 8, values[COEF][ct])
  end for

  --corners
  if atom(values[EDGE]) then  --auto corners
    poke_float8(data + AUTO_XL,{1000000,-1000000,1000000,-1000000,1000000,-1000000})
    poke_float8(data + MIN_R, {0, 0, 0, 0, values[EDGE]})
  else
    size = values[EDGE][2] - values[EDGE][1]
    delta = max / size
    poke_float8(data + MIN_R, values[EDGE][1] & delta)
    poke_float8(data + CENTRE_X, (values[EDGE][1] + values[EDGE][2]) / 2 
                & pi / size & values[EDGE][2] & size)
  end if

  --rotation dynamics and colourization
  poke_float8(data + COS_XY,
         {cos(values[ANGLE] / 180 * pi), sin(values[ANGLE] / 180 * pi),
          values[DIS], values[SPT], rnd(0, 1), 0} & params[style][ORIGIN])
  poke_float8(data + REF_X, {0,0,0,0,0,0,params[style][MIN_DIM]})
  poke_float8(data + LYAP_X,{values[SEED][1] + 0.000001,values[SEED][2],
              0,0,0,0,params[style][MIN_LYAP]})

  --initial seed vars for 2d
  poke_float8(data + X, {values[SEED][1], values[SEED][2],   0})

  --update seed z and lyap seed z vars if 3d
  if and_bits(params[style][FLAGS], THREED) then
    poke_float8(data + X + 16, values[SEED][3])
    poke_float8(data + LYAP_X + 16, values[SEED][3])
  end if

  --reset iterations and status
  poke4(data + ITER, {0, 0})
  poke(data + STATUS, {0, and_bits(params[style][FLAGS], NOMIN)})
end procedure

procedure run_engine()
  integer key, iter, auto, numpix, oldpix
  key = ' ' iter = 0 oldpix = 0
  while key != 27 do

    --continuous iterator
    poke4(data + ITERATIONS, peek4u(data + ITERATIONS) + iter_step)

    --start new fractal 
    numpix = peek4u(data + PIXLIST_LEN)
    if iter >= params[fractal_style][MAXITER] or key = ' ' or 
       (and_bits(params[fractal_style][FLAGS], PIXLIST) and 
       oldpix = numpix and iter > iter_step) then

      --new fractal
      fractal_style = rand(length(params))
      init(fractal_style)

      --choose method
      method = rand(4)
      while not params[fractal_style][COLOUR][method] do
        method = rand(4)
      end while

      --choose projection
      if params[fractal_style][PROJECT] then projection = rand(7)
      else projection = 1 end if
      insert_formula(fractal_style, method, projection)
      auto = atom(params[fractal_style][CORNERS])
    end if

    --clear screen when needed
    iter = peek4u(data + ITER)
    if ((not auto) and iter = 0) or (auto and iter = iter_step) then 
    --use either this.. (should be faster with VESA 2.0 or linear modes)
      --call(clear_maps)
      --set_page(floor(video_len / 65536))  --page juggling for clear_maps
    --or these...       (better for VESA 1.2 banked modes on my thinkpad)
      call(clearscreen)
      mem_set(hitmap, 0, video_len / bytes_per_pixel)
      poke4(data + PIXLIST_LEN, 0) --pixmap grows uncontrollably without this!
    --
      position(29, 64 - length(params[fractal_style][NAME]) 
                      - length(projections[projection][1])
                      - length(methods[method][1]))
      puts(1, "     " & params[fractal_style][NAME] & 
              " using " & methods[method][1] & 
              " on " & projections[projection][1])
    end if

    --do iter_step iterations
    call(fractal_engine)
    
    --fractal has been rejected?
    if peek(data + STATUS) then
      init(fractal_style)
    end if
    
    oldpix = numpix 
    key = get_key()
  end while  
end procedure

run_engine()

