-- NickFrac Fractal definitions precompiled include file --
-- fracasm.e --   Generated by "C:\EUPHORIA\PROGS\PARSFRAC\SS20_DOS.EX"
-- from the fractals definition file "fractals.e"

--Orbital Fractal types v1.0 Nick Metcalfe 5/02

-----------------------------------------------------------------------------
--Variable interface

--Fractal 'params' structure names
global constant 
        NAME = 1,               --fractal name
        FLAGS = 2,              --sequence of general flags to control engine
        ENABLE = 3,             --enable flag
        MAXITER = 4,            --maximum iterations
        CORNERS = 5,            --fractal window corners or border for auto
        COLOUR = 6,             --sequence of flags for methods
        ORIGIN = 7,             --rotation origin
        PROJECT = 8,            --allow projections flag
        MIN_LYAP = 9,           --minimum lyapunov exponent
        MIN_DIM = 10,           --minimum fractal dimenson
        INIT = 11,              --initialisation routine_id
        SHUFFLE = 12,           --shuffle code
        ORBIT = 13              --orbit code
        
--Fractal 'values' structure names
global constant 
        COEF = 1,           --fractal coefficients
        SEED = 2,           --initial orbital seed
        EDGE = 3,           --fractal window corners or border for auto
        SPT = 4,            --spotlight value for methods
        DIS = 5,            --distance value for methods 
        ANGLE = 6           --rotation angle

--Colouring styles
global constant
        PERIOD = 1,
        DISTANCE = 2,
        ATTRACTOR = 3,
        TEXTURE = 4

--Binary flags to control engine
global constant
        STANDARD = 1,
        IFS      = 2,
        STRANGE  = 4,
        THREED   = 8,
        PIXLIST  = 16,
        NOMIN    = 32
        
global integer num_frac
global sequence params 
global sequence methods         --colouring methods
global sequence projections     --projection styles
params = {}
-----------------------------------------------------------------------------
--Helper, shared and generic functions and macros

constant freedom = 260
--return decimal random number between l & h
global function rnd(atom l, atom h)
  return (rand(freedom) - 1) / (freedom - 1) * (h - l) + l
end function
--pn() randomly returns +1 or -1
function pn()
  return (rand(100) < 50) * 2 + 1
end function   
--sign of r
function sgn(atom r)                 
  if r < 0 then return -1 else return 1 end if
end function
--absolute value of r
function abs(atom r)                 
  return sgn(r) * r                             
end function

--Strange attractor helpers
--rand_strange() returns random strange attractor coefficients
function rand_strange(integer num)
  sequence coef coef = {}
  for count = 1 to num do
    coef &= {rnd(-1, 1)}
  end for
  return coef  
end function

--IFS helpers
--weightings assigns IFS rule probabilities based on size
function weightings(sequence co)
  atom sum, mag
  sequence prob
  prob = {} sum = 0
  for ct = 0 to length(co) - 7 by 7 do
    mag = abs(co[ct+1] * co[ct+4] - co[ct+2] * co[ct+3])
    if mag = 0 or mag > 1 then return 0 end if
    prob &= mag sum += mag
  end for
  if not sum then return 0 end if
  for ct = 1 to length(prob) do
    co[ct * 7] = prob[ct] / sum
  end for
  return co
end function  
--rand_ifs() returns random IFS coefficients
function rand_ifs(integer max_rules)
  object result  result = 0
  while atom(result) do
    result = {}
    for ct = 1 to rand(max_rules - 1) + 1 do
      result &= {rnd(-1, 1), rnd(-1, 1), rnd(-1, 1), rnd(-1, 1), rnd(-1, 1), rnd(-1, 1), 0}
    end for
    result = weightings(result)
  end while
  return result
end function  

--Inverse julia method helpers
--get_mandel returns the iteration depth of a mandel orbit
function get_mandel(integer maxiter, object c)
  atom iter, x, y, x2, y2
  if atom(c) then return c end if
  iter = 0 x = 0 y = 0 x2 = 0 y2 = 0
  while iter < maxiter and x2 + y2 < 4 do
    x2 = x * x  y2 = y * y  iter += 1
    y = 2 * x * y + c[2]  x = x2 - y2 + c[1] 
  end while  
  return iter 
end function
--get_xmandel returns the iteration depth of an x-mandel orbit
function get_xmandel(integer maxiter, object c)
  atom iter, x, y, x2, y2
  if atom(c) then return c end if
  iter = 0 x = 0 y = 0 x2 = 0 y2 = 0
  while iter < maxiter and x2 + y2 < 4 do
    x2 = x * x  y2 = y * y  iter += 1
    x = 2 * x * y + c[2]  y = x2 - y2 + c[1] 
  end while  
  return iter 
end function
--find_julia returns a likely parameter near the edge of a mandel lake 
function find_julia(integer maxiter, atom id)
  object inside, outside, pos, count
  inside = 0 outside = maxiter
  while call_func(id, {maxiter, inside}) < maxiter do
    inside = {rnd(-1.5, 1.5), rnd(-1.5, 1.5)}
  end while
  while call_func(id, {maxiter, outside}) = maxiter do
    outside = {rnd(-1.5, 1.5), rnd(-1.5, 1.5)}
  end while
  count = 1
  while 1 do
    pos = (outside - inside) / 2 + inside
    if call_func(id, {maxiter, pos}) = maxiter then 
      inside = pos
      if count > maxiter then exit end if
    else 
      outside = pos 
    end if
    count += 1
  end while
  return pos  
end function

--@@------------------------------------------------------------------------
-- Methods and Projections --
methods = {
  {"Period",
  {#DD,#05}&int_to_bytes(data + X)&{#DC,#25}&int_to_bytes(data + OLD_X)&{#D8,
  #C8,#DD,#05}&int_to_bytes(data + Y)&{#DC,#25}&int_to_bytes(data + OLD_Y)&{
  #D8,#C8,#DE,#C1,#D9,#FA,#DC,#0D}&int_to_bytes(data + DIST)&{#DC,#05}&
  int_to_bytes(data + COL)&{#DD,#1D}&int_to_bytes(data + HUE)&{#D9,#E8,#DC,
  #25}&int_to_bytes(data + HITS)&{#DD,#1D}&int_to_bytes(data + SATURATION)&{
  #DD,#05}&int_to_bytes(data + X)&{#D8,#C8,#DD,#05}&int_to_bytes(data + Y)&{
  #D8,#C8,#DE,#C1,#D9,#FA,#DC,#0D}&int_to_bytes(data + SPOT)&{#D9,#E8,#DE,#E1,
  #DC,#0D}&int_to_bytes(data + HIT_INT)&{#DD,#1D}&
  int_to_bytes(data + INTENSITY)&{#D9,#EE,#DC,#1D}&
  int_to_bytes(data + INTENSITY)&{#DF,#E0,#9E,#72,#08,#D9,#EE,#DD,#1D}&
  int_to_bytes(data + INTENSITY)&{#C3}},
  {"Distance",
  {#DD,#05}&int_to_bytes(data + X)&{#D8,#C8,#DD,#05}&int_to_bytes(data + Y)&{
  #D8,#C8,#DE,#C1,#D9,#FA,#DC,#0D}&int_to_bytes(data + SPOT)&{#DD,#1D}&
  int_to_bytes(data + HUE)&{#DD,#05}&int_to_bytes(data + HIT_SAT)&{#DD,#1D}&
  int_to_bytes(data + SATURATION)&{#DD,#05}&int_to_bytes(data + HIT_INT)&{#DD,
  #1D}&int_to_bytes(data + INTENSITY)&{#C3}},
  {"Attractor",
  {#DD,#05}&int_to_bytes(data + X)&{#DC,#25}&int_to_bytes(data + OLD_X)&{#D8,
  #C8,#DD,#05}&int_to_bytes(data + Y)&{#DC,#25}&int_to_bytes(data + OLD_Y)&{
  #D8,#C8,#DE,#C1,#D9,#FA,#DC,#0D}&int_to_bytes(data + DIST)&{#DC,#05}&
  int_to_bytes(data + COL)&{#DD,#1D}&int_to_bytes(data + HUE)&{#DD,#05}&
  int_to_bytes(data + HIT_SAT)&{#DD,#1D}&int_to_bytes(data + SATURATION)&{#DD,
  #05}&int_to_bytes(data + HIT_INT)&{#DD,#1D}&int_to_bytes(data + INTENSITY)&{
  #C3}},
  {"Texture",
  {#DD,#05}&int_to_bytes(data + X)&{#D8,#C8,#DD,#05}&int_to_bytes(data + Y)&{
  #D8,#C8,#DE,#C1,#D9,#FA,#DC,#0D}&int_to_bytes(data + SPOT)&{#DD,#1D}&
  int_to_bytes(data + HUE)&{#D9,#E8,#DC,#25}&int_to_bytes(data + HITS)&{#DD,
  #1D}&int_to_bytes(data + SATURATION)&{#DD,#05}&int_to_bytes(data + X)&{#DC,
  #25}&int_to_bytes(data + OLD_X)&{#D8,#C8,#DD,#05}&int_to_bytes(data + Y)&{
  #DC,#25}&int_to_bytes(data + OLD_Y)&{#D8,#C8,#DE,#C1,#D9,#FA,#DC,#0D}&
  int_to_bytes(data + DIST)&{#D9,#E8,#DE,#E1,#DC,#0D}&
  int_to_bytes(data + HIT_INT)&{#DD,#1D}&int_to_bytes(data + INTENSITY)&{#D9,
  #EE,#DC,#1D}&int_to_bytes(data + INTENSITY)&{#DF,#E0,#9E,#72,#08,#D9,#EE,
  #DD,#1D}&int_to_bytes(data + INTENSITY)&{#C3}}
}

projections = {
  --Linear
  {"Linear",
  {#C3}},
  --Sphere
  {"Sphere",
  {#DD,#05}&int_to_bytes(data + MAX_R)&{#DC,#25}&int_to_bytes(data + ROT_X)&{
  #DC,#0D}&int_to_bytes(data + WRP_X)&{#DD,#1D}&int_to_bytes(data + PNT_X)&{
  #DD,#05}&int_to_bytes(data + MAX_I)&{#DC,#25}&int_to_bytes(data + ROT_Y)&{
  #DC,#0D}&int_to_bytes(data + WRP_Y)&{#DD,#1D}&int_to_bytes(data + PNT_Y)&{
  #DD,#05}&int_to_bytes(data + PNT_X)&{#D9,#FF,#DD,#05}&
  int_to_bytes(data + PNT_Y)&{#D9,#FE,#DE,#C9,#DC,#0D}&
  int_to_bytes(data + PT36)&{#DC,#0D}&int_to_bytes(data + SIZE_X)&{#DC,#05}&
  int_to_bytes(data + CENTRE_X)&{#DD,#1D}&int_to_bytes(data + ROT_X)&{#DD,
  #05}&int_to_bytes(data + PNT_Y)&{#D9,#FF,#DC,#0D}&
  int_to_bytes(data + HALF)&{#DC,#0D}&int_to_bytes(data + SIZE_Y)&{#DC,#05}&
  int_to_bytes(data + CENTRE_Y)&{#DD,#1D}&int_to_bytes(data + ROT_Y)&{#C3}},
  --Horizontal cylinder
  {"Horiz Cyl",
  {#DD,#05}&int_to_bytes(data + MAX_I)&{#DC,#25}&int_to_bytes(data + ROT_Y)&{
  #DC,#0D}&int_to_bytes(data + WRP_Y)&{#DD,#1D}&int_to_bytes(data + PNT_Y)&{
  #DD,#05}&int_to_bytes(data + PNT_Y)&{#D9,#FF,#DC,#0D}&
  int_to_bytes(data + HALF)&{#DC,#0D}&int_to_bytes(data + SIZE_Y)&{#DC,#05}&
  int_to_bytes(data + CENTRE_Y)&{#DD,#1D}&int_to_bytes(data + ROT_Y)&{#C3}},
  --Vertical cylinder
  {"Vert Cyl",
  {#DD,#05}&int_to_bytes(data + MAX_R)&{#DC,#25}&int_to_bytes(data + ROT_X)&{
  #DC,#0D}&int_to_bytes(data + WRP_X)&{#DD,#1D}&int_to_bytes(data + PNT_X)&{
  #DD,#05}&int_to_bytes(data + PNT_X)&{#D9,#FF,#DC,#0D}&
  int_to_bytes(data + HALF)&{#DC,#0D}&int_to_bytes(data + SIZE_X)&{#DC,#05}&
  int_to_bytes(data + CENTRE_X)&{#DD,#1D}&int_to_bytes(data + ROT_X)&{#C3}},
  --Pillow
  {"Pillow",
  {#DD,#05}&int_to_bytes(data + MAX_R)&{#DC,#25}&int_to_bytes(data + ROT_X)&{
  #DC,#0D}&int_to_bytes(data + WRP_X)&{#DD,#1D}&int_to_bytes(data + PNT_X)&{
  #DD,#05}&int_to_bytes(data + MAX_I)&{#DC,#25}&int_to_bytes(data + ROT_Y)&{
  #DC,#0D}&int_to_bytes(data + WRP_Y)&{#DD,#1D}&int_to_bytes(data + PNT_Y)&{
  #DD,#05}&int_to_bytes(data + PNT_X)&{#D9,#FF,#DC,#0D}&
  int_to_bytes(data + HALF)&{#DC,#0D}&int_to_bytes(data + SIZE_X)&{#DC,#05}&
  int_to_bytes(data + CENTRE_X)&{#DD,#1D}&int_to_bytes(data + ROT_X)&{#DD,
  #05}&int_to_bytes(data + PNT_Y)&{#D9,#FF,#DC,#0D}&
  int_to_bytes(data + HALF)&{#DC,#0D}&int_to_bytes(data + SIZE_Y)&{#DC,#05}&
  int_to_bytes(data + CENTRE_Y)&{#DD,#1D}&int_to_bytes(data + ROT_Y)&{#C3}},
  --Torus
  {"Torus",
  {#DD,#05}&int_to_bytes(data + MAX_R)&{#DC,#25}&int_to_bytes(data + ROT_X)&{
  #DC,#0D}&int_to_bytes(data + WRP_X)&{#DD,#1D}&int_to_bytes(data + PNT_X)&{
  #DD,#05}&int_to_bytes(data + MAX_I)&{#DC,#0D}&int_to_bytes(data + PT8)&{#DC,
  #25}&int_to_bytes(data + ROT_Y)&{#DC,#0D}&int_to_bytes(data + WRP_Y)&{#DC,
  #0D}&int_to_bytes(data + TP4)&{#DD,#1D}&int_to_bytes(data + PNT_Y)&{#DD,
  #05}&int_to_bytes(data + PNT_Y)&{#D9,#FE,#DC,#0D}&
  int_to_bytes(data + PT18)&{#DD,#05}&int_to_bytes(data + PNT_X)&{#D9,#FF,#D9,
  #E8,#DE,#C1,#DE,#C9,#DC,#0D}&int_to_bytes(data + SIZE_X)&{#DC,#05}&
  int_to_bytes(data + CENTRE_X)&{#DD,#1D}&int_to_bytes(data + ROT_X)&{#DD,
  #05}&int_to_bytes(data + PNT_Y)&{#D9,#FF,#DC,#0D}&
  int_to_bytes(data + PT25)&{#DD,#05}&int_to_bytes(data + PNT_X)&{#D9,#FF,#D9,
  #E8,#DE,#C1,#DE,#C9,#DC,#0D}&int_to_bytes(data + SIZE_Y)&{#DC,#05}&
  int_to_bytes(data + CENTRE_Y)&{#DD,#1D}&int_to_bytes(data + ROT_Y)&{#C3}},
  --Inversion
  {"Inversion",
  {#DD,#05}&int_to_bytes(data + ROT_X)&{#DC,#25}&
  int_to_bytes(data + CENTRE_X)&{#DD,#1D}&int_to_bytes(data + WRP_X)&{#DD,
  #05}&int_to_bytes(data + ROT_Y)&{#DC,#25}&int_to_bytes(data + CENTRE_Y)&{
  #DD,#1D}&int_to_bytes(data + WRP_Y)&{#DD,#05}&int_to_bytes(data + WRP_Y)&{
  #DD,#05}&int_to_bytes(data + WRP_X)&{#D9,#F3,#DD,#1D}&
  int_to_bytes(data + PNT_X)&{#DD,#05}&int_to_bytes(data + WRP_X)&{#D8,#C8,
  #DD,#05}&int_to_bytes(data + WRP_Y)&{#D8,#C8,#DE,#C1,#D9,#FA,#DD,#1D}&
  int_to_bytes(data + PNT_Y)&{#DD,#05}&int_to_bytes(data + PNT_X)&{#D9,#FF,
  #DD,#05}&int_to_bytes(data + SIZE_X)&{#DC,#0D}&int_to_bytes(data + HALF)&{
  #DD,#05}&int_to_bytes(data + PNT_Y)&{#DC,#0D}&int_to_bytes(data + PT8)&{#DE,
  #E9,#DE,#C9,#DC,#05}&int_to_bytes(data + CENTRE_X)&{#DD,#1D}&
  int_to_bytes(data + ROT_X)&{#DD,#05}&int_to_bytes(data + PNT_X)&{#D9,#FE,
  #DD,#05}&int_to_bytes(data + SIZE_Y)&{#DC,#0D}&int_to_bytes(data + HALF)&{
  #DD,#05}&int_to_bytes(data + PNT_Y)&{#DC,#0D}&int_to_bytes(data + PT8)&{#DE,
  #E9,#DE,#C9,#DC,#05}&int_to_bytes(data + CENTRE_Y)&{#DD,#1D}&
  int_to_bytes(data + ROT_Y)&{#C3}}
}


----------------------------------------------------------------------------
-- Fractal definitions --

----==## Bifurication ##==----
function  Bifurication_init(sequence param)
  return {{1.09 / (param[MAXITER] + 1000)},  --coef
          {2.9, 0.4},               --seed
          param[CORNERS],           --edge
          rnd(.01, .1),             --spot
          rnd(.5, 5),               --dist
          0}                        --angle
end function
params &= {{"Bifurication",         --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{2.9,0},{4,1}},                  --coners
  {1, 1, 1, 1},                     --colour
  {3.45, 0.5},                      --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Bifurication_init"),  --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + A)&{#DD,
  #1D}&int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + OLD_Y)&{#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#D9,#E8,#DC,#25}&int_to_bytes(data + OLD_Y)&{
  #DE,#C9,#DD,#1D}&int_to_bytes(data + Y)&{#C3}}}


----==## Binary ##==----
function  Binary_init(sequence param)
  return {{.5,  .0, .0, .5, -2.563477, -0.000003, .33,
           .5,  .0, .0, .5, 2.436544, -0.000003, .33,
           .0, -.5, .5, .0, 4.873085,  7.563492, .34},  --coef
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          rand(4) * 90}             --angle
end function
params &= {{"Binary",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Binary_init"),        --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Chip ##==----
function  Chip_init(sequence param)
  return {{rnd(-50, 50), rnd(-50, 50), rnd(-50, 50)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0005, .002),         --spot
          rnd(.0005, .002),         --dist
          0}                        --angle
end function
params &= {{"Chip",                  --name
  STANDARD,                         --flags
  1,                                --enable
  2000000,                           --iterations
  {{-560,-420},{560,420}},          --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Chip_init"),          --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#9E,#74,#08,#D9,#C0,
  #D9,#E1,#DE,#F1,#EB,#04,#DD,#D8,#D9,#E8,#DD,#05}&int_to_bytes(data + B)&{
  #DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,#25}&int_to_bytes(data + C)&{#D9,
  #E1,#D9,#E8,#D9,#C9,#D9,#F1,#D9,#EA,#DE,#F9,#D8,#C8,#D9,#FF,#DE,#C9,#DD,
  #05}&int_to_bytes(data + C)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,#25}&
  int_to_bytes(data + B)&{#D9,#E1,#D9,#E8,#D9,#C9,#D9,#F1,#D9,#EA,#DE,#F9,#D8,
  #C8,#D9,#E8,#D9,#F3,#DE,#C9,#DC,#2D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&
  int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#C3}}}


----==## ComplexIFS ##==----
function  ComplexIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"ComplexIFS",            --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("ComplexIFS_init"),    --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_Y)&{#DD,#05}&int_to_bytes(data + OLD_X)&{
  #D9,#F3,#DC,#35}&int_to_bytes(data + TWO)&{#DD,#1D}&
  int_to_bytes(data + C5)&{#DD,#05}&int_to_bytes(data + OLD_X)&{#D8,#C8,#DD,
  #05}&int_to_bytes(data + OLD_Y)&{#D8,#C8,#DE,#C1,#D9,#E1,#D9,#FA,#D9,#FA,
  #DD,#1D}&int_to_bytes(data + D5)&{#DD,#05}&int_to_bytes(data + C5)&{#D9,#FF,
  #DC,#0D}&int_to_bytes(data + D5)&{#DD,#1D}&int_to_bytes(data + A5)&{#DD,
  #05}&int_to_bytes(data + C5)&{#D9,#FE,#DC,#0D}&int_to_bytes(data + D5)&{#DD,
  #1D}&int_to_bytes(data + B5)&{#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,
  #03,#DC,#0D}&int_to_bytes(data + A5)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + B5)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + A5)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + B5)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Coral ##==----
function  Coral_init(sequence param)
  return {{.307692, -.531469, -.461538, -.293706,  5.401953, 8.655175, .40,
           .307692, -.076923,  .153846, -.447552, -1.295248, 4.152990, .15,
           .000000,  .545455,  .692308, -.195804, -4.893637, 7.269794, .45},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          rand(360)}                --angle
end function
params &= {{"Coral",                 --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Coral_init"),        --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Crystal ##==----
function  Crystal_init(sequence param)
  return {{.696970, -.481061, -.393939, -.662879, 2.147003, 10.310288, .75,
           .090909, -.443182,  .515152, -.094697, 4.286558,  2.925762, .25},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          rand(360)}                --angle
end function
params &= {{"Crystal",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Crystal_init"),       --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Dragon ##==----
function  Dragon_init(sequence param)
  return {{.824074, .281482, -.212346,  .864198, -1.882290, -0.110607, .79,
           .088272, .520988, -.463889, -.377778,  0.785360,  8.095795, .21},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, 1),              --dist
          160 + rand(40)}           --angle
end function
params &= {{"Dragon",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Dragon_init"),        --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Fern ##==----
function  Fern_init(sequence param)
  return {{0,     0,     0,     0.16,  0,     0,     .01,
           0.85,  0.04,  -0.04, 0.85,  0,     1.6,   .85,
           0.2,   -0.26, 0.23,  0.22,  0,     1.6,   .07,
           -0.15, 0.28,  0.26,  0.24,  0,     0.44,  .07},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, 1),              --dist
          rand(360)}                --angle
end function
params &= {{"Fern",                 --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Fern_init"),          --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Floor ##==----
function  Floor_init(sequence param)
  return {{.0, -.5,  .5, .0, -1.732366, 3.366182, .33,
           .5,  .0,  .0, .5, -0.027891, 5.014877, .33,
           .0,  .5, -.5, .0,  1.620804, 3.310401, .34},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          135 + rand(90)}           --angle
end function
params &= {{"Floor",                --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Floor_init"),         --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Gingerbread ##==----
function  Gingerbread_init(sequence param)
  return {{rnd(.9, 1.1)},--coef
          {rnd(-3, 3), rnd(-3, 3)}, --seed
          param[CORNERS],           --edge
          rnd(.01, .1),             --spot
          rnd(.025, .075),          --dist
          rand(90)}                 --angle
end function
params &= {{"Gingerbread",           --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  2000000,                           --iterations
  0.2,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.002, 0,                      --project, lyap, dim
  routine_id("Gingerbread_init"),   --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#D9,#E8,#DC,#25}&int_to_bytes(data + OLD_Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#E1,#DC,#0D}&int_to_bytes(data + A)&{#DE,
  #C1,#DD,#1D}&int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + OLD_X)&{
  #DD,#1D}&int_to_bytes(data + Y)&{#C3}}}


----==## GingerPower ##==----
function  GingerPower_init(sequence param)
  return {{rnd(-25, 25), pn() * rnd(2, 5), rnd(-50, 50), rnd(.7, .9)},--coef
          {rnd(-3, 3), rnd(-3, 3)}, --seed
          param[CORNERS],           --edge
          .00001,                   --spot
          .0005,                    --dist
          -rand(90)}                --angle
end function
params &= {{"GingerPower",          --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  2000000,                           --iterations
  0.4,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("GingerPower_init"),   --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + E)&{#DD,
  #1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + D)&{#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&
  int_to_bytes(data + TEMP_X)&{#DC,#25}&int_to_bytes(data + C)&{#D9,#E1,#D9,
  #E8,#D9,#C9,#D9,#E1,#D9,#F1,#DE,#C9,#D9,#C0,#D9,#FC,#D9,#C9,#D8,#E1,#D9,#F0,
  #D9,#E8,#DE,#C1,#D9,#C9,#D9,#E8,#D9,#FD,#DD,#D9,#DE,#C9,#DC,#2D}&
  int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{#C3}}}


----==## Henon ##==----
function  Henon_init(sequence param)
  return {{rnd(1.38, 1.41), 0.3},   --coef
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.5, 1),               --dist
          rand(360)}                --angle
end function
params &= {{"Henon",                --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  100000,                            --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Henon_init"),         --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#D9,#E8,#DC,#05}&int_to_bytes(data + OLD_Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D8,#C8,#DC,#0D}&int_to_bytes(data + A)&{#DE,
  #E9,#DD,#1D}&int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + B)&{#DC,
  #0D}&int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#C3}}}


----==## Hopalong ##==----
function  Hopalong_init(sequence param)
  sequence ret
  ret =  {{rnd(-50, 50), rnd(-50, 50), rnd(-50, 50), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0005, .002),         --spot
          rnd(.0005, .004),         --dist
          315 + (rand(100) > 50) * 180} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][2]) + abs(ret[1][3])) / 3
  return ret        
end function
params &= {{"Hopalong",              --name
  STANDARD,                         --flags
  1,                                --enable
  2000000,                           --iterations
  {{-20, -16}, {20, 16}},           --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Hopalong_init"),      --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + D)&{#DD,
  #1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#DD,#D8,#9E,#77,#24,#DD,#05}&
  int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + TEMP_X)&{#DC,#25}&
  int_to_bytes(data + C)&{#D9,#E1,#D9,#FA,#DC,#05}&
  int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{#EB,#22,#DD,
  #05}&int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + TEMP_X)&{#DC,#25}&
  int_to_bytes(data + C)&{#D9,#E1,#D9,#FA,#DC,#2D}&
  int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{#C3}}}


----==## HopLinear ##==----
function  HopLinear_init(sequence param)
  sequence ret
  ret =  {{rnd(-33, 33), rnd(0, 0.4), rnd(-25, 25), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0005, .002),         --spot
          rnd(.0005, .004),         --dist
          315 + (rand(100) > 50) * 180} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][3])) / 2
  return ret        
end function
params &= {{"HopLinear",            --name
  STANDARD,                         --flags
  1,                                --enable
  2000000,                           --iterations
  {{-20, -16}, {20, 16}},           --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopLinear_init"),     --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + D)&{#DD,
  #1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#DD,#D8,#9E,#73,#20,#DD,#05}&
  int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + TEMP_X)&{#DC,#25}&
  int_to_bytes(data + C)&{#DC,#05}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&
  int_to_bytes(data + X)&{#EB,#1E,#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&
  int_to_bytes(data + TEMP_X)&{#DC,#25}&int_to_bytes(data + C)&{#DC,#2D}&
  int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{#C3}}}


----==## HopLog ##==----
function  HopLog_init(sequence param)
  return  {{rnd(-25, 25), 
           pn() * power(10, rnd(6, 30)), 
           pn() * power(10, rnd(0, 9)), 
           rnd(-100, 100)},         --coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.001, .003),          --spot
          rnd(.0005, .004),         --dist
          315 + (rand(100) > 50) * 180} --angle
end function
params &= {{"HopLog",               --name
  STANDARD,                         --flags
  1,                                --enable
  2000000,                           --iterations
  {{-200, -160}, {200, 160}},       --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopLog_init"),        --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + D)&{#DD,
  #1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#DD,#D8,#9E,#77,#2C,#DD,#05}&
  int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + TEMP_X)&{#DC,#25}&
  int_to_bytes(data + C)&{#D9,#E1,#D9,#E8,#D9,#C9,#D9,#F1,#D9,#E9,#DE,#F9,#DC,
  #2D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{#EB,#2A,
  #DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + TEMP_X)&{#DC,
  #25}&int_to_bytes(data + C)&{#D9,#E1,#D9,#E8,#D9,#C9,#D9,#F1,#D9,#E9,#DE,
  #F9,#DC,#05}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{
  #C3}}}


----==## HopPower ##==----
function  HopPower_init(sequence param)
  sequence ret
  ret =  {{rnd(-25, 25), pn() * rnd(2, 5), 
           rnd(-50, 50), rnd(.7, .9), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0001, .0002),        --spot
          rnd(.0001, .001),         --dist
          315 + (rand(100) > 50) * 180} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][2]) + abs(ret[1][3])) / 3
  return ret        
end function
params &= {{"HopPower",             --name
  STANDARD,                         --flags
  1,                                --enable
  2000000,                           --iterations
  {{-200, -320}, {200, 320}},       --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopPower_init"),      --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + E)&{#DD,
  #1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#DD,#D8,#9E,#77,#4A,#DD,#05}&
  int_to_bytes(data + D)&{#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&
  int_to_bytes(data + TEMP_X)&{#DC,#25}&int_to_bytes(data + C)&{#D9,#E1,#D9,
  #E8,#D9,#C9,#D9,#E1,#D9,#F1,#DE,#C9,#D9,#C0,#D9,#FC,#D9,#C9,#D8,#E1,#D9,#F0,
  #D9,#E8,#DE,#C1,#D9,#C9,#D9,#E8,#D9,#FD,#DD,#D9,#DE,#C9,#DC,#2D}&
  int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{#EB,#48,#DD,
  #05}&int_to_bytes(data + D)&{#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&
  int_to_bytes(data + TEMP_X)&{#DC,#25}&int_to_bytes(data + C)&{#D9,#E1,#D9,
  #E8,#D9,#C9,#D9,#E1,#D9,#F1,#DE,#C9,#D9,#C0,#D9,#FC,#D9,#C9,#D8,#E1,#D9,#F0,
  #D9,#E8,#DE,#C1,#D9,#C9,#D9,#E8,#D9,#FD,#DD,#D9,#DE,#C9,#DC,#05}&
  int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{#C3}}}


----==## HopRoot ##==----
function  HopRoot_init(sequence param)
  sequence ret
  ret =  {{pn() * rnd(50, 500), rnd(1, 9), rnd(-5, 5), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          .0001,                    --spot
          .001,                     --dist
          315 + (rand(100) > 50) * 45} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][3])) / 2
  return ret        
end function
params &= {{"HopRoot",              --name
  STANDARD,                         --flags
  1,                                --enable
  2000000,                           --iterations
  {{-7, -5}, {9, 7}},               --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopRoot_init"),       --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + D)&{#DD,
  #1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#DD,#D8,#9E,#73,#24,#DD,#05}&
  int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + TEMP_X)&{#DC,#25}&
  int_to_bytes(data + C)&{#D9,#E1,#D9,#FA,#DC,#2D}&
  int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{#EB,#20,#DD,
  #05}&int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + TEMP_X)&{#D9,#FE,
  #DC,#05}&int_to_bytes(data + OLD_Y)&{#DC,#25}&int_to_bytes(data + C)&{#DD,
  #1D}&int_to_bytes(data + X)&{#C3}}}


----==## HopSine ##==----
function  HopSine_init(sequence param)
  sequence ret
  ret =  {{rnd(-5, 5), rnd(.3, .4), rnd(-500, 500), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.00005, .0001),       --spot
          rnd(.00005, .0004),       --dist
          315 + (rand(100) > 50) * 180} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][3])) / 2
  return ret        
end function
params &= {{"HopSine",              --name
  STANDARD,                         --flags
  1,                                --enable
  2000000,                           --iterations
  {{-40, -32}, {40, 32}},           --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopSine_init"),       --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + D)&{#DD,
  #1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#DD,#D8,#9E,#73,#20,#DD,#05}&
  int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + TEMP_X)&{#DC,#25}&
  int_to_bytes(data + C)&{#DC,#2D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&
  int_to_bytes(data + X)&{#EB,#20,#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&
  int_to_bytes(data + TEMP_X)&{#D9,#FE,#DC,#05}&int_to_bytes(data + OLD_Y)&{
  #DC,#25}&int_to_bytes(data + C)&{#DD,#1D}&int_to_bytes(data + X)&{#C3}}}


----==## HopTrig ##==----
function  HopTrig_init(sequence param)
  sequence ret
  ret =  {{rnd(-1, 1), rnd(.5, .85), rnd(-1, 1), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0005, .002),         --spot
          rnd(.0005, .004),         --dist
          315 + (rand(100) > 50) * 45} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][3])) / 2
  return ret        
end function
params &= {{"HopTrig",              --name
  STANDARD,                         --flags
  1,                                --enable
  2000000,                           --iterations
  {{-200, -160}, {200, 160}},       --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopTrig_init"),       --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + D)&{#DD,
  #1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#DD,#D8,#9E,#73,#22,#DD,#05}&
  int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + TEMP_X)&{#D9,#FE,#DC,
  #2D}&int_to_bytes(data + OLD_Y)&{#DC,#25}&int_to_bytes(data + C)&{#DD,#1D}&
  int_to_bytes(data + X)&{#EB,#20,#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&
  int_to_bytes(data + TEMP_X)&{#D9,#FE,#DC,#05}&int_to_bytes(data + OLD_Y)&{
  #DC,#25}&int_to_bytes(data + C)&{#DD,#1D}&int_to_bytes(data + X)&{#C3}}}


----==## Icons3D ##==----
function  Icons3D_init(sequence param)
  return {{-2.34, 2.04, 0.2, rnd(.09, .1) * (rand(100)>50), 
           rnd(-0.6, 0.6), rand(3)},--coef
          {0.01, 0.003, 0},         --seed
          param[CORNERS],           --edge
          rnd(.5, 1),               --spot
          rnd(.5, 1),               --dist
          rand(360)}                --angle
end function
params &= {{"Icons3D",               --name
  STANDARD + THREED + PIXLIST,      --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 0},                     --colour
  {0, 0},                           --origin
  1, 0.1, 0,                        --project, lyap, dim
  routine_id("Icons3D_init"),       --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#D8,#C8,#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D8,#C8,#DE,#C1,#DD,#1D}&
  int_to_bytes(data + A1)&{#DD,#05}&int_to_bytes(data + OLD_X)&{#DD,#1D}&
  int_to_bytes(data + B1)&{#DD,#05}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&
  int_to_bytes(data + C1)&{#DD,#05}&int_to_bytes(data + F)&{#DB,#1D}&
  int_to_bytes(data + FLOOR)&{#A1}&int_to_bytes(data + FLOOR)&{#DD,#05}&
  int_to_bytes(data + B1)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DD,#05}&
  int_to_bytes(data + C1)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DE,#E9,#DD,
  #1D}&int_to_bytes(data + D1)&{#DD,#05}&int_to_bytes(data + C1)&{#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#DD,#05}&int_to_bytes(data + B1)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#DD,#1D}&int_to_bytes(data + E1)&{#DD,
  #05}&int_to_bytes(data + D1)&{#DD,#1D}&int_to_bytes(data + B1)&{#DD,#05}&
  int_to_bytes(data + E1)&{#DD,#1D}&int_to_bytes(data + C1)&{#48,#75,#A5,#DD,
  #05}&int_to_bytes(data + OLD_X)&{#DC,#0D}&int_to_bytes(data + B1)&{#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#DC,#0D}&int_to_bytes(data + C1)&{#DE,#E9,#DD,
  #1D}&int_to_bytes(data + D1)&{#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&
  int_to_bytes(data + A1)&{#DC,#05}&int_to_bytes(data + A)&{#DD,#05}&
  int_to_bytes(data + C)&{#DC,#0D}&int_to_bytes(data + D1)&{#DE,#C1,#DD,#1D}&
  int_to_bytes(data + E1)&{#DD,#05}&int_to_bytes(data + E1)&{#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#DD,#05}&int_to_bytes(data + D)&{#DC,#0D}&
  int_to_bytes(data + B1)&{#DE,#C1,#DD,#05}&int_to_bytes(data + E)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#E9,#DD,#1D}&int_to_bytes(data + X)&{#DD,
  #05}&int_to_bytes(data + E1)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DD,#05}&
  int_to_bytes(data + D)&{#DC,#0D}&int_to_bytes(data + C1)&{#DE,#E9,#DD,#05}&
  int_to_bytes(data + E)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DE,#C1,#DD,
  #1D}&int_to_bytes(data + Y)&{#DD,#05}&int_to_bytes(data + A1)&{#DD,#1D}&
  int_to_bytes(data + Z)&{#C3}}}


----==## Ikeda ##==----
function  Ikeda_init(sequence param)
  return {{rnd(-50, 50), 1, pn() * rnd(3.6, 50), pn() * rnd(3, 8)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.002, .02),           --spot
          rnd(.01, .05),            --dist
          (rand(100) > 50) * 180}   --angle
end function
params &= {{"Ikeda",                --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.01, 0,                       --project, lyap, dim
  routine_id("Ikeda_init"),         --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#D8,#C8,#D9,#E8,#DE,#C1,#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D8,#C8,#DE,#C1,#DC,#3D}&
  int_to_bytes(data + C)&{#DC,#2D}&int_to_bytes(data + A)&{#DD,#1D}&
  int_to_bytes(data + E)&{#DD,#05}&int_to_bytes(data + E)&{#D9,#FF,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#DD,#05}&int_to_bytes(data + E)&{#D9,#FE,#DC,
  #0D}&int_to_bytes(data + OLD_Y)&{#DE,#E9,#DC,#0D}&int_to_bytes(data + B)&{
  #DC,#05}&int_to_bytes(data + D)&{#DD,#1D}&int_to_bytes(data + X)&{#DD,#05}&
  int_to_bytes(data + E)&{#D9,#FE,#DC,#0D}&int_to_bytes(data + OLD_X)&{#DD,
  #05}&int_to_bytes(data + E)&{#D9,#FF,#DC,#0D}&int_to_bytes(data + OLD_Y)&{
  #DE,#C1,#DC,#0D}&int_to_bytes(data + B)&{#DD,#1D}&int_to_bytes(data + Y)&{
  #C3}}}


----==## Julia ##==----
function  Julia_init(sequence param)
  return {find_julia(rand(10) + 10, routine_id("get_mandel")),--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.05, .3),             --spot
          rnd(.05, .5),             --dist
          0}                        --angle
end function
params &= {{"Julia",                --name
  STANDARD + NOMIN,                 --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 0},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Julia_init"),         --init
  -- shuffle code --
  {#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E4,#DF,#E0,#DD,#D8,#80,#E4,
  #01,#88,#25}&int_to_bytes(data + P1)&{#C3},                
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#25}&int_to_bytes(data + A)&{#DD,
  #1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + OLD_Y)&{#DC,
  #25}&int_to_bytes(data + B)&{#DD,#1D}&int_to_bytes(data + TEMP_Y)&{#DD,#05}&
  int_to_bytes(data + TEMP_Y)&{#DD,#05}&int_to_bytes(data + TEMP_X)&{#D9,#F3,
  #DC,#35}&int_to_bytes(data + TWO)&{#DD,#1D}&int_to_bytes(data + C)&{#DD,
  #05}&int_to_bytes(data + TEMP_X)&{#D8,#C8,#DD,#05}&
  int_to_bytes(data + TEMP_Y)&{#D8,#C8,#DE,#C1,#D9,#E1,#D9,#FA,#D9,#FA,#DD,
  #1D}&int_to_bytes(data + D)&{#DD,#05}&int_to_bytes(data + C)&{#D9,#FF,#DC,
  #0D}&int_to_bytes(data + D)&{#DD,#1D}&int_to_bytes(data + X)&{#DD,#05}&
  int_to_bytes(data + C)&{#D9,#FE,#DC,#0D}&int_to_bytes(data + D)&{#DD,#1D}&
  int_to_bytes(data + Y)&{#80,#3D}&int_to_bytes(data + P1)&{#00,#74,#1C,#DD,
  #05}&int_to_bytes(data + X)&{#D9,#E0,#DD,#1D}&int_to_bytes(data + X)&{#DD,
  #05}&int_to_bytes(data + Y)&{#D9,#E0,#DD,#1D}&int_to_bytes(data + Y)&{#C3}}}


----==## Koch ##==----
function  Koch_init(sequence param)
  return {{.307692, -.000000,  .000000,  .294118,  4.119164, 1.604278, .15,
           .192308, -.205882,  .653846,  .088235, -0.688840, 5.978916, .25,
           .192308,  .205882, -.653846,  .088235,  0.668580, 5.962514, .25,
           .307692, -.000000,  .000000,  .294118, -4.136530, 1.604278, .15,
           .384615, -.000000,  .000000, -.294118, -0.007718, 2.941176, .20},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          (rand(100) > 50) * 180}   --angle
end function
params &= {{"Koch",                 --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Koch_init"),          --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## LinearIFS ##==----
function  LinearIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.02, .2),             --spot
          rnd(.05, .2),             --dist
          (rand(100) > 50) * 180}   --angle
end function
params &= {{"LinearIFS",            --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("LinearIFS_init"),     --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## LinTrigIFS ##==----
function  LinTrigIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.02, .2),             --spot
          rnd(.05, .2),             --dist
          (rand(100) > 50) * 180}   --angle
end function
params &= {{"LinTrigIFS",           --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("LinTrigIFS_init"),    --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D9,#FE,#DE,#C9,#DE,#C1,#83,#C3,#18,#DC,#03,#DD,
  #1D}&int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D9,#FF,#DE,#C9,#DE,#C1,#83,#C3,#10,#DC,#03,#DD,
  #1D}&int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Lissajous ##==----
function  Lissajous_init(sequence param)
  return {{rnd(-.000000000000000001, .000000000000000001), 
           rnd(-.000000000000000001, .000000000000000001), 
           rnd(-.000000000000000001, .000000000000000001), 
           rnd(-.000000000000000001, .000000000000000001), 
           rnd(1.402, 1.9999999999)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.05, .3),             --spot
          rnd(.05, .5),             --dist
          (rand(100) > 50) * 90}    --angle
end function
params &= {{"Lissajous",            --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 0},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Lissajous_init"),     --init
  -- shuffle code --
  {#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E4,#DF,#E0,#DD,#D8,#80,#E4,
  #01,#88,#25}&int_to_bytes(data + P1)&{#52,#A1}&
  int_to_bytes(data + RAND_SEED)&{#F7,#25}&int_to_bytes(data + RAND_MULT)&{
  #A3}&int_to_bytes(data + RAND_SEED)&{#5A,#DB,#05}&
  int_to_bytes(data + RAND_SEED)&{#DB,#05}&int_to_bytes(data + RAND_MAXINT)&{
  #DE,#F9,#D9,#E4,#DF,#E0,#DD,#D8,#80,#E4,#01,#88,#25}&
  int_to_bytes(data + P2)&{#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E4,#DF,#E0,#DD,#D8,#80,#E4,
  #01,#88,#25}&int_to_bytes(data + P3)&{#C3},
  -- orbit code --
  {#80,#3D}&int_to_bytes(data + P1)&{#00,#74,#26,#DD,#05}&
  int_to_bytes(data + OLD_X)&{#DC,#25}&int_to_bytes(data + A)&{#DD,#1D}&
  int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + OLD_Y)&{#DC,#25}&
  int_to_bytes(data + B)&{#DD,#1D}&int_to_bytes(data + TEMP_Y)&{#EB,#24,#DD,
  #05}&int_to_bytes(data + OLD_X)&{#DC,#25}&int_to_bytes(data + C)&{#DD,#1D}&
  int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + OLD_Y)&{#DC,#25}&
  int_to_bytes(data + D)&{#DD,#1D}&int_to_bytes(data + TEMP_Y)&{#80,#3D}&
  int_to_bytes(data + P2)&{#00,#74,#1C,#DD,#05}&int_to_bytes(data + TEMP_X)&{
  #D9,#E0,#DD,#1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&
  int_to_bytes(data + TEMP_Y)&{#D9,#E0,#DD,#1D}&int_to_bytes(data + TEMP_Y)&{
  #DD,#05}&int_to_bytes(data + TEMP_Y)&{#DD,#1D}&int_to_bytes(data + X)&{#DD,
  #05}&int_to_bytes(data + TEMP_X)&{#D8,#C8,#DC,#25}&int_to_bytes(data + E)&{
  #DD,#1D}&int_to_bytes(data + Y)&{#80,#3D}&int_to_bytes(data + P3)&{#00,#74,
  #0E,#DD,#05}&int_to_bytes(data + Y)&{#D9,#E0,#DD,#1D}&
  int_to_bytes(data + Y)&{#C3}}}


----==## Lorentz3D ##==----
function  Lorentz3D_init(sequence param)
  return {{rnd(3, 8), 15, rnd(.6, 2.5), 0.02},--coef
          {1, 1, 1},                --seed
          param[CORNERS],           --edge
          rnd(.01, .03),            --spot
          rnd(.1, 1),               --dist
          0}                        --angle
end function
params &= {{"Lorentz3D",            --name
  STANDARD + THREED + PIXLIST,      --flags
  1,                                --enable
  2000000,                           --iterations
  {{-15, 30}, {15, 0}},             --coners
  {1, 1, 1, 0},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Lorentz3D_init"),     --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + A)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,
  #0D}&int_to_bytes(data + D)&{#D9,#E0,#DC,#05}&int_to_bytes(data + OLD_X)&{
  #DD,#05}&int_to_bytes(data + A)&{#DC,#0D}&int_to_bytes(data + OLD_Z)&{#DC,
  #0D}&int_to_bytes(data + D)&{#DE,#C1,#DD,#1D}&int_to_bytes(data + X)&{#DD,
  #05}&int_to_bytes(data + C)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DC,#0D}&
  int_to_bytes(data + D)&{#D9,#E0,#DC,#05}&int_to_bytes(data + OLD_Y)&{#DD,
  #05}&int_to_bytes(data + OLD_X)&{#DC,#0D}&int_to_bytes(data + OLD_Z)&{#DC,
  #0D}&int_to_bytes(data + D)&{#DE,#C1,#DD,#1D}&int_to_bytes(data + Y)&{#DD,
  #05}&int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,#0D}&
  int_to_bytes(data + D)&{#DC,#05}&int_to_bytes(data + OLD_Z)&{#DD,#05}&
  int_to_bytes(data + OLD_Z)&{#DC,#0D}&int_to_bytes(data + D)&{#DE,#E9,#DD,
  #05}&int_to_bytes(data + OLD_Y)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,
  #0D}&int_to_bytes(data + D)&{#DE,#E9,#DD,#1D}&int_to_bytes(data + Z)&{#C3}}}


----==## Martin ##==----
function  Martin_init(sequence param)
  return {{rnd(3.07, 3.21)},        --coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.005, .01),           --spot
          rnd(.005, .025),          --dist
          (rand(100) > 50) * 45}    --angle
end function
params &= {{"Martin",               --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{-64, -64}, {64, 64}},           --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.01, 0,                       --project, lyap, dim
  routine_id("Martin_init"),        --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#D9,#FE,#DC,#2D}&
  int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{#DD,#05}&
  int_to_bytes(data + A)&{#DC,#25}&int_to_bytes(data + OLD_X)&{#DD,#1D}&
  int_to_bytes(data + Y)&{#C3}}}


----==## Nickfrac ##==----
function  Nickfrac_init(sequence param)
  return {{0.00, -0.11,  0.22,  0.00, -6.25, 4.84, .06,  --\
           0.00, -0.11,  0.22,  0.00, -5.00, 5.23, .03,  -- | 'N'
           0.08,  0.11, -0.25,  0.00, -6.70, 4.99, .04,  --/
           0.00,  0.08, -0.25,  0.02, -5.20, 4.95, .07,  ---- 'I'
           0.00,  0.11, -0.19,  0.00, -4.52, 4.99, .06,  --\
           0.09, -0.01,  0.00,  0.10, -3.48, 2.96, .03,  -- | 'C'
           -0.09, 0.00,  0.00, -0.11, -3.55, 7.10, .03,  --/
           0.00,  0.08, -0.25,  0.02, -2.60, 4.95, .07,  --\
           0.06,  0.11, -0.13,  0.00, -2.00, 4.15, .04,  -- | 'K'
           -0.06, 0.11, -0.13,  0.00, -2.00, 5.80, .04,  --/
           0.00, -0.11,  0.22,  0.00,  0.35, 4.84, .06,  --\
           0.11,  0.02,  0.00,  0.11,  0.30, 5.99, .03,  -- | 'F'
           0.06,  0.02,  0.00,  0.10,  0.35, 4.51, .02,  --/
           0.00, -0.11,  0.22,  0.00,  2.34, 4.84, .06,  --\
           0.08,  0.00,  0.00,  0.11,  2.10, 5.99, .02,  -- |
           0.00,  0.11, -0.08,  0.00,  2.30, 6.15, .02,  -- | 'R'
           -0.09, 0.00, -0.01, -0.13,  2.45, 5.94, .02,  -- |
           0.06,  0.11, -0.13,  0.00,  2.19, 4.15, .04,  --/
           0.03, -0.11,  0.23,  0.11,  4.56, 4.43, .07,  --\
           0.03,  0.11, -0.25,  0.00,  4.27, 4.99, .07,  -- | 'A'
           0.06,  0.00,  0.00,  0.11,  4.40, 4.46, .02,  --/
           0.00,  0.11, -0.19,  0.00,  5.32, 4.99, .06,  --\
           0.09, -0.01,  0.00,  0.10,  6.35, 2.96, .03,  -- | 'C'
           -0.09, 0.00,  0.00, -0.11,  6.35, 7.10, .03}, --/
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          rand(360)}                --angle
end function
params &= {{"Nickfrac",             --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Nickfrac_init"),      --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#DD,#05}&int_to_bytes(data + X)&{#D9,#E0,#DD,
  #1D}&int_to_bytes(data + X)&{#C3}}}


----==## Pickover3D ##==----
function  Pickover3D_init(sequence param)
  return {{rnd(1.8, 2.5), rnd(-1, 1), rnd(-1, 1), rnd(1.7, 4)},--coef
          {1, 1, 1},                --seed
          param[CORNERS],           --edge
          rnd(.05, .4),             --spot
          rnd(.05, .5),             --dist
          rand(360)}                --angle
end function
params &= {{"Pickover3D",           --name
  STANDARD + THREED + PIXLIST,      --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.3, 0,                        --project, lyap, dim
  routine_id("Pickover3D_init"),    --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + A)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#D9,
  #FE,#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{
  #D9,#FF,#DC,#0D}&int_to_bytes(data + OLD_Z)&{#DE,#E9,#DD,#1D}&
  int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + C)&{#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#D9,#FE,#DC,#0D}&int_to_bytes(data + OLD_Z)&{
  #DD,#05}&int_to_bytes(data + D)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#D9,
  #FF,#DE,#E9,#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#FE,#DD,#1D}&int_to_bytes(data + Z)&{#C3}}}


----==## PolyAbs ##==----
function  PolyAbs_init(sequence param)
  return {rand_strange(10),         --coef
          {0.05, 0.05, 0.05},       --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.1, 1),               --dist
          0}                        --angle
end function
params &= {{"PolyAbs",              --name
  STRANGE + THREED + PIXLIST,       --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("PolyAbs_init"),       --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,
  #05}&int_to_bytes(data + A)&{#DD,#05}&int_to_bytes(data + C)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#DD,#05}&int_to_bytes(data + OLD_X)&{
  #D9,#E1,#DC,#0D}&int_to_bytes(data + D)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D9,#E1,#DC,#0D}&int_to_bytes(data + E)&{#DE,
  #C1,#DD,#1D}&int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + G)&{#DC,
  #0D}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + F)&{#DD,#05}&
  int_to_bytes(data + A1)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + OLD_X)&{#D9,#E1,#DC,#0D}&int_to_bytes(data + B1)&{
  #DE,#C1,#DD,#05}&int_to_bytes(data + OLD_Y)&{#D9,#E1,#DC,#0D}&
  int_to_bytes(data + C1)&{#DE,#C1,#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D8,#C8,#DD,#05}&int_to_bytes(data + OLD_Y)&{
  #D8,#C8,#DE,#C1,#DD,#1D}&int_to_bytes(data + Z)&{#C3}}}


----==## PolyBoolean ##==----
function  PolyBoolean_init(sequence param)
  return {rand_strange(14),         --coef
          {0.05, 0.05, 0.05},       --seed
          param[CORNERS],           --edge
          rnd(.01, .03),            --spot
          rnd(.01, .1),             --dist
          0}                        --angle
end function
params &= {{"PolyBoolean",          --name
  STRANGE + THREED + PIXLIST,       --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("PolyBoolean_init"),   --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + D)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,
  #05}&int_to_bytes(data + HALF)&{#DB,#1D}&int_to_bytes(data + FLOOR)&{#A1}&
  int_to_bytes(data + FLOOR)&{#DD,#05}&int_to_bytes(data + E)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DC,#05}&int_to_bytes(data + HALF)&{#DB,#1D}&
  int_to_bytes(data + FLOOR)&{#23,#05}&int_to_bytes(data + FLOOR)&{#A3}&
  int_to_bytes(data + FLOOR)&{#D9,#05}&int_to_bytes(data + FLOOR)&{#DD,#1D}&
  int_to_bytes(data + A3)&{#DD,#05}&int_to_bytes(data + F)&{#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + HALF)&{#DB,#1D}&
  int_to_bytes(data + FLOOR)&{#A1}&int_to_bytes(data + FLOOR)&{#DD,#05}&
  int_to_bytes(data + G)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DC,#05}&
  int_to_bytes(data + HALF)&{#DB,#1D}&int_to_bytes(data + FLOOR)&{#23,#05}&
  int_to_bytes(data + FLOOR)&{#A3}&int_to_bytes(data + FLOOR)&{#DD,#05}&
  int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,#05}&
  int_to_bytes(data + A)&{#DD,#05}&int_to_bytes(data + C)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#DC,#05}&int_to_bytes(data + A3)&{#DB,
  #05}&int_to_bytes(data + FLOOR)&{#DE,#C1,#DD,#1D}&int_to_bytes(data + X)&{
  #DD,#05}&int_to_bytes(data + D1)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,
  #05}&int_to_bytes(data + HALF)&{#DB,#1D}&int_to_bytes(data + FLOOR)&{#A1}&
  int_to_bytes(data + FLOOR)&{#DD,#05}&int_to_bytes(data + E1)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DC,#05}&int_to_bytes(data + HALF)&{#DB,#1D}&
  int_to_bytes(data + FLOOR)&{#23,#05}&int_to_bytes(data + FLOOR)&{#A3}&
  int_to_bytes(data + FLOOR)&{#D9,#05}&int_to_bytes(data + FLOOR)&{#DD,#1D}&
  int_to_bytes(data + A3)&{#DD,#05}&int_to_bytes(data + F1)&{#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + HALF)&{#DB,#1D}&
  int_to_bytes(data + FLOOR)&{#A1}&int_to_bytes(data + FLOOR)&{#DD,#05}&
  int_to_bytes(data + G1)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DC,#05}&
  int_to_bytes(data + HALF)&{#DB,#1D}&int_to_bytes(data + FLOOR)&{#23,#05}&
  int_to_bytes(data + FLOOR)&{#A3}&int_to_bytes(data + FLOOR)&{#DD,#05}&
  int_to_bytes(data + B1)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,#05}&
  int_to_bytes(data + A1)&{#DD,#05}&int_to_bytes(data + C1)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#DC,#05}&int_to_bytes(data + A3)&{#DB,
  #05}&int_to_bytes(data + FLOOR)&{#DE,#C1,#DD,#1D}&int_to_bytes(data + Y)&{
  #DD,#05}&int_to_bytes(data + OLD_X)&{#D8,#C8,#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D8,#C8,#DE,#C1,#DD,#1D}&
  int_to_bytes(data + Z)&{#C3}}}


----==## PolyPower ##==----
function  PolyPower_init(sequence param)
  return {rand_strange(14),         --coef
          {0.05, 0.05, 0.05},       --seed
          param[CORNERS],           --edge
          rnd(.005, .05),           --spot
          rnd(.01, .1),             --dist
          0}                        --angle
end function
params &= {{"PolyPower",            --name
  STRANGE + THREED + PIXLIST,       --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("PolyPower_init"),     --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,
  #05}&int_to_bytes(data + A)&{#DD,#05}&int_to_bytes(data + C)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#DD,#05}&int_to_bytes(data + E)&{#DD,
  #05}&int_to_bytes(data + OLD_X)&{#D9,#E1,#D9,#E8,#D9,#C9,#D9,#E1,#D9,#F1,
  #DE,#C9,#D9,#C0,#D9,#FC,#D9,#C9,#D8,#E1,#D9,#F0,#D9,#E8,#DE,#C1,#D9,#C9,#D9,
  #E8,#D9,#FD,#DD,#D9,#DE,#C9,#DC,#0D}&int_to_bytes(data + D)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + G)&{#DD,#05}&int_to_bytes(data + OLD_Y)&{#D9,#E1,
  #D9,#E8,#D9,#C9,#D9,#E1,#D9,#F1,#DE,#C9,#D9,#C0,#D9,#FC,#D9,#C9,#D8,#E1,#D9,
  #F0,#D9,#E8,#DE,#C1,#D9,#C9,#D9,#E8,#D9,#FD,#DD,#D9,#DE,#C9,#DC,#0D}&
  int_to_bytes(data + F)&{#DE,#C1,#DD,#1D}&int_to_bytes(data + X)&{#DD,#05}&
  int_to_bytes(data + B1)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,#05}&
  int_to_bytes(data + A1)&{#DD,#05}&int_to_bytes(data + C1)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#DD,#05}&int_to_bytes(data + E1)&{#DD,
  #05}&int_to_bytes(data + OLD_X)&{#D9,#E1,#D9,#E8,#D9,#C9,#D9,#E1,#D9,#F1,
  #DE,#C9,#D9,#C0,#D9,#FC,#D9,#C9,#D8,#E1,#D9,#F0,#D9,#E8,#DE,#C1,#D9,#C9,#D9,
  #E8,#D9,#FD,#DD,#D9,#DE,#C9,#DC,#0D}&int_to_bytes(data + D1)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + G1)&{#DD,#05}&int_to_bytes(data + OLD_Y)&{#D9,#E1,
  #D9,#E8,#D9,#C9,#D9,#E1,#D9,#F1,#DE,#C9,#D9,#C0,#D9,#FC,#D9,#C9,#D8,#E1,#D9,
  #F0,#D9,#E8,#DE,#C1,#D9,#C9,#D9,#E8,#D9,#FD,#DD,#D9,#DE,#C9,#DC,#0D}&
  int_to_bytes(data + F1)&{#DE,#C1,#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D8,#C8,#DD,#05}&int_to_bytes(data + OLD_Y)&{
  #D8,#C8,#DE,#C1,#DD,#1D}&int_to_bytes(data + Z)&{#C3}}}


----==## PolyTrig ##==----
function  PolyTrig_init(sequence param)
  return {rand_strange(6),          --coef
          {0.05, 0.05, 0.05},       --seed
          param[CORNERS],           --edge
          rnd(.001, .005),          --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"PolyTrig",             --name
  STRANGE + THREED + PIXLIST,       --flags
  1,                                --enable
  2000000,                           --iterations
  0.3,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("PolyTrig_init"),      --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + TEN)&{#DC,#0D}&int_to_bytes(data + F)&{#DC,
  #05}&int_to_bytes(data + THIRTEEN)&{#D9,#EB,#DE,#F1,#DD,#1D}&
  int_to_bytes(data + A1)&{#DD,#05}&int_to_bytes(data + A1)&{#D9,#FE,#DD,#1D}&
  int_to_bytes(data + B1)&{#DD,#05}&int_to_bytes(data + A1)&{#D9,#FF,#DD,#1D}&
  int_to_bytes(data + C1)&{#DD,#05}&int_to_bytes(data + C)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DC,#05}&int_to_bytes(data + D)&{#D9,#FE,#DC,
  #0D}&int_to_bytes(data + B)&{#DC,#05}&int_to_bytes(data + OLD_X)&{#DD,#1D}&
  int_to_bytes(data + D1)&{#DD,#05}&int_to_bytes(data + TEN)&{#DC,#0D}&
  int_to_bytes(data + A)&{#DD,#05}&int_to_bytes(data + D1)&{#DC,#0D}&
  int_to_bytes(data + C1)&{#DE,#C1,#DD,#05}&int_to_bytes(data + OLD_Y)&{#DC,
  #0D}&int_to_bytes(data + B1)&{#DE,#C1,#DD,#1D}&int_to_bytes(data + X)&{#DD,
  #05}&int_to_bytes(data + TEN)&{#DC,#0D}&int_to_bytes(data + E)&{#DD,#05}&
  int_to_bytes(data + D1)&{#DC,#0D}&int_to_bytes(data + B1)&{#DE,#E9,#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#DC,#0D}&int_to_bytes(data + C1)&{#DE,#C1,#DD,
  #1D}&int_to_bytes(data + Y)&{#DD,#05}&int_to_bytes(data + OLD_X)&{#D8,#C8,
  #DD,#05}&int_to_bytes(data + OLD_Y)&{#D8,#C8,#DE,#C1,#DD,#1D}&
  int_to_bytes(data + Z)&{#C3}}}


----==## Popcorn ##==----
function  Popcorn_init(sequence param)
  return {{rnd(.03, .07)},          --coef
          {0.0001, 0.0001},         --seed
          param[CORNERS],           --edge
          rnd(.01, .05),            --spot
          rnd(2, 8),                --dist
          (rand(100) > 50) * 45}    --angle
end function
params &= {{"Popcorn",              --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  500000,                           --iterations
  {{-25, -18}, {25, 18}},           --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.1, 0.1,                      --project, lyap, dim
  routine_id("Popcorn_init"),       --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + CONST3)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{
  #D9,#F2,#DD,#D8,#DC,#05}&int_to_bytes(data + OLD_Y)&{#D9,#FE,#DC,#0D}&
  int_to_bytes(data + A)&{#DC,#2D}&int_to_bytes(data + OLD_X)&{#DD,#1D}&
  int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + CONST3)&{#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#D9,#F2,#DD,#D8,#DC,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#FE,#DC,#0D}&int_to_bytes(data + A)&{#DC,
  #05}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + Y)&{#C3}}}


----==## PowerIFS ##==----
function  PowerIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"PowerIFS",             --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("PowerIFS_init"),      --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#D8,#C8,#DD,#1D}&
  int_to_bytes(data + A5)&{#DD,#05}&int_to_bytes(data + OLD_Y)&{#D8,#C8,#DD,
  #1D}&int_to_bytes(data + B5)&{#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,
  #03,#DC,#0D}&int_to_bytes(data + A5)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + B5)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + A5)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + B5)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Quadrature ##==----
function  Quadrature_init(sequence param)
  return {{rnd(-50, 50), rnd(-50, 50), rnd(-50, 50)}, --coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.005, .01),           --spot
          rnd(.005, .01),           --dist
          (rand(100) > 50) * 45}    --angle
end function
params &= {{"Quadrature",           --name
  STANDARD,                         --flags
  1,                                --enable
  1000000,                           --iterations
  {{-82, -55}, {112, 90}},          --coners
  {1, 1, 1, 1},                     --colour
  {15, 17.5},                       --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Quadrature_init"),    --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#9E,#74,#08,#D9,#C0,
  #D9,#E1,#DE,#F1,#EB,#04,#DD,#D8,#D9,#E8,#DD,#05}&int_to_bytes(data + B)&{
  #DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,#25}&int_to_bytes(data + C)&{#D9,
  #E1,#D9,#E8,#D9,#C9,#D9,#F1,#D9,#EA,#DE,#F9,#D9,#FE,#DE,#C9,#DD,#05}&
  int_to_bytes(data + C)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,#25}&
  int_to_bytes(data + B)&{#D9,#E1,#D9,#E8,#D9,#C9,#D9,#F1,#D9,#EA,#DE,#F9,#D8,
  #C8,#D9,#E8,#D9,#F3,#DE,#C9,#DC,#2D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&
  int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#C3}}}


----==## RootIFS ##==----
function  RootIFS_init(sequence param)
  return {rand_ifs(4),             --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"RootIFS",              --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("RootIFS_init"),       --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#9E,#74,#08,#D9,#C0,
  #D9,#E1,#DE,#F1,#EB,#04,#DD,#D8,#D9,#E8,#DD,#1D}&int_to_bytes(data + C5)&{
  #DD,#05}&int_to_bytes(data + OLD_X)&{#D9,#E1,#D9,#FA,#DC,#0D}&
  int_to_bytes(data + C5)&{#DD,#1D}&int_to_bytes(data + A5)&{#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D9,#E4,#DF,#E0,#9E,#74,#08,#D9,#C0,#D9,#E1,#DE,
  #F1,#EB,#04,#DD,#D8,#D9,#E8,#DD,#1D}&int_to_bytes(data + D5)&{#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D9,#E1,#D9,#FA,#DC,#0D}&
  int_to_bytes(data + D5)&{#DD,#1D}&int_to_bytes(data + B5)&{#53,#8B,#1D}&
  int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&int_to_bytes(data + A5)&{#83,
  #C3,#08,#DD,#03,#DC,#0D}&int_to_bytes(data + B5)&{#DE,#C1,#83,#C3,#18,#DC,
  #03,#DD,#1D}&int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + A5)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + B5)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Rorschach ##==----
function  Rorschach_init(sequence param)
  return {{rnd(-50, 50), rnd(-50, 50)}, --coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.001, .005),          --spot
          rnd(.0005, .005),         --dist
          45}                       --angle
end function
params &= {{"Rorschach",            --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  1000000,                           --iterations
  {{-300, -240}, {300, 240}},       --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.2, 0.3,                      --project, lyap, dim
  routine_id("Rorschach_init"),     --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + A)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#D9,
  #FE,#DC,#2D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + X)&{
  #DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#D9,
  #FF,#DC,#2D}&int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{
  #C3}}}


----==## Sierpinski ##==----
function  Sierpinski_init(sequence param)
  return {{0.5,   0,     0,     0.5,   1,     1,     .33,
           0.5,   0,     0,     0.5,   1,     50,    .33,
           0.5,   0,     0,     0.5,   50,    50,    .34},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.005, .007),          --spot
          rnd(.005, .01),           --dist
          (rand(4) - 1) * 90}       --angle
end function
params &= {{"Sierpinski",           --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {50, 50},                         --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Sierpinski_init"),    --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Spiral ##==----
function  Spiral_init(sequence param)
  return {{ .787879, -.424242, .242424, .859848,  1.758647, 1.408065, .90,
           -.121212,  .257576, .151515, .053030, -6.721654, 1.377236, .05,
            .181818, -.136364, .090909, .181818,  6.086107, 1.568035, .05},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, 1),              --dist
          rand(360)}                --angle
end function
params &= {{"Spiral",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Spiral_init"),        --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Strange2D ##==----
function  Strange2D_init(sequence param)
  return {rand_strange(12),         --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.1, .3),              --spot
          rnd(.1, .3),              --dist
          0}                        --angle
end function
params &= {{"Strange2D",            --name
  STRANGE + PIXLIST,                --flags
  1,                                --enable
  3000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("Strange2D_init"),     --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{
  #DD,#1D}&int_to_bytes(data + A2)&{#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,
  #0D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + B2)&{#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&
  int_to_bytes(data + C2)&{#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + A)&{#DD,#05}&
  int_to_bytes(data + C)&{#DC,#0D}&int_to_bytes(data + A2)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + D)&{#DC,#0D}&int_to_bytes(data + B2)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + E)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + F)&{#DC,#0D}&int_to_bytes(data + C2)&{#DE,#C1,#DD,
  #1D}&int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + B)&{#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + G)&{#DD,#05}&
  int_to_bytes(data + A1)&{#DC,#0D}&int_to_bytes(data + A2)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + B1)&{#DC,#0D}&int_to_bytes(data + B2)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + C1)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + D1)&{#DC,#0D}&int_to_bytes(data + C2)&{#DE,#C1,#DD,
  #1D}&int_to_bytes(data + Y)&{#C3}}}


----==## Strange3D ##==----
function  Strange3D_init(sequence param)
  return {rand_strange(60),         --coef
          {0.05, 0.05, 0.05},       --seed
          param[CORNERS],           --edge
          rnd(.5, .7),              --spot
          rnd(.2, .5),              --dist
          0}                        --angle
end function
params &= {{"Strange3D",            --name
  STRANGE + THREED + PIXLIST,       --flags
  1,                                --enable
  3000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("Strange3D_init"),     --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{
  #DD,#1D}&int_to_bytes(data + A9)&{#DD,#05}&int_to_bytes(data + A9)&{#DC,
  #0D}&int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + B9)&{#DD,#05}&
  int_to_bytes(data + A9)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&
  int_to_bytes(data + C9)&{#DD,#05}&int_to_bytes(data + A9)&{#DC,#0D}&
  int_to_bytes(data + OLD_Z)&{#DD,#1D}&int_to_bytes(data + D9)&{#DD,#05}&
  int_to_bytes(data + OLD_X)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&
  int_to_bytes(data + E9)&{#DD,#05}&int_to_bytes(data + E9)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + F9)&{#DD,#05}&
  int_to_bytes(data + E9)&{#DC,#0D}&int_to_bytes(data + OLD_Z)&{#DD,#1D}&
  int_to_bytes(data + G9)&{#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,#0D}&
  int_to_bytes(data + OLD_Z)&{#DD,#1D}&int_to_bytes(data + A10)&{#DD,#05}&
  int_to_bytes(data + A10)&{#DC,#0D}&int_to_bytes(data + OLD_Z)&{#DD,#1D}&
  int_to_bytes(data + B10)&{#DD,#05}&int_to_bytes(data + OLD_Y)&{#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DD,#1D}&int_to_bytes(data + C10)&{#DD,#05}&
  int_to_bytes(data + C10)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&
  int_to_bytes(data + D10)&{#DD,#05}&int_to_bytes(data + C10)&{#DC,#0D}&
  int_to_bytes(data + OLD_Z)&{#DD,#1D}&int_to_bytes(data + E10)&{#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#DC,#0D}&int_to_bytes(data + OLD_Z)&{#DD,#1D}&
  int_to_bytes(data + F10)&{#DD,#05}&int_to_bytes(data + F10)&{#DC,#0D}&
  int_to_bytes(data + OLD_Z)&{#DD,#1D}&int_to_bytes(data + G10)&{#DD,#05}&
  int_to_bytes(data + OLD_Z)&{#DC,#0D}&int_to_bytes(data + OLD_Z)&{#DD,#1D}&
  int_to_bytes(data + A11)&{#DD,#05}&int_to_bytes(data + A11)&{#DC,#0D}&
  int_to_bytes(data + OLD_Z)&{#DD,#1D}&int_to_bytes(data + B11)&{#DD,#05}&
  int_to_bytes(data + B)&{#DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,#05}&
  int_to_bytes(data + A)&{#DD,#05}&int_to_bytes(data + C)&{#DC,#0D}&
  int_to_bytes(data + A9)&{#DE,#C1,#DD,#05}&int_to_bytes(data + D)&{#DC,#0D}&
  int_to_bytes(data + B9)&{#DE,#C1,#DD,#05}&int_to_bytes(data + E)&{#DC,#0D}&
  int_to_bytes(data + C9)&{#DE,#C1,#DD,#05}&int_to_bytes(data + F)&{#DC,#0D}&
  int_to_bytes(data + D9)&{#DE,#C1,#DD,#05}&int_to_bytes(data + G)&{#DC,#0D}&
  int_to_bytes(data + E9)&{#DE,#C1,#DD,#05}&int_to_bytes(data + A1)&{#DC,#0D}&
  int_to_bytes(data + F9)&{#DE,#C1,#DD,#05}&int_to_bytes(data + B1)&{#DC,#0D}&
  int_to_bytes(data + G9)&{#DE,#C1,#DD,#05}&int_to_bytes(data + C1)&{#DC,#0D}&
  int_to_bytes(data + A10)&{#DE,#C1,#DD,#05}&int_to_bytes(data + D1)&{#DC,
  #0D}&int_to_bytes(data + B10)&{#DE,#C1,#DD,#05}&int_to_bytes(data + E1)&{
  #DC,#0D}&int_to_bytes(data + OLD_Y)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + F1)&{#DC,#0D}&int_to_bytes(data + C10)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + G1)&{#DC,#0D}&int_to_bytes(data + D10)&{#DE,#C1,
  #DD,#05}&int_to_bytes(data + A2)&{#DC,#0D}&int_to_bytes(data + E10)&{#DE,
  #C1,#DD,#05}&int_to_bytes(data + B2)&{#DC,#0D}&int_to_bytes(data + F10)&{
  #DE,#C1,#DD,#05}&int_to_bytes(data + C2)&{#DC,#0D}&
  int_to_bytes(data + G10)&{#DE,#C1,#DD,#05}&int_to_bytes(data + D2)&{#DC,
  #0D}&int_to_bytes(data + OLD_Z)&{#DE,#C1,#DD,#05}&int_to_bytes(data + E2)&{
  #DC,#0D}&int_to_bytes(data + A11)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + F2)&{#DC,#0D}&int_to_bytes(data + B11)&{#DE,#C1,#DD,
  #1D}&int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + A3)&{#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + G2)&{#DD,#05}&
  int_to_bytes(data + B3)&{#DC,#0D}&int_to_bytes(data + A9)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + C3)&{#DC,#0D}&int_to_bytes(data + B9)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + D3)&{#DC,#0D}&int_to_bytes(data + C9)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + E3)&{#DC,#0D}&int_to_bytes(data + D9)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + F3)&{#DC,#0D}&int_to_bytes(data + E9)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + G3)&{#DC,#0D}&int_to_bytes(data + F9)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + A4)&{#DC,#0D}&int_to_bytes(data + G9)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + B4)&{#DC,#0D}&int_to_bytes(data + A10)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + C4)&{#DC,#0D}&int_to_bytes(data + B10)&{#DE,#C1,
  #DD,#05}&int_to_bytes(data + D4)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{#DE,
  #C1,#DD,#05}&int_to_bytes(data + E4)&{#DC,#0D}&int_to_bytes(data + C10)&{
  #DE,#C1,#DD,#05}&int_to_bytes(data + F4)&{#DC,#0D}&
  int_to_bytes(data + D10)&{#DE,#C1,#DD,#05}&int_to_bytes(data + G4)&{#DC,
  #0D}&int_to_bytes(data + E10)&{#DE,#C1,#DD,#05}&int_to_bytes(data + A5)&{
  #DC,#0D}&int_to_bytes(data + F10)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + B5)&{#DC,#0D}&int_to_bytes(data + G10)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + C5)&{#DC,#0D}&int_to_bytes(data + OLD_Z)&{#DE,#C1,
  #DD,#05}&int_to_bytes(data + D5)&{#DC,#0D}&int_to_bytes(data + A11)&{#DE,
  #C1,#DD,#05}&int_to_bytes(data + E5)&{#DC,#0D}&int_to_bytes(data + B11)&{
  #DE,#C1,#DD,#1D}&int_to_bytes(data + Y)&{#DD,#05}&int_to_bytes(data + G5)&{
  #DC,#0D}&int_to_bytes(data + OLD_X)&{#DC,#05}&int_to_bytes(data + F5)&{#DD,
  #05}&int_to_bytes(data + A6)&{#DC,#0D}&int_to_bytes(data + A9)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + B6)&{#DC,#0D}&int_to_bytes(data + B9)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + C6)&{#DC,#0D}&int_to_bytes(data + C9)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + D6)&{#DC,#0D}&int_to_bytes(data + D9)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + E6)&{#DC,#0D}&int_to_bytes(data + E9)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + F6)&{#DC,#0D}&int_to_bytes(data + F9)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + G6)&{#DC,#0D}&int_to_bytes(data + G9)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + A7)&{#DC,#0D}&int_to_bytes(data + A10)&{#DE,#C1,
  #DD,#05}&int_to_bytes(data + B7)&{#DC,#0D}&int_to_bytes(data + B10)&{#DE,
  #C1,#DD,#05}&int_to_bytes(data + C7)&{#DC,#0D}&int_to_bytes(data + OLD_Y)&{
  #DE,#C1,#DD,#05}&int_to_bytes(data + D7)&{#DC,#0D}&
  int_to_bytes(data + C10)&{#DE,#C1,#DD,#05}&int_to_bytes(data + E7)&{#DC,
  #0D}&int_to_bytes(data + D10)&{#DE,#C1,#DD,#05}&int_to_bytes(data + F7)&{
  #DC,#0D}&int_to_bytes(data + E10)&{#DE,#C1,#DD,#05}&
  int_to_bytes(data + G7)&{#DC,#0D}&int_to_bytes(data + F10)&{#DE,#C1,#DD,
  #05}&int_to_bytes(data + A8)&{#DC,#0D}&int_to_bytes(data + G10)&{#DE,#C1,
  #DD,#05}&int_to_bytes(data + B8)&{#DC,#0D}&int_to_bytes(data + OLD_Z)&{#DE,
  #C1,#DD,#05}&int_to_bytes(data + C8)&{#DC,#0D}&int_to_bytes(data + A11)&{
  #DE,#C1,#DD,#05}&int_to_bytes(data + D8)&{#DC,#0D}&
  int_to_bytes(data + B11)&{#DE,#C1,#DD,#1D}&int_to_bytes(data + Z)&{#C3}}}


----==## Swirl ##==----
function  Swirl_init(sequence param)
  return {{ .745455, -.459091,  .406061,  .887121, 1.460279, 0.691072, .91,
           -.424242, -.065152, -.175758, -.218182, 3.809567, 6.741476, .09},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.05, .2),             --dist
          rand(360)}                --angle
end function
params &= {{"Swirl",                --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Swirl_init"),         --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## TanIFS ##==----
function  TanIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"TanIFS",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("TanIFS_init"),        --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#9E,#74,#08,#D9,#C0,
  #D9,#E1,#DE,#F1,#EB,#04,#DD,#D8,#D9,#E8,#DD,#1D}&int_to_bytes(data + C5)&{
  #DD,#05}&int_to_bytes(data + OLD_X)&{#D9,#E8,#D9,#F3,#DC,#0D}&
  int_to_bytes(data + C5)&{#DD,#1D}&int_to_bytes(data + A5)&{#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D9,#E4,#DF,#E0,#9E,#74,#08,#D9,#C0,#D9,#E1,#DE,
  #F1,#EB,#04,#DD,#D8,#D9,#E8,#DD,#1D}&int_to_bytes(data + D5)&{#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D9,#E8,#D9,#F3,#DC,#0D}&
  int_to_bytes(data + D5)&{#DD,#1D}&int_to_bytes(data + B5)&{#53,#8B,#1D}&
  int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&int_to_bytes(data + A5)&{#83,
  #C3,#08,#DD,#03,#DC,#0D}&int_to_bytes(data + B5)&{#DE,#C1,#83,#C3,#18,#DC,
  #03,#DD,#1D}&int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + A5)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + B5)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## Threeply ##==----
function  Threeply_init(sequence param)
  return {{rnd(-50, 50), rnd(-50, 50), rnd(-50, 50)}, --coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0001, .001),         --spot
          rnd(.0005, .001),         --dist
          315 + (rand(100) > 50) * 180} --angle
end function
params &= {{"Threeply",             --name
  STANDARD,                         --flags
  1,                                --enable
  1000000,                           --iterations
  {{-1000, -700}, {1000, 700}},     --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Threeply_init"),      --init
  -- shuffle code --
  {#C3},                               --shuffle
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_X)&{#D9,#E4,#DF,#E0,#9E,#74,#08,#D9,#C0,
  #D9,#E1,#DE,#F1,#EB,#04,#DD,#D8,#D9,#E8,#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#FE,#DD,#05}&int_to_bytes(data + B)&{#D9,
  #FF,#DE,#C9,#DC,#05}&int_to_bytes(data + C)&{#DD,#05}&
  int_to_bytes(data + A)&{#DC,#05}&int_to_bytes(data + B)&{#DC,#05}&
  int_to_bytes(data + C)&{#D9,#FE,#DC,#0D}&int_to_bytes(data + OLD_X)&{#DE,
  #E9,#D9,#E1,#DE,#C9,#DC,#2D}&int_to_bytes(data + OLD_Y)&{#DD,#1D}&
  int_to_bytes(data + X)&{#DD,#05}&int_to_bytes(data + A)&{#DC,#25}&
  int_to_bytes(data + OLD_X)&{#DD,#1D}&int_to_bytes(data + Y)&{#C3}}}


----==## Tree ##==----
function  Tree_init(sequence param)
  return {{0,     0,     0,     0.5,   0,     0,     .05,
           0.42,  -0.42, 0.42,  0.42,  0,     0.2,   .40,
           0.42,  0.42,  -0.42, 0.42,  0,     0.2,   .40,
           0.1,   0,     0,     0.1,   0,     0.2,   .15},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.5, 2),               --dist
          175 + rand(10)}           --angle
end function
params &= {{"Tree",                 --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, .25},                         --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Tree_init"),          --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


----==## TrigIFS ##==----
function  TrigIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"TrigIFS",              --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("TrigIFS_init"),       --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#FF,#DE,#C9,#83,#C3,#08,#DD,#03,#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D9,#FE,#DE,#C9,#DE,#C1,#83,#C3,#18,#DC,#03,#DD,
  #1D}&int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DD,#05}&
  int_to_bytes(data + OLD_X)&{#D9,#FE,#DE,#C9,#83,#C3,#08,#DD,#03,#DD,#05}&
  int_to_bytes(data + OLD_Y)&{#D9,#FF,#DE,#C9,#DE,#C1,#83,#C3,#10,#DC,#03,#DD,
  #1D}&int_to_bytes(data + Y)&{#5B,#C3}}}


----==## XJulia ##==----
function  XJulia_init(sequence param)
  return {find_julia(rand(10) + 10, routine_id("get_xmandel")),--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.05, .3),             --spot
          rnd(.05, .5),             --dist
          0}                        --angle
end function
params &= {{"XJulia",               --name
  STANDARD + NOMIN,                 --flags
  1,                                --enable
  2000000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 0},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("XJulia_init"),        --init
  -- shuffle code --
  {#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E4,#DF,#E0,#DD,#D8,#80,#E4,
  #01,#88,#25}&int_to_bytes(data + P1)&{#C3},                
  -- orbit code --
  {#DD,#05}&int_to_bytes(data + OLD_Y)&{#DC,#25}&int_to_bytes(data + A)&{#DD,
  #1D}&int_to_bytes(data + TEMP_X)&{#DD,#05}&int_to_bytes(data + OLD_X)&{#DC,
  #25}&int_to_bytes(data + B)&{#DD,#1D}&int_to_bytes(data + TEMP_Y)&{#DD,#05}&
  int_to_bytes(data + TEMP_Y)&{#DD,#05}&int_to_bytes(data + TEMP_X)&{#D9,#F3,
  #DC,#35}&int_to_bytes(data + TWO)&{#DD,#1D}&int_to_bytes(data + C)&{#DD,
  #05}&int_to_bytes(data + TEMP_X)&{#D8,#C8,#DD,#05}&
  int_to_bytes(data + TEMP_Y)&{#D8,#C8,#DE,#C1,#D9,#E1,#D9,#FA,#D9,#FA,#DD,
  #1D}&int_to_bytes(data + D)&{#DD,#05}&int_to_bytes(data + C)&{#D9,#FF,#DC,
  #0D}&int_to_bytes(data + D)&{#DD,#1D}&int_to_bytes(data + X)&{#DD,#05}&
  int_to_bytes(data + C)&{#D9,#FE,#DC,#0D}&int_to_bytes(data + D)&{#DD,#1D}&
  int_to_bytes(data + Y)&{#80,#3D}&int_to_bytes(data + P1)&{#00,#74,#1C,#DD,
  #05}&int_to_bytes(data + X)&{#D9,#E0,#DD,#1D}&int_to_bytes(data + X)&{#DD,
  #05}&int_to_bytes(data + Y)&{#D9,#E0,#DD,#1D}&int_to_bytes(data + Y)&{#C3}}}


----==## Zigzag ##==----
function  Zigzag_init(sequence param)
  return {{-.632407, -.614815, -.545370, .659259, 3.840822, 1.282321, .89,
           -.036111,  .444444,  .210185, .037037, 2.071081, 8.330552, .11},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          rand(360)}                --angle
end function
params &= {{"Zigzag",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Zigzag_init"),        --init
  -- shuffle code --
  {#53,#51,#52,#A1}&int_to_bytes(data + RAND_SEED)&{#F7,#25}&
  int_to_bytes(data + RAND_MULT)&{#A3}&int_to_bytes(data + RAND_SEED)&{#5A,
  #DB,#05}&int_to_bytes(data + RAND_SEED)&{#DB,#05}&
  int_to_bytes(data + RAND_MAXINT)&{#DE,#F9,#D9,#E8,#DE,#C1,#DC,#35}&
  int_to_bytes(data + TWO)&{#BB}&int_to_bytes(data + A)&{#83,#EB,#08,#B9}&
  int_to_bytes(data + RULES)&{#83,#C3,#38,#DC,#13,#DF,#E0,#9E,#72,#04,#DC,#23,
  #E2,#F2,#DD,#D8,#83,#EB,#30,#89,#1D}&int_to_bytes(data + RULE)&{#59,#5B,#C3},                         --shuffle
  -- orbit code --
  {#53,#8B,#1D}&int_to_bytes(data + RULE)&{#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#18,#DC,#03,#DD,#1D}&
  int_to_bytes(data + X)&{#83,#EB,#10,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_X)&{#83,#C3,#08,#DD,#03,#DC,#0D}&
  int_to_bytes(data + OLD_Y)&{#DE,#C1,#83,#C3,#10,#DC,#03,#DD,#1D}&
  int_to_bytes(data + Y)&{#5B,#C3}}}


num_frac = length(params)
-- End of fracasm.e --

