
--Orbital Fractal types v1.0 Nick Metcalfe 5/02

-----------------------------------------------------------------------------
--Variable interface

--Fractal 'params' structure names
global constant 
        NAME = 1,               --fractal name
        FLAGS = 2,              --sequence of general flags to control engine
        ENABLE = 3,             --enable flag
        MAXITER = 4,            --maximum iterations
        CORNERS = 5,            --fractal window corners or border for auto
        COLOUR = 6,             --sequence of flags for methods
        ORIGIN = 7,             --rotation origin
        PROJECT = 8,            --allow projections flag
        MIN_LYAP = 9,           --minimum lyapunov exponent
        MIN_DIM = 10,           --minimum fractal dimenson
        INIT = 11,              --initialisation routine_id
        SHUFFLE = 12,           --shuffle code
        ORBIT = 13              --orbit code
        
--Fractal 'values' structure names
global constant 
        COEF = 1,           --fractal coefficients
        SEED = 2,           --initial orbital seed
        EDGE = 3,           --fractal window corners or border for auto
        SPT = 4,            --spotlight value for methods
        DIS = 5,            --distance value for methods 
        ANGLE = 6           --rotation angle

--Colouring styles
global constant
        PERIOD = 1,
        DISTANCE = 2,
        ATTRACTOR = 3,
        TEXTURE = 4

--Binary flags to control engine
global constant
        STANDARD = 1,
        IFS      = 2,
        STRANGE  = 4,
        THREED   = 8,
        PIXLIST  = 16,
        NOMIN    = 32
        
global integer num_frac
global sequence params 
global sequence methods         --colouring methods
global sequence projections     --projection styles
params = {}
-----------------------------------------------------------------------------
--Helper, shared and generic functions and macros

constant freedom = 260
--return decimal random number between l & h
global function rnd(atom l, atom h)
  return (rand(freedom) - 1) / (freedom - 1) * (h - l) + l
end function
--pn() randomly returns +1 or -1
function pn()
  return (rand(100) < 50) * 2 + 1
end function   
--sign of r
function sgn(atom r)                 
  if r < 0 then return -1 else return 1 end if
end function
--absolute value of r
function abs(atom r)                 
  return sgn(r) * r                             
end function

--Strange attractor helpers
--rand_strange() returns random strange attractor coefficients
function rand_strange(integer num)
  sequence coef coef = {}
  for count = 1 to num do
    coef &= {rnd(-1, 1)}
  end for
  return coef  
end function

--IFS helpers
--weightings assigns IFS rule probabilities based on size
function weightings(sequence co)
  atom sum, mag
  sequence prob
  prob = {} sum = 0
  for ct = 0 to length(co) - 7 by 7 do
    mag = abs(co[ct+1] * co[ct+4] - co[ct+2] * co[ct+3])
    if mag = 0 or mag > 1 then return 0 end if
    prob &= mag sum += mag
  end for
  if not sum then return 0 end if
  for ct = 1 to length(prob) do
    co[ct * 7] = prob[ct] / sum
  end for
  return co
end function  
--rand_ifs() returns random IFS coefficients
function rand_ifs(integer max_rules)
  object result  result = 0
  while atom(result) do
    result = {}
    for ct = 1 to rand(max_rules - 1) + 1 do
      result &= {rnd(-1, 1), rnd(-1, 1), rnd(-1, 1), rnd(-1, 1), rnd(-1, 1), rnd(-1, 1), 0}
    end for
    result = weightings(result)
  end while
  return result
end function  

--Inverse julia method helpers
--get_mandel returns the iteration depth of a mandel orbit
function get_mandel(integer maxiter, object c)
  atom iter, x, y, x2, y2
  if atom(c) then return c end if
  iter = 0 x = 0 y = 0 x2 = 0 y2 = 0
  while iter < maxiter and x2 + y2 < 4 do
    x2 = x * x  y2 = y * y  iter += 1
    y = 2 * x * y + c[2]  x = x2 - y2 + c[1] 
  end while  
  return iter 
end function
--get_xmandel returns the iteration depth of an x-mandel orbit
function get_xmandel(integer maxiter, object c)
  atom iter, x, y, x2, y2
  if atom(c) then return c end if
  iter = 0 x = 0 y = 0 x2 = 0 y2 = 0
  while iter < maxiter and x2 + y2 < 4 do
    x2 = x * x  y2 = y * y  iter += 1
    x = 2 * x * y + c[2]  y = x2 - y2 + c[1] 
  end while  
  return iter 
end function
--find_julia returns a likely parameter near the edge of a mandel lake 
function find_julia(integer maxiter, atom id)
  object inside, outside, pos, count
  inside = 0 outside = maxiter
  while call_func(id, {maxiter, inside}) < maxiter do
    inside = {rnd(-1.5, 1.5), rnd(-1.5, 1.5)}
  end while
  while call_func(id, {maxiter, outside}) = maxiter do
    outside = {rnd(-1.5, 1.5), rnd(-1.5, 1.5)}
  end while
  count = 1
  while 1 do
    pos = (outside - inside) / 2 + inside
    if call_func(id, {maxiter, pos}) = maxiter then 
      inside = pos
      if count > maxiter then exit end if
    else 
      outside = pos 
    end if
    count += 1
  end while
  return pos  
end function

--@@  DO NOT REMOVE THIS LINE  used by the build functions
-----------------------------------------------------------------------------
--Colouring methods
methods = {
  {"Period", --Period
  parse("hue = sqrt(sqr(x - old_x) + sqr(y - old_y)) * dist + col : " &
        "saturation = 1 - hits : " &
        "intensity = (1 - (sqrt(sqr(x) + sqr(y)) * spot)) * hit_int")&
        "fldz "&
        "fcomp qword ptr[intensity] "&
        "fstsw ax "&
        "sahf " &
        "jb method_ok "&
        "  fldz " &
        "  fstp qword ptr[intensity] "&
        "method_ok: "},
  {"Distance",--Distance
  parse("hue = sqrt(sqr(x) + sqr(y)) * spot : " &
        "saturation = hit_sat : " &
        "intensity = hit_int")},
  {"Attractor",--Attractor
  parse("hue = sqrt(sqr(x - old_x) + sqr(y - old_y)) * dist + col : " &
        "saturation = hit_sat : " &
        "intensity = hit_int")},
  {"Texture",--Texture
  parse("hue = sqrt(sqr(x) + sqr(y)) * spot : " &
        "saturation = 1 - hits : " &
        "intensity = (1 - sqrt(sqr(x - old_x) + sqr(y - old_y)) * dist) * hit_int")&
        "fldz "&
        "fcomp qword ptr[intensity] "&
        "fstsw ax "&
        "sahf " &
        "jb method_ok "&
        "  fldz " &
        "  fstp qword ptr[intensity] "&
        "method_ok: "}}
  --IFS?
  
--Projection styles
projections = {
  --Linear
  {"Linear", ""},
  --Sphere
  {"Sphere", parse("pnt_x = (max_r - rot_x) * wrp_x : "&
        "pnt_y = (max_i - rot_y) * wrp_y : "&
        "rot_x = cos(pnt_x) * sin(pnt_y) * pt36 * size_x + centre_x : "&
        "rot_y = cos(pnt_y) * half * size_y + centre_y")},
  --Horizontal cylinder
  {"Horiz Cyl", parse("pnt_y = (max_i - rot_y) * wrp_y : "&
        "rot_y = cos(pnt_y) * half * size_y + centre_y")},
  --Vertical cylinder
  {"Vert Cyl", parse("pnt_x = (max_r - rot_x) * wrp_x : "&
        "rot_x = cos(pnt_x) * half * size_x + centre_x")},
  --Pillow
  {"Pillow", parse("pnt_x = (max_r - rot_x) * wrp_x : "&
        "pnt_y = (max_i - rot_y) * wrp_y : "&
        "rot_x = cos(pnt_x) * half * size_x + centre_x : " &
        "rot_y = cos(pnt_y) * half * size_y + centre_y")},
  --Torus
  {"Torus", parse("pnt_x = (max_r - rot_x) * wrp_x : "&
        "pnt_y = (max_i * pt8 - rot_y) * wrp_y * tp4 : "&
        "rot_x = sin(pnt_y) * pt18 * (1 + cos(pnt_x)) * size_x + centre_x : " &
        "rot_y = cos(pnt_y) * pt25 * (1 + cos(pnt_x)) * size_y + centre_y")},
  --Inversion
  {"Inversion", parse("wrp_x = rot_x - centre_x : "&
        "wrp_y = rot_y - centre_y : "&
        "pnt_x = arctan2(wrp_x, wrp_y) : "&
        "pnt_y = sqrt(sqr(wrp_x) + sqr(wrp_y)) : " &
        "rot_x = cos(pnt_x) * (size_x * half - pnt_y * pt8) + centre_x : " &
        "rot_y = sin(pnt_x) * (size_y * half - pnt_y * pt8) + centre_y")}}

-----------------------------------------------------------------------------
--Fractals

--IFS shared functions
--get_form randomly determines the IFS rule to use
constant get_form = 
  "push ebx \n " &
  "push ecx \n " &
  parse("(rnd() + 1) / two") & --leave rnd on fpu stack
  "mov ebx, a "& --start of parameter space
  "sub ebx, 8 "& --point to previous probability param
  "mov ecx, rules " & --rule counter
  "get_form_loop: " &
  "  add ebx, 56 " &             --move prob ptr forward one rule
  "  fcom qword ptr[ebx] \n" &   --compare rnd with prob
  "  fstsw ax \n" &
  "  sahf \n " &
  "  jb get_form_exit \n " &     --exit if smaller
  "  fsub qword ptr[ebx] \n " &  --else shrink rnd by this prob
  "loop get_form_loop " &        --do for number of rules
  "get_form_exit: " &
  "fstp st(0) \n" & --get rid of rnd
  "sub ebx, 48 " & --point to start of entry
--  "mov ebx, a "& --start of parameter space
  "mov dword ptr[rule], ebx " &  --put it in rule
  "pop ecx \n" &
  "pop ebx \n"
--lin_ifs is the generic linear ifs form
constant lin_ifs = 
  "push ebx " &
  --x value
  "mov ebx, dword ptr[rule] "& --value 1
  "fld qword ptr[ebx] " &      
  "fmul qword ptr[old_x] " &
  "add ebx, 8 " &              --value 2
  "fld qword ptr[ebx] " &
  "fmul qword ptr[old_y] " &
  "faddp st(1), st(0) " &      
  "add ebx, 24 \n" &           --value 5
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[x] " &
  --y value 
  "sub ebx, 16 " &             --value 3
  "fld qword ptr[ebx] " &      
  "fmul qword ptr[old_x] " &
  "add ebx, 8 " &              --value 4
  "fld qword ptr[ebx] " &
  "fmul qword ptr[old_y] " &
  "faddp st(1), st(0) " &      
  "add ebx, 16 \n" &           --value 6
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[y] " &
  "pop ebx "



--bifurication(1)
function Bifurication_init(sequence param)
  return {{1.09 / (param[MAXITER] + 1000)},  --coef
          {2.9, 0.4},               --seed
          param[CORNERS],           --edge
          rnd(.01, .1),             --spot
          rnd(.5, 5),               --dist
          0}                        --angle
end function
params &= {{"Bifurication",         --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{2.9,0},{4,1}},                  --coners
  {1, 1, 1, 1},                     --colour
  {3.45, 0.5},                      --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Bifurication_init"),  --init
  "",                               --shuffle
  parse("x = old_x + a : y = old_y * old_x * (1 - old_y)")
}}

--binary(2)
function Binary_init(sequence param)
  return {{.5,  .0, .0, .5, -2.563477, -0.000003, .33,
           .5,  .0, .0, .5, 2.436544, -0.000003, .33,
           .0, -.5, .5, .0, 4.873085,  7.563492, .34},  --coef
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          rand(4) * 90}             --angle
end function
params &= {{"Binary",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Binary_init"),        --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--chip(3)
function Chip_init(sequence param)
  return {{rnd(-50, 50), rnd(-50, 50), rnd(-50, 50)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0005, .002),         --spot
          rnd(.0005, .002),         --dist
          0}                        --angle
end function
params &= {{"Chip",                  --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{-560,-420},{560,420}},          --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Chip_init"),          --init
  "",                               --shuffle
  parse("x = old_y - sgn(old_x) * cos(sqr(ln(abs(b * old_x - c)))) * arctan(sqr(ln(abs(c * old_x - b)))) : " &
        "y = a - old_x")            --orbit
}}

--complexIFS (4)
function ComplexIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"ComplexIFS",            --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  1000000,                          --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("ComplexIFS_init"),    --init
  get_form,                         --shuffle
  parse("c5 = arctan2(old_x, old_y) / two : " &
        "d5 = sqrt(sqrt(abs(sqr(old_x) + sqr(old_y)))) : " &
        "a5 = cos(c5) * d5 : b5 = sin(c5) * d5") &
  "push ebx \n" &
  --x value
  "mov ebx, dword ptr[rule] \n"&    --value 1
  "fld qword ptr[ebx] \n" &      
  "fmul qword ptr[a5] \n" &
  "add ebx, 8 \n" &                 --value 2
  "fld qword ptr[ebx] \n" &
  "fmul qword ptr[b5] \n" &
  "faddp st(1), st(0) \n" &      
  "add ebx, 24 \n" &                --value 5
  "fadd qword ptr[ebx] \n" &
  "fstp qword ptr[x] \n" &
  --y value 
  "sub ebx, 16 \n" &                --value 3
  "fld qword ptr[ebx] \n" &      
  "fmul qword ptr[a5] \n" &
  "add ebx, 8 \n" &                 --value 4
  "fld qword ptr[ebx] \n" &
  "fmul qword ptr[b5] \n" &
  "faddp st(1), st(0) \n" &      
  "add ebx, 16 \n" &                --value 6
  "fadd qword ptr[ebx] \n" &
  "fstp qword ptr[y] \n" &
  "pop ebx \n"                      --orbit
}}

--coral(5)
function Coral_init(sequence param)
  return {{.307692, -.531469, -.461538, -.293706,  5.401953, 8.655175, .40,
           .307692, -.076923,  .153846, -.447552, -1.295248, 4.152990, .15,
           .000000,  .545455,  .692308, -.195804, -4.893637, 7.269794, .45},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          rand(360)}                --angle
end function
params &= {{"Coral",                 --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Coral_init"),        --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--crystal(6)
function Crystal_init(sequence param)
  return {{.696970, -.481061, -.393939, -.662879, 2.147003, 10.310288, .75,
           .090909, -.443182,  .515152, -.094697, 4.286558,  2.925762, .25},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          rand(360)}                --angle
end function
params &= {{"Crystal",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Crystal_init"),       --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--dragon(7)
function Dragon_init(sequence param)
  return {{.824074, .281482, -.212346,  .864198, -1.882290, -0.110607, .79,
           .088272, .520988, -.463889, -.377778,  0.785360,  8.095795, .21},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, 1),              --dist
          160 + rand(40)}           --angle
end function
params &= {{"Dragon",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Dragon_init"),        --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--fern(8)
function Fern_init(sequence param)
  return {{0,     0,     0,     0.16,  0,     0,     .01,
           0.85,  0.04,  -0.04, 0.85,  0,     1.6,   .85,
           0.2,   -0.26, 0.23,  0.22,  0,     1.6,   .07,
           -0.15, 0.28,  0.26,  0.24,  0,     0.44,  .07},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, 1),              --dist
          rand(360)}                --angle
end function
params &= {{"Fern",                 --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Fern_init"),          --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--floor(9)
function Floor_init(sequence param)
  return {{.0, -.5,  .5, .0, -1.732366, 3.366182, .33,
           .5,  .0,  .0, .5, -0.027891, 5.014877, .33,
           .0,  .5, -.5, .0,  1.620804, 3.310401, .34},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          135 + rand(90)}           --angle
end function
params &= {{"Floor",                --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Floor_init"),         --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}
  
--gingerbread(10)
function Gingerbread_init(sequence param)
  return {{rnd(.9, 1.1)},--coef
          {rnd(-3, 3), rnd(-3, 3)}, --seed
          param[CORNERS],           --edge
          rnd(.01, .1),             --spot
          rnd(.025, .075),          --dist
          rand(90)}                 --angle
end function
params &= {{"Gingerbread",           --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  500000,                           --iterations
  0.2,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.002, 0,                      --project, lyap, dim
  routine_id("Gingerbread_init"),   --init
  "",                               --shuffle
  parse("x = 1 - old_y + a * abs(old_x) : y = old_x") --orbit
}}

--gingerpower(11)
function GingerPower_init(sequence param)
  return {{rnd(-25, 25), pn() * rnd(2, 5), rnd(-50, 50), rnd(.7, .9)},--coef
          {rnd(-3, 3), rnd(-3, 3)}, --seed
          param[CORNERS],           --edge
          .00001,                   --spot
          .0005,                    --dist
          -rand(90)}                --angle
end function
params &= {{"GingerPower",          --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  500000,                           --iterations
  0.4,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("GingerPower_init"),   --init
  "",                               --shuffle
  parse("temp_x = old_x + e : y = a - old_x : " & --orbit      
        "x = old_y - power(abs(b * temp_x - c), d)")
}}

--henon(12) : a=1.4 b=.3 @10w/h
function Henon_init(sequence param)
  return {{rnd(1.38, 1.41), 0.3},   --coef
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.5, 1),               --dist
          rand(360)}                --angle
end function
params &= {{"Henon",                --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  50000,                            --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Henon_init"),         --init
  "",                               --shuffle
  parse("x = 1 + old_y - a * sqr(old_x) : y = b * old_x") --orbit
}}

--hopalong(13) 
function Hopalong_init(sequence param)
  sequence ret
  ret =  {{rnd(-50, 50), rnd(-50, 50), rnd(-50, 50), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0005, .002),         --spot
          rnd(.0005, .004),         --dist
          315 + (rand(100) > 50) * 180} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][2]) + abs(ret[1][3])) / 3
  return ret        
end function
params &= {{"Hopalong",              --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{-20, -16}, {20, 16}},           --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Hopalong_init"),      --init
  "",                               --shuffle
  parse("temp_x = old_x + d : y = a - old_x")&       
  "fld qword ptr[old_x] " &
  "ftst " &
  "fstsw ax " &
  "fstp st(0) " &
  "sahf " &
  "ja hopper_pos " &
  parse("x = old_y + sqrt(abs(b * temp_x - c))") &
  "jmp hopper_done " &
  "hopper_pos: " &
  parse("x = old_y - sqrt(abs(b * temp_x - c))") &
  "hopper_done: "
}}

--hoplin(14) 
function HopLinear_init(sequence param)
  sequence ret
  ret =  {{rnd(-33, 33), rnd(0, 0.4), rnd(-25, 25), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0005, .002),         --spot
          rnd(.0005, .004),         --dist
          315 + (rand(100) > 50) * 180} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][3])) / 2
  return ret        
end function
params &= {{"HopLinear",            --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{-20, -16}, {20, 16}},           --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopLinear_init"),     --init
  "",                               --shuffle
  parse("temp_x = old_x + d : y = a - old_x")&       
  "fld qword ptr[old_x] " &
  "ftst " &
  "fstsw ax " &
  "fstp st(0) " &
  "sahf " &
  "jae hopper_pos " &
  parse("x = old_y + (b * temp_x - c)") &
  "jmp hopper_done " &
  "hopper_pos: " &
  parse("x = old_y - (b * temp_x - c)") &
  "hopper_done: "
}}

--hoplog(15) 
function HopLog_init(sequence param)
  return  {{rnd(-25, 25), 
           pn() * power(10, rnd(6, 30)), 
           pn() * power(10, rnd(0, 9)), 
           rnd(-100, 100)},         --coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.001, .003),          --spot
          rnd(.0005, .004),         --dist
          315 + (rand(100) > 50) * 180} --angle
end function
params &= {{"HopLog",               --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{-200, -160}, {200, 160}},       --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopLog_init"),        --init
  "",                               --shuffle
  parse("temp_x = old_x + d : y = a - old_x")&       
  "fld qword ptr[old_x] " &
  "ftst " &
  "fstsw ax " &
  "fstp st(0) " &
  "sahf " &
  "ja hopper_pos " &
  parse("x = old_y - log(abs(b * temp_x - c))") &
  "jmp hopper_done " &
  "hopper_pos: " &
  parse("x = old_y + log(abs(b * temp_x - c))") &
  "hopper_done: "
}}

--hoppwr(16) 
function HopPower_init(sequence param)
  sequence ret
  ret =  {{rnd(-25, 25), pn() * rnd(2, 5), 
           rnd(-50, 50), rnd(.7, .9), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0001, .0002),        --spot
          rnd(.0001, .001),         --dist
          315 + (rand(100) > 50) * 180} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][2]) + abs(ret[1][3])) / 3
  return ret        
end function
params &= {{"HopPower",             --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{-200, -320}, {200, 320}},       --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopPower_init"),      --init
  "",                               --shuffle
  parse("temp_x = old_x + e : y = a - old_x")&       
  "fld qword ptr[old_x] " &
  "ftst " &
  "fstsw ax " &
  "fstp st(0) " &
  "sahf " &
  "ja hopper_pos " &
  parse("x = old_y - power(abs(b * temp_x - c), d)") &
  "jmp hopper_done " &
  "hopper_pos: " &
  parse("x = old_y + power(abs(b * temp_x - c), d)") &
  "hopper_done: "
}}

--hopsqrt(17) 
function HopRoot_init(sequence param)
  sequence ret
  ret =  {{pn() * rnd(50, 500), rnd(1, 9), rnd(-5, 5), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          .0001,                    --spot
          .001,                     --dist
          315 + (rand(100) > 50) * 45} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][3])) / 2
  return ret        
end function
params &= {{"HopRoot",              --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{-7, -5}, {9, 7}},               --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopRoot_init"),       --init
  "",                               --shuffle
  parse("temp_x = old_x + d : y = a - old_x")&       
  "fld qword ptr[old_x] " &
  "ftst " &
  "fstsw ax " &
  "fstp st(0) " &
  "sahf " &
  "jae hopper_pos " &
  parse("x = old_y - sqrt(abs(b * temp_x - c))") &
  "jmp hopper_done " &
  "hopper_pos: " &
  parse("x = old_y + sin(b * temp_x) - c") &
  "hopper_done: "
}}

--hopsin(18) 
function HopSine_init(sequence param)
  sequence ret
  ret =  {{rnd(-5, 5), rnd(.3, .4), rnd(-500, 500), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.00005, .0001),       --spot
          rnd(.00005, .0004),       --dist
          315 + (rand(100) > 50) * 180} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][3])) / 2
  return ret        
end function
params &= {{"HopSine",              --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{-40, -32}, {40, 32}},           --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopSine_init"),       --init
  "",                               --shuffle
  parse("temp_x = old_x + d : y = a - old_x")&       
  "fld qword ptr[old_x] " &
  "ftst " &
  "fstsw ax " &
  "fstp st(0) " &
  "sahf " &
  "jae hopper_pos " &
  parse("x = old_y - (b * temp_x - c)") &
  "jmp hopper_done " &
  "hopper_pos: " &
  parse("x = old_y + sin(b * temp_x) - c") &
  "hopper_done: "
}}

--hoptrig(19) 
function HopTrig_init(sequence param)
  sequence ret
  ret =  {{rnd(-1, 1), rnd(.5, .85), rnd(-1, 1), rnd(-100, 100)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0005, .002),         --spot
          rnd(.0005, .004),         --dist
          315 + (rand(100) > 50) * 45} --angle
  ret[EDGE] *= (abs(ret[1][1]) + abs(ret[1][3])) / 2
  return ret        
end function
params &= {{"HopTrig",              --name
  STANDARD,                         --flags
  1,                                --enable
  500000,                           --iterations
  {{-200, -160}, {200, 160}},       --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("HopTrig_init"),       --init
  "",                               --shuffle
  parse("temp_x = old_x + d : y = a - old_x")&       
  "fld qword ptr[old_x] " &
  "ftst " &
  "fstsw ax " &
  "fstp st(0) " &
  "sahf " &
  "jae hopper_pos " &
  parse("x = old_y - sin(b * temp_x) - c") &
  "jmp hopper_done " &
  "hopper_pos: " &
  parse("x = old_y + sin(b * temp_x) - c") &
  "hopper_done: "
}}

--icons3d(20) 
function Icons3D_init(sequence param)
  return {{-2.34, 2.04, 0.2, rnd(.09, .1) * (rand(100)>50), 
           rnd(-0.6, 0.6), rand(3)},--coef
          {0.01, 0.003, 0},         --seed
          param[CORNERS],           --edge
          rnd(.5, 1),               --spot
          rnd(.5, 1),               --dist
          rand(360)}                --angle
end function
params &= {{"Icons3D",               --name
  STANDARD + THREED + PIXLIST,      --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 0},                     --colour
  {0, 0},                           --origin
  1, 0.1, 0,                        --project, lyap, dim
  routine_id("Icons3D_init"),       --init
  "",                               --shuffle
  parse("a1 = sqr(old_x) + sqr(old_y) : "&
        "b1 = old_x : c1 = old_y : floor = f")&
  "mov eax, dword ptr[floor] " &
  "icons3d_loop: " &
  parse("d1 = b1 * old_x - c1 * old_y : "&
        "e1 = c1 * old_x + b1 * old_y : "&
        "b1 = d1 : c1 = e1")&
  "  dec eax " &
  "jnz icons3d_loop " &
  parse("d1 = old_x * b1 - old_y * c1 : "&
        "e1 = a + b * a1 + c * d1 : "&
        "x = e1 * old_x + d * b1 - e * old_y : "&
        "y = e1 * old_y - d * c1 + e * old_x : "&
        "z = a1")
}}

--ikeda(21) 
function Ikeda_init(sequence param)
  return {{rnd(-50, 50), 1, pn() * rnd(3.6, 50), pn() * rnd(3, 8)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.002, .02),           --spot
          rnd(.01, .05),            --dist
          (rand(100) > 50) * 180}   --angle
end function
params &= {{"Ikeda",                --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.01, 0,                       --project, lyap, dim
  routine_id("Ikeda_init"),         --init
  "",                               --shuffle
  parse("e = a - c / (1 + sqr(old_x) + sqr(old_y)) : " &--orbit
        "x = d + b * (old_x * cos(e) - old_y * sin(e)) :" &
        "y = b * (old_x * sin(e) + old_y * cos(e))")
}}

--julia(22) 
function Julia_init(sequence param)
  return {find_julia(rand(10) + 10, routine_id("get_mandel")),--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.05, .3),             --spot
          rnd(.05, .5),             --dist
          0}                        --angle
end function
params &= {{"Julia",                --name
  STANDARD + NOMIN,                 --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 0},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Julia_init"),         --init
  parse("rnd()") &                  --shuffle
  "ftst " &
  "fstsw ax \n" &
  "fstp st(0) " &
  "and ah, 1 \n" &
  "mov [p1], ah \n",                
  parse("temp_x = old_x - a : temp_y = old_y - b : " &
        "c = arctan2(temp_x, temp_y) / two : " &
        "d = sqrt(sqrt(abs(sqr(temp_x) + sqr(temp_y)))) : " &
        "x = cos(c) * d : y = sin(c) * d") &
  "cmp byte ptr[p1], 0 \n " &
  "je no_jul_swap \n" &
     parse("x = -x : y = -y") &
  "no_jul_swap: \n"
}}

--koch(23)
function Koch_init(sequence param)
  return {{.307692, -.000000,  .000000,  .294118,  4.119164, 1.604278, .15,
           .192308, -.205882,  .653846,  .088235, -0.688840, 5.978916, .25,
           .192308,  .205882, -.653846,  .088235,  0.668580, 5.962514, .25,
           .307692, -.000000,  .000000,  .294118, -4.136530, 1.604278, .15,
           .384615, -.000000,  .000000, -.294118, -0.007718, 2.941176, .20},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          (rand(100) > 50) * 180}   --angle
end function
params &= {{"Koch",                 --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Koch_init"),          --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--linearifs(24)
function LinearIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.02, .2),             --spot
          rnd(.05, .2),             --dist
          (rand(100) > 50) * 180}   --angle
end function
params &= {{"LinearIFS",            --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("LinearIFS_init"),     --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--lintrigIFS(25)
function LinTrigIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.02, .2),             --spot
          rnd(.05, .2),             --dist
          (rand(100) > 50) * 180}   --angle
end function
params &= {{"LinTrigIFS",           --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("LinTrigIFS_init"),    --init
  get_form,                         --shuffle
  "push ebx " &                     --orbit
  --x value
  "mov ebx, dword ptr[rule] "&      --value 1
  "fld qword ptr[ebx] " &      
  "fmul qword ptr[old_x] " &
  "add ebx, 8 " &                   --value 2
  "fld qword ptr[ebx] " &
  "fld qword ptr[old_y] " &
  "fsin " &
  "fmulp st(1), st(0) " &
  "faddp st(1), st(0) " &      
  "add ebx, 24 \n" &                --value 5
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[x] " &
  --y value 
  "sub ebx, 16 " &                  --value 3
  "fld qword ptr[ebx] " &      
  "fmul qword ptr[old_x] " &
  "add ebx, 8 " &                   --value 4
  "fld qword ptr[ebx] " &
  "fld qword ptr[old_y] " &
  "fcos " &
  "fmulp st(1), st(0) " &
  "faddp st(1), st(0) " &      
  "add ebx, 16 \n" &                --value 6
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[y] " &
  "pop ebx "
}}

--lissajous(26) 
function Lissajous_init(sequence param)
  return {{rnd(-.000000000000000001, .000000000000000001), 
           rnd(-.000000000000000001, .000000000000000001), 
           rnd(-.000000000000000001, .000000000000000001), 
           rnd(-.000000000000000001, .000000000000000001), 
           rnd(1.402, 1.9999999999)},--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.05, .3),             --spot
          rnd(.05, .5),             --dist
          (rand(100) > 50) * 90}    --angle
end function
params &= {{"Lissajous",            --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 0},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Lissajous_init"),     --init
  parse("rnd()") &                  --shuffle
  "ftst " &
  "fstsw ax \n" &
  "fstp st(0) " &
  "and ah, 1 \n" &
  "mov byte ptr[p1], ah \n" &
  parse("rnd()") &
  "ftst " &
  "fstsw ax \n" &
  "fstp st(0) " &
  "and ah, 1 \n" &
  "mov byte ptr[p2], ah \n" &
  parse("rnd()") &
  "ftst " &
  "fstsw ax \n" &
  "fstp st(0) " &
  "and ah, 1 \n" &
  "mov byte ptr[p3], ah \n",
  "cmp byte ptr[p1], 0 \n" &        --orbit
  "je lissa_usecd \n" &
     parse("temp_x = old_x - a : temp_y = old_y - b") &
    "jmp lissa_useab \n" &
  "lissa_usecd: \n" &   
     parse("temp_x = old_x - c : temp_y = old_y - d") &
  "lissa_useab: \n" &   
  "cmp byte ptr[p2], 0 \n" &
  "je no_lissa_neg \n" &
     parse("temp_x = -temp_x : temp_y = -temp_y") &
  "no_lissa_neg: \n" &
  parse("x = temp_y : y = sqr(temp_x) - e") &
  "cmp byte ptr[p3], 0 \n" &
  "je no_lissa_swap \n" &
     parse("y = -y") &
  "no_lissa_swap: \n"
}}

--lorentz3d(27) 
function Lorentz3D_init(sequence param)
  return {{rnd(3, 8), 15, rnd(.6, 2.5), 0.02},--coef
          {1, 1, 1},                --seed
          param[CORNERS],           --edge
          rnd(.01, .03),            --spot
          rnd(.1, 1),               --dist
          0}                        --angle
end function
params &= {{"Lorentz3D",            --name
  STANDARD + THREED + PIXLIST,      --flags
  1,                                --enable
  500000,                           --iterations
  {{-15, 30}, {15, 0}},             --coners
  {1, 1, 1, 0},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Lorentz3D_init"),     --init
  "",                               --shuffle
  parse("x = old_x + (-a * old_x * d) + (a * old_z * d) : "&
        "y = old_y + (-c * old_y * d) + (old_x * old_z * d) : "&
        "z = old_z + ( b * old_x * d) - (old_z * d) - (old_y * old_x * d)")
}}

--martin(28)
function Martin_init(sequence param)
  return {{rnd(3.07, 3.21)},        --coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.005, .01),           --spot
          rnd(.005, .025),          --dist
          (rand(100) > 50) * 45}    --angle
end function
params &= {{"Martin",               --name
  STANDARD,                         --flags
  1,                                --enable
  100000,                           --iterations
  {{-64, -64}, {64, 64}},           --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.01, 0,                       --project, lyap, dim
  routine_id("Martin_init"),        --init
  "",                               --shuffle
  parse("x = old_y - sin(old_x) : y = a - old_x") --orbit
}}

--nickfrac(29)
function Nickfrac_init(sequence param)
  return {{0.00, -0.11,  0.22,  0.00, -6.25, 4.84, .06,  --\
           0.00, -0.11,  0.22,  0.00, -5.00, 5.23, .03,  -- | 'N'
           0.08,  0.11, -0.25,  0.00, -6.70, 4.99, .04,  --/
           0.00,  0.08, -0.25,  0.02, -5.20, 4.95, .07,  ---- 'I'
           0.00,  0.11, -0.19,  0.00, -4.52, 4.99, .06,  --\
           0.09, -0.01,  0.00,  0.10, -3.48, 2.96, .03,  -- | 'C'
           -0.09, 0.00,  0.00, -0.11, -3.55, 7.10, .03,  --/
           0.00,  0.08, -0.25,  0.02, -2.60, 4.95, .07,  --\
           0.06,  0.11, -0.13,  0.00, -2.00, 4.15, .04,  -- | 'K'
           -0.06, 0.11, -0.13,  0.00, -2.00, 5.80, .04,  --/
           0.00, -0.11,  0.22,  0.00,  0.35, 4.84, .06,  --\
           0.11,  0.02,  0.00,  0.11,  0.30, 5.99, .03,  -- | 'F'
           0.06,  0.02,  0.00,  0.10,  0.35, 4.51, .02,  --/
           0.00, -0.11,  0.22,  0.00,  2.34, 4.84, .06,  --\
           0.08,  0.00,  0.00,  0.11,  2.10, 5.99, .02,  -- |
           0.00,  0.11, -0.08,  0.00,  2.30, 6.15, .02,  -- | 'R'
           -0.09, 0.00, -0.01, -0.13,  2.45, 5.94, .02,  -- |
           0.06,  0.11, -0.13,  0.00,  2.19, 4.15, .04,  --/
           0.03, -0.11,  0.23,  0.11,  4.56, 4.43, .07,  --\
           0.03,  0.11, -0.25,  0.00,  4.27, 4.99, .07,  -- | 'A'
           0.06,  0.00,  0.00,  0.11,  4.40, 4.46, .02,  --/
           0.00,  0.11, -0.19,  0.00,  5.32, 4.99, .06,  --\
           0.09, -0.01,  0.00,  0.10,  6.35, 2.96, .03,  -- | 'C'
           -0.09, 0.00,  0.00, -0.11,  6.35, 7.10, .03}, --/
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          rand(360)}                --angle
end function
params &= {{"Nickfrac",             --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Nickfrac_init"),      --init
  get_form,                         --shuffle
  lin_ifs & parse("x = -x")         --orbit
}}

--pickover3d(30) 
function Pickover3D_init(sequence param)
  return {{rnd(1.8, 2.5), rnd(-1, 1), rnd(-1, 1), rnd(1.7, 4)},--coef
          {1, 1, 1},                --seed
          param[CORNERS],           --edge
          rnd(.05, .4),             --spot
          rnd(.05, .5),             --dist
          rand(360)}                --angle
end function
params &= {{"Pickover3D",           --name
  STANDARD + THREED + PIXLIST,      --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.3, 0,                        --project, lyap, dim
  routine_id("Pickover3D_init"),    --init
  "",                               --shuffle
  parse("x = sin(a * old_y) - old_z * cos(b * old_x) : " &
        "y = old_z * sin(c * old_x) - cos(d * old_y) : " &
        "z = sin(old_x)")           --orbit
}}

--polyabs(31)
function PolyAbs_init(sequence param)
  return {rand_strange(10),         --coef
          {0.05, 0.05, 0.05},       --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.1, 1),               --dist
          0}                        --angle
end function
params &= {{"PolyAbs",              --name
  STRANGE + THREED + PIXLIST,       --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("PolyAbs_init"),       --init
  "",                               --shuffle
  parse(                            --orbit
  "x = a + b * old_x + c * old_y + d * abs(old_x) + e * abs(old_y) : "&
  "y = f + g * old_x + a1 * old_y + b1 * abs(old_x) + c1 * abs(old_y) : "&
  "z = sqr(old_x) + sqr(old_y)")
}}

--polyboolean(32)
function PolyBoolean_init(sequence param)
  return {rand_strange(14),         --coef
          {0.05, 0.05, 0.05},       --seed
          param[CORNERS],           --edge
          rnd(.01, .03),            --spot
          rnd(.01, .1),             --dist
          0}                        --angle
end function
params &= {{"PolyBoolean",          --name
  STRANGE + THREED + PIXLIST,       --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("PolyBoolean_init"),   --init
  "",                               --shuffle
  parse("floor = d * old_x + half") &
  "mov eax, dword ptr[floor] " &
  parse("floor = e * old_y + half") &
  "and eax, dword ptr[floor] " &
  "mov dword ptr[floor], eax " &
  parse("a3 = floor") &
  parse("floor = f * old_x + half") &
  "mov eax, dword ptr[floor] " &
  parse("floor = g * old_y + half") &
  "and eax, dword ptr[floor] " &
  "mov dword ptr[floor], eax " &
  parse("x = a + b * old_x + c * old_y + a3 + floor : "&
        "floor = d1 * old_x + half") &
  "mov eax, dword ptr[floor] " &
  parse("floor = e1 * old_y + half") &
  "and eax, dword ptr[floor] " &
  "mov dword ptr[floor], eax " &
  parse("a3 = floor") &
  parse("floor = f1 * old_x + half") &
  "mov eax, dword ptr[floor] " &
  parse("floor = g1 * old_y + half") &
  "and eax, dword ptr[floor] " &
  "mov dword ptr[floor], eax " &
  parse("y = a1 + b1 * old_x + c1 * old_y + a3 + floor : "&
        "z = sqr(old_x) + sqr(old_y)")
}}

--polypower (33)
function PolyPower_init(sequence param)
  return {rand_strange(14),         --coef
          {0.05, 0.05, 0.05},       --seed
          param[CORNERS],           --edge
          rnd(.005, .05),           --spot
          rnd(.01, .1),             --dist
          0}                        --angle
end function
params &= {{"PolyPower",            --name
  STRANGE + THREED + PIXLIST,       --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("PolyPower_init"),     --init
  "",                               --shuffle
  parse(                            --orbit
  "x = a + b * old_x + c * old_y + d * power(abs(old_x),e) + f * power(abs(old_y),g) : "&
  "y = a1 + b1 * old_x + c1 * old_y + d1 * power(abs(old_x),e1) + f1 * power(abs(old_y),g1) : "&
  "z = sqr(old_x) + sqr(old_y)")
}}

--polytrig(34)
function PolyTrig_init(sequence param)
  return {rand_strange(6),          --coef
          {0.05, 0.05, 0.05},       --seed
          param[CORNERS],           --edge
          rnd(.001, .005),          --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"PolyTrig",             --name
  STRANGE + THREED + PIXLIST,       --flags
  1,                                --enable
  500000,                           --iterations
  0.3,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("PolyTrig_init"),      --init
  "",                               --shuffle
  parse(                            --orbit
  "a1 = pi / (thirteen + ten * f) : "&
  "b1 = sin(a1) : "&
  "c1 = cos(a1) : "&
  "d1 = old_x + b * sin(c * old_y + d) : "&
  "x = ten * a + d1 * c1 + old_y * b1 : "&
  "y = ten * e - d1 * b1 + old_y * c1 : "&
  "z = sqr(old_x) + sqr(old_y)")
}}

--popcorn(35) 
function Popcorn_init(sequence param)
  return {{rnd(.03, .07)},          --coef
          {0.0001, 0.0001},         --seed
          param[CORNERS],           --edge
          rnd(.01, .05),            --spot
          rnd(2, 8),                --dist
          (rand(100) > 50) * 45}    --angle
end function
params &= {{"Popcorn",              --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  100000,                           --iterations
  {{-25, -18}, {25, 18}},           --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.1, 0.1,                      --project, lyap, dim
  routine_id("Popcorn_init"),       --init
  "",                               --shuffle
  parse("x = old_x - a * sin(old_y + tan(3 * old_y)) : "&
        "y = old_y + a * sin(old_x + tan(3 * old_x))")
}}

--powerIFS(36)
function PowerIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"PowerIFS",             --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("PowerIFS_init"),      --init
  get_form,                         --shuffle
  parse("a5 = sqr(old_x) : "&       --orbit
        "b5 = sqr(old_y)")&
  "push ebx " &
  --x value
  "mov ebx, dword ptr[rule] "&      --value 1
  "fld qword ptr[ebx] " &      
  "fmul qword ptr[a5] " &
  "add ebx, 8 " &                   --value 2
  "fld qword ptr[ebx] " &
  "fmul qword ptr[b5] " &
  "faddp st(1), st(0) " &      
  "add ebx, 24 \n" &                --value 5
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[x] " &
  --y value 
  "sub ebx, 16 " &                  --value 3
  "fld qword ptr[ebx] " &      
  "fmul qword ptr[a5] " &
  "add ebx, 8 " &                   --value 4
  "fld qword ptr[ebx] " &
  "fmul qword ptr[b5] " &
  "faddp st(1), st(0) " &      
  "add ebx, 16 \n" &                --value 6
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[y] " &
  "pop ebx "
}}

--quad(37)
function Quadrature_init(sequence param)
  return {{rnd(-50, 50), rnd(-50, 50), rnd(-50, 50)}, --coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.005, .01),           --spot
          rnd(.005, .01),           --dist
          (rand(100) > 50) * 45}    --angle
end function
params &= {{"Quadrature",           --name
  STANDARD,                         --flags
  1,                                --enable
  200000,                           --iterations
  {{-82, -55}, {112, 90}},          --coners
  {1, 1, 1, 1},                     --colour
  {15, 17.5},                       --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Quadrature_init"),    --init
  "",                               --shuffle
  parse("x = old_y - sgn(old_x) * sin(ln(abs(b * old_x - c))) * arctan(sqr(ln(abs(c * old_x - b)))), y = a - old_x")
}}

--rootIFS(38)
function RootIFS_init(sequence param)
  return {rand_ifs(4),             --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"RootIFS",              --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("RootIFS_init"),       --init
  get_form,                         --shuffle
  parse("c5 = sgn(old_x) : a5 = sqrt(abs(old_x)) * c5 : "&
        "d5 = sgn(old_y) : b5 = sqrt(abs(old_y)) * d5")&
  "push ebx " &
  --x value
  "mov ebx, dword ptr[rule] "& --value 1
  "fld qword ptr[ebx] " &      
  "fmul qword ptr[a5] " &
  "add ebx, 8 " &              --value 2
  "fld qword ptr[ebx] " &
  "fmul qword ptr[b5] " &
  "faddp st(1), st(0) " &      
  "add ebx, 24 \n" &           --value 5
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[x] " &
  --y value 
  "sub ebx, 16 " &             --value 3
  "fld qword ptr[ebx] " &      
  "fmul qword ptr[a5] " &
  "add ebx, 8 " &              --value 4
  "fld qword ptr[ebx] " &
  "fmul qword ptr[b5] " &
  "faddp st(1), st(0) " &      
  "add ebx, 16 \n" &           --value 6
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[y] " &
  "pop ebx "
}}

--rorschach(39) 
function Rorschach_init(sequence param)
  return {{rnd(-50, 50), rnd(-50, 50)}, --coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.001, .005),          --spot
          rnd(.0005, .005),         --dist
          45}                       --angle
end function
params &= {{"Rorschach",            --name
  STANDARD + PIXLIST,               --flags
  1,                                --enable
  200000,                           --iterations
  {{-300, -240}, {300, 240}},       --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.2, 0.3,                      --project, lyap, dim
  routine_id("Rorschach_init"),     --init
  "",                               --shuffle
  parse("x = old_y - sin(a * old_x) : y = old_x - cos(b * old_y)")
}}

--sierpinski(40)
function Sierpinski_init(sequence param)
  return {{0.5,   0,     0,     0.5,   1,     1,     .33,
           0.5,   0,     0,     0.5,   1,     50,    .33,
           0.5,   0,     0,     0.5,   50,    50,    .34},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.005, .007),          --spot
          rnd(.005, .01),           --dist
          (rand(4) - 1) * 90}       --angle
end function
params &= {{"Sierpinski",           --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {50, 50},                         --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Sierpinski_init"),    --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--spiral(41)
function Spiral_init(sequence param)
  return {{ .787879, -.424242, .242424, .859848,  1.758647, 1.408065, .90,
           -.121212,  .257576, .151515, .053030, -6.721654, 1.377236, .05,
            .181818, -.136364, .090909, .181818,  6.086107, 1.568035, .05},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, 1),              --dist
          rand(360)}                --angle
end function
params &= {{"Spiral",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Spiral_init"),        --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--strange2d(42)
function Strange2D_init(sequence param)
  return {rand_strange(12),         --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.1, .3),              --spot
          rnd(.1, .3),              --dist
          0}                        --angle
end function
params &= {{"Strange2D",            --name
  STRANGE + PIXLIST,                --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("Strange2D_init"),     --init
  "",                               --shuffle
  parse(                            --orbit
  "a2 = old_x * old_x : b2 = old_x * old_y : c2 = old_y * old_y : "&    
  "x = a + b * old_x + c * a2 + d * b2 + e * old_y + f * c2 : " &
  "y = g + b * old_x + a1 * a2 + b1 * b2 + c1 * old_y + d1 * c2")
}}

--strange3d(43)
function Strange3D_init(sequence param)
  return {rand_strange(60),         --coef
          {0.05, 0.05, 0.05},       --seed
          param[CORNERS],           --edge
          rnd(.5, .7),              --spot
          rnd(.2, .5),              --dist
          0}                        --angle
end function
params &= {{"Strange3D",            --name
  STRANGE + THREED + PIXLIST,       --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0.005, 0,                      --project, lyap, dim
  routine_id("Strange3D_init"),     --init
  "",                               --shuffle
  parse(                            --orbit
  "a9 = old_x * old_x : b9 = a9 * old_x : c9 = a9 * old_y : "&
  "d9 = a9 * old_z : e9 = old_x * old_y : f9 = e9 * old_y : "&
  "g9 = e9 * old_z : a10 = old_x * old_z : b10 = a10 * old_z : "&
  "c10 = old_y * old_y : d10 = c10 * old_y : e10 = c10 * old_z : "&
  "f10 = old_y * old_z : g10 = f10 * old_z : a11 = old_z * old_z : "&
  "b11 = a11 * old_z : "&
  "x = a + b * old_x + c * a9 + d * b9 + e * c9 + f * d9 + "&
  "    g * e9 + a1 * f9 + b1 * g9 + c1 * a10 + d1 * b10 + "&
  "    e1 * old_y + f1 * c10 + g1 * d10 + a2 * e10 + "&
  "    b2 * f10 + c2 * g10 + d2 * old_z + e2 * a11 + f2 * b11 : "&
  "y = g2 + a3 * old_x + b3 * a9 + c3 * b9 + d3 * c9 + e3 * d9 + "&
  "    f3 * e9 + g3 * f9 + a4 * g9 + b4 * a10 + c4 * b10 + "&
  "    d4 * old_y + e4 * c10 + f4 * d10 + g4 * e10 + "&
  "    a5 * f10 + b5 * g10 + c5 * old_z + d5 * a11 + e5 * b11 : "&
  "z = f5 + g5 * old_x + a6 * a9 + b6 * b9 + c6 * c9 + d6 * d9 + "&
  "    e6 * e9 + f6 * f9 + g6 * g9 + a7 * a10 + b7 * b10 + "&
  "    c7 * old_y + d7 * c10 + e7 * d10 + f7 * e10 + "&
  "    g7 * f10 + a8 * g10 + b8 * old_z + c8 * a11 + d8 * b11")
}}

--swirl(44)
function Swirl_init(sequence param)
  return {{ .745455, -.459091,  .406061,  .887121, 1.460279, 0.691072, .91,
           -.424242, -.065152, -.175758, -.218182, 3.809567, 6.741476, .09},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.05, .2),             --dist
          rand(360)}                --angle
end function
params &= {{"Swirl",                --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Swirl_init"),         --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--tanIFS(45)
function TanIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"TanIFS",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("TanIFS_init"),        --init
  get_form,                         --shuffle
  parse("c5 = sgn(old_x) : a5 = arctan(old_x) * c5 : "&
        "d5 = sgn(old_y) : b5 = arctan(old_y) * d5")&
  "push ebx " &
  --x value
  "mov ebx, dword ptr[rule] "& --value 1
  "fld qword ptr[ebx] " &      
  "fmul qword ptr[a5] " &
  "add ebx, 8 " &              --value 2
  "fld qword ptr[ebx] " &
  "fmul qword ptr[b5] " &
  "faddp st(1), st(0) " &      
  "add ebx, 24 \n" &           --value 5
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[x] " &
  --y value 
  "sub ebx, 16 " &             --value 3
  "fld qword ptr[ebx] " &      
  "fmul qword ptr[a5] " &
  "add ebx, 8 " &              --value 4
  "fld qword ptr[ebx] " &
  "fmul qword ptr[b5] " &
  "faddp st(1), st(0) " &      
  "add ebx, 16 \n" &           --value 6
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[y] " &
  "pop ebx "
}}

--threeply(46)
function Threeply_init(sequence param)
  return {{rnd(-50, 50), rnd(-50, 50), rnd(-50, 50)}, --coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.0001, .001),         --spot
          rnd(.0005, .001),         --dist
          315 + (rand(100) > 50) * 180} --angle
end function
params &= {{"Threeply",             --name
  STANDARD,                         --flags
  1,                                --enable
  200000,                           --iterations
  {{-1000, -700}, {1000, 700}},     --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Threeply_init"),      --init
  "",                               --shuffle
  parse("x = old_y - sgn(old_x) * (abs(sin(old_x) * cos(b) + c - old_x * sin(a + b + c))) : y = a - old_x")
}}

--tree(47)
function Tree_init(sequence param)
  return {{0,     0,     0,     0.5,   0,     0,     .05,
           0.42,  -0.42, 0.42,  0.42,  0,     0.2,   .40,
           0.42,  0.42,  -0.42, 0.42,  0,     0.2,   .40,
           0.1,   0,     0,     0.1,   0,     0.2,   .15},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.5, 2),               --dist
          175 + rand(10)}           --angle
end function
params &= {{"Tree",                 --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, .25},                         --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Tree_init"),          --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

--trigIFS(48)
function TrigIFS_init(sequence param)
  return {rand_ifs(4),              --coef
          {0.05, 0.05},             --seed
          param[CORNERS],           --edge
          rnd(.05, .5),             --spot
          rnd(.05, .2),             --dist
          0}                        --angle
end function
params &= {{"TrigIFS",              --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 0},                           --origin
  1, -0.2, 0.5,                     --project, lyap, dim
  routine_id("TrigIFS_init"),       --init
  get_form,                         --shuffle
  "push ebx " &
  --x value
  "mov ebx, dword ptr[rule] "& --value 1
  "fld qword ptr[ebx] " &      
  "fld qword ptr[old_x] " &
  "fcos " &
  "fmulp st(1), st(0) " &
  "add ebx, 8 " &              --value 2
  "fld qword ptr[ebx] " &
  "fld qword ptr[old_y] " &
  "fsin " &
  "fmulp st(1), st(0) " &
  "faddp st(1), st(0) " &      
  "add ebx, 24 \n" &           --value 5
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[x] " &
  --y value 
  "sub ebx, 16 " &             --value 3
  "fld qword ptr[ebx] " &      
  "fld qword ptr[old_x] " &
  "fsin " &
  "fmulp st(1), st(0) " &
  "add ebx, 8 " &              --value 4
  "fld qword ptr[ebx] " &
  "fld qword ptr[old_y] " &
  "fcos " &
  "fmulp st(1), st(0) " &
  "faddp st(1), st(0) " &      
  "add ebx, 16 \n" &           --value 6
  "fadd qword ptr[ebx] " &
  "fstp qword ptr[y] " &
  "pop ebx "
}}

--xjulia(49) 
function XJulia_init(sequence param)
  return {find_julia(rand(10) + 10, routine_id("get_xmandel")),--coef
          {0, 0},                   --seed
          param[CORNERS],           --edge
          rnd(.05, .3),             --spot
          rnd(.05, .5),             --dist
          0}                        --angle
end function
params &= {{"XJulia",               --name
  STANDARD + NOMIN,                 --flags
  1,                                --enable
  500000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 0},                     --colour
  {0, 0},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("XJulia_init"),        --init
  parse("rnd()") &                  --shuffle
  "ftst " &
  "fstsw ax \n" &
  "fstp st(0) " &
  "and ah, 1 \n" &
  "mov [p1], ah \n",                
  parse("temp_x = old_y - a : temp_y = old_x - b : " &
        "c = arctan2(temp_x, temp_y) / two : " &
        "d = sqrt(sqrt(abs(sqr(temp_x) + sqr(temp_y)))) : " &
        "x = cos(c) * d : y = sin(c) * d") &
  "cmp byte ptr[p1], 0 \n " &
  "je no_xjul_swap \n" &
     parse("x = -x : y = -y") &
  "no_xjul_swap: \n"
}}

--zigzag(50)
function Zigzag_init(sequence param)
  return {{-.632407, -.614815, -.545370, .659259, 3.840822, 1.282321, .89,
           -.036111,  .444444,  .210185, .037037, 2.071081, 8.330552, .11},
          {rnd(-1, 1), 0},          --seed
          param[CORNERS],           --edge
          rnd(.05, .1),             --spot
          rnd(.01, .1),             --dist
          rand(360)}                --angle
end function
params &= {{"Zigzag",               --name
  IFS + PIXLIST,                    --flags
  1,                                --enable
  100000,                           --iterations
  0.1,                              --coners
  {1, 1, 1, 1},                     --colour
  {0, 5},                           --origin
  1, 0, 0,                          --project, lyap, dim
  routine_id("Zigzag_init"),        --init
  get_form,                         --shuffle
  lin_ifs                           --orbit
}}

num_frac = length(params)
--end of fractals.e

