include asm.e
--------------------------------------------------------------------------
sequence convert_code
  convert_code = 
  "pushad \n" &
  "mov esi, data \n" &                  --source index = data
  "finit \n" &                          --initialize FPU
  "fldcw [esi + 38] \n" &               --load FPU control word
  "mov al, [esi + 48] \n" &             --al = size of source number
  "cmp al, 4 \n" &                      --al == 4?
  "  jne next_qw \n" &                  --no, try again
  "  fld dword ptr[esi] \n" &           --yes, load a double word
  "  jmp endload \n" &                  --goto end
  "next_qw: \n" &                       --try for a quad word
  "cmp al, 8 \n" &                      --al == 8?
  "  jne next_tb \n" &                  --no, try again
  "  fld qword ptr[esi] \n" &           --yes, load a quad word
  "  jmp endload \n" &                  --goto end
  "next_tb: \n" &                       --must be a ten byte
  "fld tbyte ptr[esi] \n" &             --load a ten byte
  "endload: \n" &                       --end
  --chek number range and representation
  "fxam \n" &
  "fstsw ax \n" &
  "and ax, 18176 \n" &                  --mask out condition code flags
  "cmp ah, 4 \n" &                      -- +normalized
  "  jz in_range \n" &
  "cmp ah, 6 \n" &                      -- -normalized
  "  jz in_range \n" &
  "mov byte ptr[esi + 48], 0 \n" &      --flag a range error
  "mov [esi], ax \n" &                  --keep condition
  "jmp end \n" &                        --and quit
  "in_range: \n" &
  --extract exponent and mantissa
  "fxtract \n" &                        --do the extraction
  "fxch st(1) \n" &                     --exchange - exp(0) and mant(1)
  "fldl2t \n" &                         --load constant log2(10)
  "fdivrp st(1), st(0) \n" &            --exp /= l2t ; exp -> base 10
  "fld st(0) \n" &                      --copy exp
  "fbstp tbyte ptr[esi + 20] \n" &      --store integer component
  "fbld tbyte ptr[esi + 20] \n" &       --and reload it
  "fsubrp st(1), st(0) \n" &            --get fractional part
  "fldl2t \n" &                         --reload l2t
  "fmulp st(1), st(0) \n" &             --exp.frac *= l2t; exp.frac -> base 2
  --calculate 2^exp.frac
  "fld st(0) \n" &                      --copy exp.frac
  "frndint \n" &                        --get integer portion
  "fxch st(1) \n" &                     --exchange - exp(0), int(1)
  "fsub st(0), st(1) \n" &              --get fractional portion
  "f2xm1 \n" &                          --2^frac-1, -1 < frac < 1
  "fld1 \n" &                           --get 1
  "faddp st(1), st(0) \n" &             --and add, -> 2^frac
  "fxch st(1) \n" &                     --exchange - int(0), 2^frac(1)
  "fld1 \n" &                           --get 1
  "fscale \n" &                         --and scale - 2^int
  "fstp st(1) \n" &                     --get rid of that 1
  "fmulp st(1), st(0) \n" &             --result = 2^int * 2^frac
  --adjust mantissa to base 10
  "fmulp st(1), st(0) \n" &             --mantissa *= 2^exp.frac
  --renormalize
  "fcom qword ptr[esi + 40] \n" &       --compare mantissa to 10
  "fstsw ax \n" &                       --set ax with condition flags
  "sahf \n" &                           --load into integer unit
  "jb num_ok \n" &                      --mantissa < 10? goto ok
  "  fbld tbyte ptr[esi + 20] \n" &     --retrieve exponent
  "  fld1 \n" &                         --get 1
  "  faddp st(1), st(0) \n" &           --add 1 to exponent
  "  fbstp tbyte ptr[esi + 20] \n" &    --and replace it
  "  fdiv qword ptr[esi + 40] \n" &     --mantissa /= 10
  "num_ok: \n" &
  --overflow 80 bit vars for more precision
  "mov al, [esi + 48] \n" &             --al = size of source number
  "cmp al, 10 \n" &                     --number is tbyte?
  "  jne no_overflow \n" &              --no, don't do the overflow thing
  "  fmul qword ptr[esi + 59] \n" &     --yes, scale it up by over
  "  fld st(0) \n" &                    --copy scaled mantissa
  "  frndint \n" &                      --get integer part
  "  fsubr st(1), st(0) \n" &           --get fractional part
  "  fbstp tbyte ptr[esi + 49] \n" &    --store the integer in ov80
  "no_overflow \n" &
  "fmul qword ptr[esi + 30] \n" &       --adjust the mantissa into integer
  "fbstp tbyte ptr[esi + 10] \n" &      --store it
  "end: " &
  "finit \n" &                          --reset condition flags
  "popad \n" &
  "ret \n"
sequence code
atom convert, data, over
code = get_asm(convert_code)
convert = allocate(length(code))
poke(convert, code)             --assemble convert code
data = allocate(67)                         --allocate data seg.
resolve_param(convert, "data",  data)                --resolve data reference
poke(data + 38, {#7F, #0F})                 --poke the control word
poke(data + 40, atom_to_float64(10))        --poke constant 10
over = 3                                    --set extra precision overflow
poke(data + 59, atom_to_float64(power(10, over - 1))) --over scaling factor
--BCD to String converts a 10 byte (20 digit) BCD number to a readable string
function bcd_to_string(sequence in)
  sequence out, sel
  integer lead, val
  out = {}  lead = 1
  for nibble = 19 to 1 by -1 do                     --scan thru nibbles
    if remainder(nibble, 2) then sel = {15, 1}      --low nibble
    else sel = {240, .0625} end if                  --high nibble                       
    val = and_bits(in[floor((nibble - 1) / 2) + 1], sel[1]) * sel[2]
    if (lead and val) or nibble = 1 then lead = 0 end if --remove lead 0s
    if not lead then out &= val end if              --glue it together
  end for
  out = {(and_bits(in[10], 128) != 0) * 2 + 43} & out + 48
  return out
end function  
--Float to String takes a floating point number memory image and makes a
-- string numeral from it with maximum possible precision.  Floating point
-- numbers can be 4 byte (float32), 8 byte (float64) or 10 byte (80 bit 
-- with no Euphoria representation).  String numerals are always in 
-- scientific notation (mantissa * 10^exponent) with the signs of both
-- mantissa and exponent explicitly represented.  Out of range numbers
-- should return the correct FPU condition code. 
global function float_to_string(sequence in)
  sequence bcd
  integer l10
  atom adjust
  poke(data + 48, length(in))                       --poke length of number
  l10 = 0
  if length(in) = 4 then adjust = power(10, 7)      --4 byte adjust
  elsif length(in) = 8 then adjust = power(10, 15)  --8 byte adjust
  elsif length(in) = 10 then 
    adjust = power(10, 18)                          --10 byte adjust + over!
    l10 = 1                                         --10 byte number flag
  else
    puts(1, "\nFatal Error : Floating point number byte size incorrect in float_to_string()\n")
    abort(1)
  end if
  poke(data + 30, atom_to_float64(adjust))          --poke adjust
  poke(data, in)                                    --poke the actual number
  call(convert)                                     --and convert it!
  if peek(data + 48) = 0 then
    if peek(data + 1) = 0 then return "+unnormalized"
    elsif peek(data + 1) = 2 then return "-unnormalized"
    elsif peek(data + 1) = 64 then return "+0"
    elsif peek(data + 1) = 66 then return "-0"
    elsif peek(data + 1) = 68 then return "+denormalized"
    elsif peek(data + 1) = 70 then return "-denormalized"
    elsif peek(data + 1) = 1 then return "+NaN"
    elsif peek(data + 1) = 3 then return "-NaN"
    elsif peek(data + 1) = 5 then return "+infinity"
    elsif peek(data + 1) = 7 then return "-infinity"
    else return "Empty?!" end if
  end if  
  bcd = {peek({data + 10, 10}), peek({data + 20, 10})} --read mant, exp
  if l10 then bcd &= {peek({data + 49, 10})} end if --read overflow
  for index = 1 to length(bcd) do                   --convert bcd numbers
    bcd[index] = bcd_to_string(bcd[index])
  end for  
  if l10 then                                       --80 bit, use overflow
    return bcd[3][1..2] & "." & bcd[3][3..over + 1] & --construct 80 bit num
           bcd[1][2..length(bcd[1])] & "e" & bcd[2][1] & 
           repeat('0', 5 - length(bcd[2])) & bcd[2][2..length(bcd[2])]
  else
    return bcd[1][1..2] & "." & bcd[1][3..length(bcd[1])] & "e" & bcd[2][1] & 
           repeat('0', 3 + (length(in) = 8) - length(bcd[2])) & 
           bcd[2][2..length(bcd[2])]
  end if
end function

sequence float32, float64, float80, sci, pi64, pi80
atom float
--Testing..
if 0 then
  pi64 = {24,45,68,84,251,33,9,64}
  pi80 = {53,194,104,33,162,218,15,201,0,64} 
  float64 = pi64
  float80 = pi80
  float = float64_to_atom(float64)
  float32 = atom_to_float32(float)
  float64 = atom_to_float64(float)
  printf(1, "%1.15f\n",float) ?float32 ?float64 ?float80
  puts(1, "-- 32 Bit -------------------------------------------------------\n")
  sci = float_to_string(float32)
  puts(1, sci & "\n")
  puts(1, "-- 64 Bit -------------------------------------------------------\n")
  sci = float_to_string(float64)
  puts(1, sci & "\n")
  puts(1, "-- 80 Bit -------------------------------------------------------\n")
  sci = float_to_string(float80)
  puts(1, sci & "\n")
end if

--the low-level memory manipulators

global procedure poke_byte(atom a, object b) poke(a, b) end procedure

global function peek_byte(atom a) return peek(a) end function

global procedure poke_word(atom a, object b) 
  if atom(b) then b = {b} end if
  for count = 1 to length(b) do
    poke(a, remainder(b[count], 256)) 
    poke(a + 1, floor(b[count] / 256)) 
  end for
end procedure

global function peek_word(atom a) return peek(a) + peek(a + 1) * 256 end function

global procedure poke_dword(atom a, object b) poke4(a, b) end procedure

global function peek_dword(atom a) return peek4u(a) end function

global procedure poke_fixpt(atom a, object b) poke4(a, b) end procedure

global function peek_fixpt(atom a) return peek4s(a) end function

global procedure poke_float4(atom a, object b) 
  if atom(b) then b = {b} end if
  for count = 1 to length(b) do
    poke(a + (count - 1) * 8, atom_to_float32(b[count]))
  end for
end procedure

global function peek_float4(atom a) return float32_to_atom(peek({a, 8})) end function

global procedure poke_float8(atom a, object b) 
  if atom(b) then b = {b} end if
  for count = 1 to length(b) do
    poke(a + (count - 1) * 8, atom_to_float64(b[count]))
  end for
end procedure

global function peek_float8(atom a) return float64_to_atom(peek({a, 8})) end function

--global procedure poke_float10(atom a, atom b) poke(a, atom_to_float64(b)) end procedure

--global function peek_float10(atom a) return float64_to_atom(peek({a, 8})) end function

global procedure poke_real(atom a, object b) poke_float8(a,b) end procedure -----------

global function peek_real(atom a) return peek_float8(a) end function ------wrong!----

global procedure poke_complex(atom a, object b) poke_float8(a,b[1]) poke_float8(a+8,b[2]) end procedure -----------

global function peek_complex(atom a) return {peek_float8(a),peek_float8(a+8)} end function ------wrong!----



